c     This subroutine computes ALL the virtual corrections along the 
c     quark line depicted below and returns it as a Lorentz vector. Only 
c     TWO contractions with external currents (among jmu1, jmu2, jmu3) are done
c     
c
c      q,jmu1            t,jmu3
c        |                 |
c       \|/               \|/
c        |                 |
c -->-------------------------->---
c   p            |             p'
c               /|\
c                |
c              l,jmu2
c
c     p^2 = p'^2 = 0
c     
c     The notation for the external momenta is
c     
c     p + q + l + t = p' = pp
c     musq = mu^2 = common external scale (renormalization scale, eventually)
c     
c  
c     Only TWO external currents are passed to the subroutine
c     
c     Carlo Oleari    13/6/2005
c
      subroutine J_virtual_tri_box_pent(
c     bra(p'), ket(p) and helicity (of the ket fermion)
     #     brapp,ketp,hel,
c     momenta and squared renormalization scale
     #     p,q,l,t,pp,musq,
c     input currents (any couple among [J1,J2,J3]). ORDER IS IMPORTANT!!
c     They can be  jmu1,jmu2  or  jmu1,jmu3   or   jmu2,jmu3
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3
     #     num_ret_current,
c     returned current
     #     Jret)

      implicit none
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      real * 8 musq
      complex * 16 jal(0:3),jbe(0:3)
      integer num_ret_current
      complex * 16 jret(0:3)

      complex*16 jmu1(0:3),jmu2(0:3),jmu3(0:3)
      real * 8 psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, ppp, ppq, ppl
      real * 8 pq_sq,pql_sq,pqlt_sq,ql_sq,qlt_sq,lt_sq
      complex * 16 D04m_fin,D03m_fin,D02m_fin,C0fin,B0fin
      complex*16 E0
      complex * 16 B0_12,B0_13,B0_14,B0_15,B0_23,B0_24,B0_25,B0_34,
     #     B0_35,B0_45
      complex * 16 C0_345,C0_245,C0_235,C0_234,C0_145,C0_135,C0_134,
     #     C0_125,C0_124,C0_123
      complex * 16 Cij_345(2,4),Cij_245(2,4),Cij_235(2,4),Cij_234(2,4),
     #     Cij_145(2,4),Cij_135(2,4),Cij_134(2,4),Cij_125(2,4),
     #     Cij_124(2,4),Cij_123(2,4)
      complex * 16 D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
      complex * 16 Dij_2345(3,13),Dij_1345(3,13),Dij_1245(3,13),
     #     Dij_1235(3,13),Dij_1234(3,13)
      complex * 16 E0_12345, Eij_12345(4,46)
      common/E_functions/E0_12345, Eij_12345
      common/D_functions/D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234
      common/C_functions/
     #     C0_345,C0_245,C0_235,C0_234,C0_145,
     #     C0_135,C0_134,C0_125,C0_124,C0_123,
     #     Cij_345,Cij_245,Cij_235,Cij_234,Cij_145,
     #     Cij_135,Cij_134,Cij_125,Cij_124,Cij_123
      common/B_functions/B0_13,B0_14
      integer compon
      complex * 16 tri_box,virt_tri_box,pent,virt_pent

      logical debug
c      parameter (debug=.true.)
      parameter (debug=.false.)

      complex * 16 jref(0:3,0:3)
      integer mu
      real * 8 dotp

      psq = 0d0                 ! dotp(p,p)
      qsq = dotp(q,q)
      lsq = dotp(l,l)
      tsq = dotp(t,t)

      pq = dotp(p,q)
      pl = dotp(p,l)
      ql = dotp(q,l)
      qt = dotp(q,t)
      lt = dotp(l,t)

      pt = dotp(p,t)

      ppp = dotp(p,pp)
      ppq = dotp(pp,q)
      ppl = dotp(pp,l)

c (p+q)^2
      pq_sq = qsq+2*pq+psq
c (p+q+l)^2
      pql_sq = lsq+2*ql+2*pl+qsq+2*pq+psq
c (p+q+l+t)^2
      pqlt_sq =  0d0 ! tsq+2*lt+2*qt+2*pt+lsq+2*ql+2*pl+qsq+2*pq+psq
c (q+l)^2
      ql_sq = lsq+2*ql+qsq  
c (q+l+t)^2
      qlt_sq = tsq+2*lt+2*qt+lsq+2*ql+qsq
c (l+t)^2
      lt_sq = tsq+2*lt+lsq

      if (debug) then
         write(*,*) 'more prod scal'
         write(*,*) pq_sq,pql_sq,pqlt_sq,ql_sq,qlt_sq,lt_sq
      endif

c B0(p)
      B0_12 = 0d0 ! B0fin(psq,musq)
c B0(p+q)
      B0_13 = B0fin(pq_sq,musq)
c B0(p+q+l)
      B0_14 = B0fin(pql_sq,musq)
c B0(p+q+l+t)
      B0_15 = B0fin(pqlt_sq,musq)
c B0(q)
      B0_23 = B0fin(qsq,musq)
c B0(q+l)
      B0_24 = B0fin(ql_sq,musq)
c B0(q+l+t)
      B0_25 = B0fin(qlt_sq,musq)
c B0(l)
      B0_34 = B0fin(lsq,musq)
c B0(l+t)
      B0_35 = B0fin(lt_sq,musq)
c B0(t)
      B0_45 = B0fin(tsq,musq)

c C0(l,t)
      C0_345 = C0fin(lsq,tsq,lt_sq,musq)
c C0(q+l,t)
      C0_245 = C0fin(ql_sq,tsq,qlt_sq,musq)
c C0(q,l+t)
      C0_235 = C0fin(qsq,lt_sq,qlt_sq,musq)
c C0(q,l)
      C0_234 = C0fin(qsq,lsq,ql_sq,musq)
c C0(p+q+l,t)
      C0_145 = C0fin(pql_sq,tsq,pqlt_sq,musq)
c C0(p+q,l+t)
      C0_135 = C0fin(pq_sq,lt_sq,pqlt_sq,musq)
c C0(p+q,l)
      C0_134 = C0fin(pq_sq,lsq,pql_sq,musq)
c C0(p,q+l+t)
      C0_125 = C0fin(psq,qlt_sq,pqlt_sq,musq)
c C0(p,q+l)
      C0_124 = C0fin(psq,ql_sq,pql_sq,musq)
c C0(p,q)
      C0_123 = C0fin(psq,qsq,pq_sq,musq)

c Cij_345
      call tens_red3(lsq,tsq,lt_sq, B0_45, B0_35, B0_34, 
     #     C0_345, Cij_345)
c Cij_245
      call tens_red3(ql_sq,tsq,qlt_sq, B0_45, B0_25, B0_24, 
     #     C0_245, Cij_245)
c Cij_235
      call tens_red3(qsq,lt_sq,qlt_sq, B0_35, B0_25, B0_23, 
     #     C0_235, Cij_235)
c Cij_234
      call tens_red3(qsq,lsq,ql_sq, B0_34, B0_24, B0_23, 
     #     C0_234, Cij_234)
c Cij_145
      call tens_red3(pql_sq,tsq,pqlt_sq, B0_45, B0_15, B0_14, 
     #     C0_145, Cij_145)
c Cij_135
      call tens_red3(pq_sq,lt_sq,pqlt_sq, B0_35, B0_15, B0_13, 
     #     C0_135, Cij_135)
c Cij_134
      call tens_red3(pq_sq,lsq,pql_sq, B0_34, B0_14, B0_13, 
     #     C0_134, Cij_134)
c Cij_125
      call tens_red3(psq,qlt_sq,pqlt_sq, B0_25, B0_15, B0_12, 
     #     C0_125, Cij_125)
c Cij_124
      call tens_red3(psq,ql_sq,pql_sq, B0_24, B0_14, B0_12, 
     #     C0_124, Cij_124)
c Cij_123
      call tens_red3(psq,qsq,pq_sq, B0_23, B0_13, B0_12, 
     #     C0_123, Cij_123)

c      call print_Cij(C0_123,Cij_123)
c      call print_Cij(C0_245,Cij_245)

c D0_2345 = D00(q,l,t)
c      D0_2345 = D0(qsq,lsq,tsq,ql,qt,lt)
c the following two expressions are equivalent
      D0_2345 = D04m_fin(lt_sq,ql_sq,qlt_sq,qsq,lsq,tsq,musq)
c      write(*,*) 'D0_2345',D0_2345
c      D0_2345 = D04m_fin(ql_sq,lt_sq,qsq,lsq,tsq,qlt_sq,musq)
c      write(*,*) 'D0_2345',D0_2345
      
c D0_1345 = D00(p+q,l,t)         
c      D0_1345 = D0(pq_sq,lsq,tsq,ql+pl,qt+pt,lt)
      D0_1345 = D03m_fin(lt_sq,pql_sq,pq_sq,lsq,tsq,musq)
c      write(*,*) 'D0_1345',D0_1345

c D0_1245 = D00(p,q+l,t) 
c      D0_1245 = D0(psq,ql_sq,tsq,pl+pq,pt,lt+qt)
      D0_1245 = D02m_fin(qlt_sq,pql_sq,ql_sq,tsq,musq)
c      write(*,*) 'D0_1245',D0_1245

c D0_1235 = D00(p,q,l+t)  
c      D0_1235 = D0(psq,qsq,lt_sq,pq,pt+pl,qt+ql)
      D0_1235 = D02m_fin(qlt_sq,pq_sq,qsq,lt_sq,musq)
c      write(*,*) 'D0_1235',D0_1235

c D0_1234 = D00(p,q,l)
c      D0_1234 = D0(psq,qsq,lsq,pq,pl,ql)
      D0_1234 = D03m_fin(ql_sq,pq_sq,pql_sq,lsq,qsq,musq)
c      write(*,*) 'D0_1234',D0_1234

c Dij_2345
      call tens_red4(qsq,lsq,tsq,ql,qt,lt,
     #     C0_345, C0_245, C0_235, C0_234,   
     #     Cij_345, Cij_245, Cij_235, Cij_234,   
     #     D0_2345, 
     #     Dij_2345)

c Dij_1345
      call tens_red4(pq_sq,lsq,tsq,ql+pl,qt+pt,lt,
     #     C0_345, C0_145, C0_135, C0_134,   
     #     Cij_345, Cij_145, Cij_135, Cij_134,   
     #     D0_1345, 
     #     Dij_1345)

c Dij_1245
      call tens_red4(psq,ql_sq,tsq,pl+pq,pt,lt+qt,
     #     C0_245, C0_145, C0_125, C0_124,   
     #     Cij_245, Cij_145, Cij_125, Cij_124,   
     #     D0_1245, 
     #     Dij_1245)

c Dij_1235
      call tens_red4(psq,qsq,lt_sq,pq,pt+pl,qt+ql,
     #     C0_235, C0_135, C0_125, C0_123,   
     #     Cij_235, Cij_135, Cij_125, Cij_123,   
     #     D0_1235, 
     #     Dij_1235)

c Dij_1234
      call tens_red4(psq,qsq,lsq,pq,pl,ql,
     #     C0_234, C0_134, C0_124, C0_123,   
     #     Cij_234, Cij_134, Cij_124, Cij_123,   
     #     D0_1234, 
     #     Dij_1234)

c      call print_Dij(D0_1234,Dij_1234)
c      call print_Dij(D0_2345,Dij_2345)      
c      call print_Dij(D0_1345,Dij_1345)      
c      call print_Dij(D0_1245,Dij_1245)      
c      call print_Dij(D0_1235,Dij_1235)      

c E0_12345 = E0(p,q,l,t)
      E0_12345 = E0(psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, 
     #     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234)
c      write(*,*) 'E0',E0_12345

c Eij_12345
      call tens_red5(psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, 
     #     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #     E0_12345, 
     #     Eij_12345)

      if (debug) then
         write(*,*) 'E0_12345' 
         write(*,*) E0_12345
         write(*,*) 'D0_2345,D0_1345,D0_1245,D0_1235,D0_1234'
         write(*,*) D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
         write(*,*) 'C0_345,C0_245,C0_235,C0_234,C0_145,'//
     #        'C0_135,C0_134,C0_125,C0_124,C0_123'
         write(*,*) C0_345,C0_245,C0_235,C0_234,C0_145,
     #        C0_135,C0_134,C0_125,C0_124,C0_123
         write(*,*) 'B0_12,B0_13,B0_14,B0_15,B0_23,B0_24,B0_25,B0_34,'//
     #        'B0_35,B0_45'
         write(*,*) B0_12,B0_13,B0_14,B0_15,B0_23,B0_24,B0_25,B0_34,
     #        B0_35,B0_45
      endif
      
c      call print_Cij(C0_123,Cij_123)
c      call print_Dij(D0_2345,Dij_2345)
c      call print_Eij(E0_12345,Eij_12345)            
c      write(*,*) qsq,lsq,pq,pl,ppp,ppq,ppl,ql
      
      jref(0,0)=1d0 
      jref(0,1)=0d0
      jref(0,2)=0d0
      jref(0,3)=0d0
      
      jref(1,0)=0d0
      jref(1,1)=-1d0
      jref(1,2)=0d0
      jref(1,3)=0d0
      
      jref(2,0)=0d0
      jref(2,1)=0d0
      jref(2,2)=-1d0
      jref(2,3)=0d0

      jref(3,0)=0d0
      jref(3,1)=0d0
      jref(3,2)=0d0
      jref(3,3)=-1d0
      
      if  (num_ret_current.eq.1) then
         do mu=0,3
            jmu2(mu)=jal(mu)
            jmu3(mu)=jbe(mu)
         enddo
         do compon=0,3
            do mu=0,3
               jmu1(mu) = jref(compon,mu)
            enddo
            call compute_Jprod_gammaprod(
     #           brapp,ketp,hel,
     #           p,q,l,t,pp,
     #           jmu1,jmu2,jmu3)
            tri_box = virt_tri_box(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            pent    =    virt_pent(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            Jret(compon) = tri_box + pent
         enddo
      elseif  (num_ret_current.eq.2) then
         do mu=0,3
            jmu1(mu)=jal(mu)
            jmu3(mu)=jbe(mu)
         enddo
         do compon=0,3
            do mu=0,3
               jmu2(mu) = jref(compon,mu)
            enddo
            call compute_Jprod_gammaprod(
     #           brapp,ketp,hel,
     #           p,q,l,t,pp,
     #           jmu1,jmu2,jmu3)
            tri_box = virt_tri_box(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            pent    =    virt_pent(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            Jret(compon) = tri_box + pent
         enddo
      elseif  (num_ret_current.eq.3) then
         do mu=0,3
            jmu1(mu)=jal(mu)
            jmu2(mu)=jbe(mu)
         enddo
         do compon=0,3
            do mu=0,3
               jmu3(mu) = jref(compon,mu)
            enddo
            call compute_Jprod_gammaprod(
     #           brapp,ketp,hel,
     #           p,q,l,t,pp,
     #           jmu1,jmu2,jmu3)
            tri_box = virt_tri_box(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            pent    =    virt_pent(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            Jret(compon) = tri_box + pent
         enddo
      else
         write(*,*) 'WRONG returned current',num_ret_current
         stop
      endif
      end



c Same as J_virtual_tri_box_pent, but this subroutine 
c returns the interpolated current across the 
c unstable region, where the momenta become planar
c
c  Carlo Oleari    13/6/2005
c
      subroutine interp_J_virtual_tri_box_pent(
c     bra(p'), ket(p) and helicity of the ket
     #     brapp,ketp,hel,
c     bra ferm_type  and ket ferm_type
     #     bra_ferm_type,ket_ferm_type,
c     momenta and squared renormalization scale
     #     p,q,l,t,pp,musq,
c     input currents (any couple among [J1,J2,J3]). ORDER IS IMPORTANT!!
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3
     #     num_ret_current,
c     returned current
     #     Jret)

      implicit none
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      integer bra_ferm_type,ket_ferm_type
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      real * 8 musq
      complex * 16 jal(0:3),jbe(0:3)
      integer num_ret_current
      complex * 16 jret(0:3)

      complex * 16 brapp_new(2,-1:1),ketp_new(2,-1:1)
      complex * 16 jretvirt(0:3,3),rJret(0:3),iJret(0:3)
      real * 8 pout(0:3),ppout(0:3),qout(0:3),lout(0:3),tout(0:3)
      real * 8 ptmp(0:3),pptmp(0:3),qtmp(0:3),ltmp(0:3),ttmp(0:3)
      real * 8 azimuth_vec_y_yz
      integer n,al,mu
      real * 8 beta(1:3),phi_qq3,th_qq3,phi_p      
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      real * 8 q_new(0:3),l_new(0:3),t_new(0:3),zero(0:3)
      real * 8 interp_parab
      integer casus
      real * 8 mod_qqaux1,th_x,phi_y_yz,th_x_new(3)
      real * 8 qqaux(0:3,3),qqaux_new(0:3,3)       
      real * 8 angle_th_x,angle_th_x_step,angle_th_x_increm
      logical is_planar, planar_cond

      logical debug
      parameter (debug = .false.)
c      parameter (debug = .true.)

      logical verbose
      parameter (verbose = .false.)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCC        ADJUSTABLE PARAMETERS                CCCCCCCCCCCCCC
C     incremental step in the search for "good" angles to be used
c     during the interpolation procedure
      angle_th_x_step = 0.02d0
c     once an angle is found, find the nex one by moving by a quantity 
c     equal to:
      angle_th_x_increm = 0.08d0      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c     go to a planar configuration 
      call to_planar(p,q,l,t,pp,bra_ferm_type,ket_ferm_type,
     #     pout,qout,lout,tout,ppout,beta,phi_qq3,th_qq3,phi_p,
     #     casus,qqaux,brapp_new,ketp_new)

      if (debug) then
         call from_planar(pout,qout,lout,tout,ppout,
     #        beta,phi_qq3,th_qq3,phi_p,
     #        ptmp,qtmp,ltmp,ttmp,pptmp)

         write(*,*) 'momenta check'
         write(*,*) 'first line:  in momenta'
         write(*,*) 'second line: back from planar momenta'
         write(*,*) 'third line:  planar momenta'
         call write_mom(p)
         call write_mom(ptmp)
         call write_mom(pout)
         write(*,*)
         call write_mom(pp)
         call write_mom(pptmp)
         call write_mom(ppout)
         write(*,*)
         call write_mom(q)
         call write_mom(qtmp)
         call write_mom(qout)
         write(*,*)
         call write_mom(l)
         call write_mom(ltmp)
         call write_mom(lout)
         write(*,*)
         call write_mom(t)
         call write_mom(ttmp)
         call write_mom(tout)
      endif

c     The first two vectors of qqaux are timelike
c      
c     Compute the angle formed by the first timelike vector
c     with respect to the x axis      
      mod_qqaux1 = sqrt(qqaux(1,1)**2+qqaux(2,1)**2+qqaux(3,1)**2)
      th_x = acos(qqaux(1,1)/mod_qqaux1)
      
c     Azimuthal angle of qqaux(0,1) in the yz plane with respect to the y axis
      phi_y_yz = azimuth_vec_y_yz(qqaux(0,1))

      if (debug) then
         write(*,*) 'check angles'
         write(*,*) '1 =>',mod_qqaux1*cos(th_x)/qqaux(1,1)
         write(*,*) '1 =>',mod_qqaux1*sin(th_x)*cos(phi_y_yz)/qqaux(2,1)
         write(*,*) '1 =>',mod_qqaux1*sin(th_x)*sin(phi_y_yz)/qqaux(3,1)
      endif      

      planar_cond = is_planar(brapp_new,ketp_new,hel,
     #     pout,qout,lout,tout,ppout,musq)
c
c
      if (planar_cond) then
c     the event is close to planar ==> compute 3 angles for 
c     the interpolation
         if (verbose) then
            write(*,*) '***********  INTERPOLATING... **************'
         endif
         
c     the spacelike vector will NOT be changed
         do al=0,3
            qqaux_new(al,3) = qqaux(al,3)
         enddo
         
         if (th_x.lt.pi/2) then
            angle_th_x = th_x
            do while (planar_cond.and.(angle_th_x.gt.0d0))
c     go to smaller and smaller angle_th_x to find good interpolating points
               angle_th_x = angle_th_x - angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
c
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.le.0d0) then
               if (verbose) then
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found an angle outside the "critical region"
               th_x_new(2) = angle_th_x
            endif
c
c     to find a second angle, decrese th_x_new(2) by a fixed amount
            th_x_new(1) = th_x_new(2) - angle_th_x_increm
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  put here condition for th_x_new(1)< 0
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c            
c     now find a third angle, across the critical region
            angle_th_x = th_x
            planar_cond = .true.
            do while (planar_cond.and.(angle_th_x.lt.pi))
c     go to bigger and bigger angle_th_x to find good interpolating points
               angle_th_x = angle_th_x + angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
               
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.gt.pi) then
               if (verbose) then
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found an angle across the "critical region"
               th_x_new(3) = angle_th_x
            endif            
         else
c the initial angle th_x is bigger than pi/2
            angle_th_x = th_x
            do while (planar_cond.and.(angle_th_x.gt.0d0))
c     go to smaller and smaller angle_th_x to find good interpolating points
               angle_th_x = angle_th_x - angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.le.0d0) then
               if (verbose) then 
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found an angle across the "critical region"
               th_x_new(1) = angle_th_x
            endif
c     now find a second angle, bigger than pi/2
            angle_th_x = th_x
            planar_cond = .true.
            do while (planar_cond.and.(angle_th_x.lt.pi))
c     go to bigger and bigger angle_th_x to find good interpolating points
               angle_th_x = angle_th_x + angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.gt.pi) then
               if (verbose) then
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found the second angle
               th_x_new(2) = angle_th_x
            endif
c
c     to find a third angle, increse th_x_new(2) by a fixed amount
            th_x_new(3) = th_x_new(2) + angle_th_x_increm
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  put here condition for th_x_new(3)> pi
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c            
         endif
c     if we arrive here, than the subroutine has found 3 angles to
c     be used for the interpolation
         if (verbose) then
            write(*,*) 'INTERPOLATION SUCCESSFUL'
         endif
c     
         do n=1,3
            qqaux_new(0,1) = qqaux(0,1)
            qqaux_new(1,1) = mod_qqaux1*cos(th_x_new(n))
            qqaux_new(2,1) = mod_qqaux1*sin(th_x_new(n))*cos(phi_y_yz)
            qqaux_new(3,1) = mod_qqaux1*sin(th_x_new(n))*sin(phi_y_yz)
c     
            qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
            qqaux_new(1,2) = -qqaux_new(1,1)
            qqaux_new(2,2) = -qqaux_new(2,1)
            qqaux_new(3,2) = -qqaux_new(3,1)
c                       
            if (debug) then
               write(*,*) 'interpolation vector number',n
               write(*,*) 'original'
               call write_mom(qqaux(0,1))
               write(*,*) 'slightly moved'
               call write_mom(qqaux_new(0,1))
               write(*,*) 'original'
               call write_mom(qqaux(0,2))
               write(*,*) 'slightly moved'
               call write_mom(qqaux_new(0,2))
               write(*,*)
            endif            
c
c     re-assign the new momenta to the correct order         
            call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
c     now go back to NON planar configuration            
            call from_planar(pout,q_new,l_new,t_new,ppout,
     #           beta,phi_qq3,th_qq3,phi_p,
     #           ptmp,qtmp,ltmp,ttmp,pptmp)
c            
            if (debug) then
               write(*,*) 'NUMBER',n
               write(*,*) 'first line: old momenta'
               write(*,*) 'second line: new momenta'
               write(*,*) 'p'
               call write_mom(pout)
               call write_mom(ptmp)
               write(*,*) 'pp'
               call write_mom(ppout)
               call write_mom(pptmp)
               write(*,*) 'q'
               call write_mom(q_new)
               call write_mom(qtmp)
               write(*,*) 'l'
               call write_mom(l_new)
               call write_mom(ltmp)
               write(*,*) 't'
               call write_mom(t_new)
               call write_mom(ttmp)
               do al=0,3
                  zero(al) = 
     #                 ptmp(al)+qtmp(al)+ltmp(al)+ttmp(al)-pptmp(al)
               enddo
               write(*,*) 'p+q+l+t-pp'
               call write_mom(zero)
            endif
c     
            call J_virtual_tri_box_pent(
     #           brapp,ketp,hel,
     #           ptmp,qtmp,ltmp,ttmp,pptmp,musq,
     #           jal,jbe,
     #           num_ret_current,
     #           Jretvirt(0,n))                           
         enddo   
c         
         do mu=0,3         
            rJret(mu) = interp_parab(
     #           th_x_new(1),dreal(Jretvirt(mu,1)),
     #           th_x_new(2),dreal(Jretvirt(mu,2)),
     #           th_x_new(3),dreal(Jretvirt(mu,3)),
     #           th_x)

            iJret(mu) = interp_parab(
     #           th_x_new(1),dimag(Jretvirt(mu,1)),
     #           th_x_new(2),dimag(Jretvirt(mu,2)),
     #           th_x_new(3),dimag(Jretvirt(mu,3)),
     #           th_x)
         enddo
c
         do mu=0,3
            Jret(mu) = rJret(mu)+(0,1)*iJret(mu)
         enddo
      else
c     the event is NOT planar
         call J_virtual_tri_box_pent(
     #     brapp,ketp,hel,
     #     p,q,l,t,pp,musq,
     #     jal,jbe,
     #     num_ret_current,
     #     Jret)
      endif      
      end



c     Compute J.p_i, where J's are the external currents, and all the 
c     <bra|...|ket> gamma prods
c     The returned values are in the common blocks
c
c  Carlo Oleari    13/6/2005
c
      subroutine compute_Jprod_gammaprod(
c     bra(p'), ket(p) and helicity 
     #     bra,ket,hel,
c     momenta
     #     p,q,l,t,pp,
c     input currents
     #     jmu1,jmu2,jmu3)
      implicit none
      complex * 16 bra(2,-1:1),ket(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      complex*16 jmu1(0:3),jmu2(0:3),jmu3(0:3)

      complex * 16 cq(0:3),cl(0:3)
      complex * 16 dpmu1,dpmu2,dpmu3,dqmu1,dqmu2,dqmu3
      complex * 16 dlmu1,dlmu2,dlmu3,dppmu1,dppmu2,dppmu3
      complex * 16 dmu1mu2,dmu1mu3,dmu2mu3
      common/dotp_pol_vec/dpmu1,dpmu2,dpmu3,dqmu1,dqmu2,dqmu3,
     #     dlmu1,dlmu2,dlmu3,dppmu1,dppmu2,dppmu3,
     #     dmu1mu2,dmu1mu3,dmu2mu3
      complex * 16 gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q
      complex * 16 gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2
      complex * 16 gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3
      complex * 16 gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
      common/gammaprods/gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q,
     #     gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2,
     #     gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3,
     #     gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
c      
      complex * 16 local_mom(0:3,5), cdotp, ccdotp
      integer al
      
      logical debug
c      parameter (debug=.true.)
      parameter (debug=.false.)

      dpmu1 = cdotp(p,jmu1)
      dpmu2 = cdotp(p,jmu2)
      dpmu3 = cdotp(p,jmu3)

      dqmu1 = cdotp(q,jmu1)
      dqmu2 = cdotp(q,jmu2)
      dqmu3 = cdotp(q,jmu3)

      dlmu1 = cdotp(l,jmu1)
      dlmu2 = cdotp(l,jmu2)
      dlmu3 = cdotp(l,jmu3)

      dppmu1 = cdotp(pp,jmu1)
      dppmu2 = cdotp(pp,jmu2)
      dppmu3 = cdotp(pp,jmu3)

      dmu1mu2 = ccdotp(jmu1,jmu2)
      dmu1mu3 = ccdotp(jmu1,jmu3)
      dmu2mu3 = ccdotp(jmu2,jmu3)
      
      do al=0,3
         cl(al) = l(al)
         cq(al) = q(al)
      enddo

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = jmu2(al)
         local_mom(al,4) = cq(al)
         local_mom(al,5) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,5,gp_mu3_l_mu2_q_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_l_q)

      do al=0,3
         local_mom(al,1) = jmu2(al)
         local_mom(al,2) = cq(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu2_q_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_mu2_q)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = jmu2(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_l_mu2)

      do al=0,3
         local_mom(al,1) = cl(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_l_mu2_q)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_mu2_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_l_mu1)

      do al=0,3
         local_mom(al,1) = cl(al)
         local_mom(al,2) = cq(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_l_q_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cq(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_q_mu1)

      do al=0,3
         local_mom(al,1) = cl(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_l_mu2_mu1)

      do al=0,3
         local_mom(al,1) = cl(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_l)

      do al=0,3
         local_mom(al,1) = jmu2(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_mu2)

      do al=0,3
         local_mom(al,1) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_q)

      do al=0,3
         local_mom(al,1) = jmu3(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_mu3)

      do al=0,3
         local_mom(al,1) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_mu1)

      if (debug) then
         write(*,*) 'gamma prods'
         write(*,*) gp_l,gp_q,gp_mu2,gp_mu3,gp_mu3_l_mu2,gp_mu3_l_q,
     #        gp_mu3_mu2_q,gp_mu3_l_mu2,gp_l_mu2_q
         
         write(*,*) 
         write(*,*) gp_mu3_l_mu2_q_mu1,gp_mu2_q_mu1,gp_mu3_mu2_mu1,
     #        gp_mu3_l_mu1,gp_l_q_mu1,gp_l_mu2_mu1,gp_mu3_q_mu1,gp_mu1
      endif

      if (debug) then
         write(*,*) gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q,
     #        gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2,
     #        gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3,
     #        gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
      endif
      end



c     Given casus and qqaux, the subroutine returns the 
c     right ordered q, l and t
c     See subroutine for details
c
c     Carlo Oleari    13/6/2005
c
      subroutine from_qqaux_to_qlt(casus,qqaux,q,l,t)
      implicit none
      integer casus
      real * 8 qqaux(0:3,3)
      real * 8 q(0:3),l(0:3),t(0:3)
      integer al

      if (casus.eq.1) then
         do al=0,3
            q(al) = qqaux(al,1)
            l(al) = qqaux(al,2)
            t(al) = qqaux(al,3)
         enddo
      elseif (casus.eq.2)then
         do al=0,3
            q(al) = qqaux(al,1)
            t(al) = qqaux(al,2)
            l(al) = qqaux(al,3)
         enddo
      elseif (casus.eq.3)then
         do al=0,3
            l(al) = qqaux(al,1)
            t(al) = qqaux(al,2)
            q(al) = qqaux(al,3)
         enddo
      endif
      end




c     This subroutine receives pin,qin,lin,tin and ppin = pin+qin+lin+tin, 
c     ("in" stands for initial) as input momenta
c     Among qin, lin and tin there are two timelike and one spacelike vector
c
c     The subroutine boosts the system in the centre of mass of the 
c     two timelike momenta.
c     Then it rotates the system so that the spacelike momentum has 
c     vector component along the z axis.
c     It rotates the system so that p belongs to the yz plane.
c     At this point, pp belongs to the yz plane too.
c
c     Carlo Oleari 13/6/2005
c
      subroutine to_planar(pin,qin,lin,tin,ppin,
c     bra ferm_type and ket ferm_type
     #     bra_ferm_type,ket_ferm_type,
c     returned momenta
     #     p,q,l,t,pp,
c     returned angles and boost vector
     #     beta,phi_qq3,th_qq3,phi_p,
c     returned case and q l t vectors ordered according their invariant mass
     #     casus, qqaux,
c     returned bra and ket for the boosted p and pp
     #     brapp_new,ketp_new)

      implicit none
      real * 8 pin(0:3),qin(0:3),lin(0:3),tin(0:3),ppin(0:3)
      integer bra_ferm_type,ket_ferm_type
      real * 8 beta(1:3),phi_qq3,th_qq3,phi_p
      real * 8 p(0:3),q(0:3),l(0:3),t(0:3),pp(0:3)
      integer casus      
      real * 8 qqaux(0:3,3)
      complex * 16 brapp_new(2,-1:1),ketp_new(2,-1:1)      

      real * 8 ptmp(0:3),pptmp(0:3)
      real * 8 paux(0:3),ppaux(0:3)
      real * 8 qq(0:3,3), qqtmp(0:3,3)
      integer i
      real * 8 dotp,qqaux_sq(1:3)
      integer al
      real * 8 azimuth_vec_y
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)

      logical debug
      parameter (debug=.false.)
c     parameter (debug=.true.)
      
      if (debug) then
         write(*,*) 'test dot prods'
         write(*,*) dotp(pin,pin),dotp(ppin,ppin)
         write(*,*) dotp(qin,qin),dotp(lin,lin),dotp(tin,tin)
         write(*,*) dotp(pin,qin),dotp(pin,lin),dotp(pin,tin),
     #        dotp(pin,ppin)
         write(*,*) dotp(qin,lin),dotp(qin,tin),dotp(qin,ppin)
         write(*,*) dotp(lin,tin),dotp(lin,ppin)
         write(*,*) dotp(tin,ppin)
      endif

      do al=0,3
         paux(al)  = pin(al)
         qqaux(al,1) = qin(al)
         qqaux(al,2) = lin(al)
         qqaux(al,3) = tin(al)
         ppaux(al) = ppin(al)
      enddo

c     find the two TIMELIKE momenta and call them qq(mu,1) and qq(mu,2)
c     Then the third SPACELIKE momentum is called qq(mu,3)
c     Three cases  for 2 timelike (T) and one spacelike (S) q l t ==>
c     1) T T S
c     2) T S T
c     3) S T T
      do i=1,3
         qqaux_sq(i) = dotp(qqaux(0,i),qqaux(0,i))
      enddo
      if ((qqaux_sq(1).gt.0d0).and.(qqaux_sq(2).gt.0d0)) then
         casus = 1
         do al=0,3
            qq(al,1) = qqaux(al,1)
            qq(al,2) = qqaux(al,2)
            qq(al,3) = qqaux(al,3)
         enddo
      elseif ((qqaux_sq(1).gt.0d0).and.(qqaux_sq(3).gt.0d0)) then
         casus = 2
         do al=0,3
            qq(al,1) = qqaux(al,1)
            qq(al,2) = qqaux(al,3)
            qq(al,3) = qqaux(al,2)
         enddo
      elseif ((qqaux_sq(2).gt.0d0).and.(qqaux_sq(3).gt.0d0)) then
         casus = 3
         do al=0,3
            qq(al,1) = qqaux(al,2)
            qq(al,2) = qqaux(al,3)
            qq(al,3) = qqaux(al,1)
         enddo
      else
         write(*,*) 'ERROR: case NOT implemented in to_planar'
         write(*,*) qqaux_sq(1),qqaux_sq(2),qqaux_sq(3)
         stop
      endif
c
c     boost in the rest frame of qq(1) + qq(2)      
      do al=1,3
         beta(al) = - (qq(al,1)+qq(al,2))/(qq(0,1)+qq(0,2))
      enddo     
      call boost(beta,paux,ptmp)
      call boost(beta,ppaux,pptmp)
      do i=1,3
         call boost(beta,qq(0,i),qqtmp(0,i))
      enddo
c
      if (debug) then
         write(*,*) 'boosted momenta in the rest frame of the'// 
     #        ' two timelike momenta'
         call write_mom(ptmp)
         call write_mom(pptmp)
         do i=1,3
            call write_mom(qqtmp(0,i))
         enddo
      endif
c      
c     determine the angles of qq3 wrt the z axis
      phi_qq3 = azimuth_vec_y(qqtmp(0,3))
      th_qq3 = acos(qqtmp(3,3)/
     #     sqrt(qqtmp(1,3)**2+qqtmp(2,3)**2+qqtmp(3,3)**2))
c      
c     now rotate the spacelike momenta to be parallel to the z axis
c     and accordingly all the other momenta      
      call rotatez(-phi_qq3,ptmp,paux)
      call rotatex(th_qq3,paux,ptmp)
      call rotatez(-phi_qq3,pptmp,ppaux)
      call rotatex(th_qq3,ppaux,pptmp)
      do i=1,3
         call rotatez(-phi_qq3,qqtmp(0,i),qqaux(0,i))
         call rotatex(th_qq3,qqaux(0,i),qqtmp(0,i))
      enddo
c      
      if (debug) then
         write(*,*) 
     #        'The spacelike momentun must be parallel to the z axis'
         call write_mom(ptmp)
         call write_mom(pptmp)
         do i=1,3
            call write_mom(qqtmp(0,i))
         enddo
      endif
c
c     determine the azimuthal angle of p wrt the yz plane
      phi_p = azimuth_vec_y(ptmp)
c      
c     now rotate p into the yz plane
c     and accordingly all the other momenta      
      call rotatez(-phi_p,ptmp,paux)
      call rotatez(-phi_p,pptmp,ppaux)
      do i=1,3
         call rotatez(-phi_p,qqtmp(0,i),qqaux(0,i))
      enddo
      if (debug) then
         write(*,*) 
     #        'The spacelike momentun must be parallel to the z axis'
         write(*,*) 'p in the yz plane'
         call write_mom(paux)
         call write_mom(ppaux)
         do i=1,3
            call write_mom(qqaux(0,i))
         enddo
      endif
c
      do al=0,3
         p(al)  = paux(al)
         pp(al) = ppaux(al)
      enddo
c
      call from_qqaux_to_qlt(casus,qqaux,q,l,t)
c     
      if (debug) then
         write(*,*) 'test dot prods'
         write(*,*) dotp(p,p),dotp(pp,pp)
         write(*,*) dotp(q,q),dotp(l,l),dotp(t,t)
         write(*,*) dotp(p,q),dotp(p,l),dotp(p,t),dotp(p,pp)
         write(*,*) dotp(q,l),dotp(q,t),dotp(q,pp)
         write(*,*) dotp(l,t),dotp(l,pp)
         write(*,*) dotp(t,pp)
      endif    
c     now compute the boosted bra and ket
      call bra(pp,bra_ferm_type,brapp_new)
      call ket(p,ket_ferm_type,ketp_new) 
      end



c     This subroutine receives pin,qin,lin,tin and ppin = pin+qin+lin+tin, 
c     ("in" stands for initial) 
c     and rotates and boosts the momenta according to the angles
c     and boost: beta,phi_qq3,th_qq3,phi_p  back to the NON planar 
c     configuration.
c     See subroutine to_planar for the meaning of these inputs
c
c     Carlo Oleari    13/6/2005
c
      subroutine from_planar(pin,qin,lin,tin,ppin,
     #     beta,phi_qq3,th_qq3,phi_p,p,q,l,t,pp)
      implicit none
      real * 8 pin(0:3),qin(0:3),lin(0:3),tin(0:3),ppin(0:3)
      real * 8 beta(1:3),phi_qq3,th_qq3,phi_p
      real * 8 p(0:3),q(0:3),l(0:3),t(0:3),pp(0:3)

      real * 8 ptmp(0:3),pptmp(0:3),qtmp(0:3),ltmp(0:3),ttmp(0:3)      
      real * 8 paux(0:3),ppaux(0:3),qaux(0:3),laux(0:3),taux(0:3)      
      real * 8 mbeta(1:3)
      real * 8 dotp
      integer al
      
      logical debug
      parameter (debug=.false.)
      
      if (debug) then
         write(*,*) 'test dot prods in from_planar_new'
         write(*,*) dotp(pin,pin),dotp(ppin,ppin)
         write(*,*) dotp(qin,qin),dotp(lin,lin),dotp(tin,tin)
         write(*,*) dotp(pin,qin),dotp(pin,lin),dotp(pin,tin),
     #        dotp(pin,ppin)
         write(*,*) dotp(qin,lin),dotp(qin,tin),dotp(qin,ppin)
         write(*,*) dotp(lin,tin),dotp(lin,ppin)
         write(*,*) dotp(tin,ppin)
      endif
      
      call rotatez(phi_p,pin,p)
      call rotatez(phi_p,ppin,pp)
      call rotatez(phi_p,qin,q)
      call rotatez(phi_p,lin,l)
      call rotatez(phi_p,tin,t)

      if (debug) then
         write(*,*) 'after rotation around z away from yz plane'
         call write_mom(p)
         call write_mom(pp)
         call write_mom(q)
         call write_mom(l)
         call write_mom(t)
      endif

      call rotatex(-th_qq3,p,paux)
      call rotatex(-th_qq3,pp,ppaux)
      call rotatex(-th_qq3,q,qaux)
      call rotatex(-th_qq3,l,laux)
      call rotatex(-th_qq3,t,taux)

      call rotatez(phi_qq3,paux,ptmp)
      call rotatez(phi_qq3,ppaux,pptmp)
      call rotatez(phi_qq3,qaux,qtmp)
      call rotatez(phi_qq3,laux,ltmp)
      call rotatez(phi_qq3,taux,ttmp)

      if (debug) then
         write(*,*) 'just before boost back...'
         call write_mom(ptmp)
         call write_mom(pptmp)
         call write_mom(qtmp)
         call write_mom(ltmp)
         call write_mom(ttmp)
      endif

      do al=1,3
         mbeta(al) = -beta(al)
      enddo
      call boost(mbeta,ptmp,p)
      call boost(mbeta,pptmp,pp)
      call boost(mbeta,qtmp,q)
      call boost(mbeta,ttmp,t)
      call boost(mbeta,ltmp,l)

      if (debug) then
         write(*,*) 'test dot prods in from_planar_new'
         write(*,*) dotp(p,p),dotp(pp,pp)
         write(*,*) dotp(q,q),dotp(l,l),dotp(t,t)
         write(*,*) dotp(p,q),dotp(p,l),dotp(p,t),dotp(p,pp)
         write(*,*) dotp(q,l),dotp(q,t),dotp(q,pp)
         write(*,*) dotp(l,t),dotp(l,pp)
         write(*,*) dotp(t,pp)
      endif

      end

      
c     It returns true if the configuration is "close to" planar,
c     according to a given definition of "planar".
c
c     Carlo Oleari    13/6/2005
c     
      function is_planar(brapp,ketp,hel,p,q,l,t,pp,musq)
      implicit none
      logical is_planar
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      real * 8 musq

      integer al
      complex * 16 virt,tree,born,ratio,ccdotp      
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      real * 8 rratio,iratio,rratio_cut,iratio_cut,onemax,onemin,
     #     zeromax,zeromin
      complex * 16 jmu1(0:3),jmu2(0:3),jmu3(0:3),jret(0:3)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC      
CCCCCCCCCCCCCCC        ADJUSTABLE PARAMETERS                CCCCCCCCCCCCC
C     How close to 1 the ratio virtual/tree in the gauge check should be
      rratio_cut = 0.01d0
      iratio_cut = 0.01d0
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC      
      
      onemax = 1d0 + rratio_cut
      onemin = 1d0 - rratio_cut
      zeromax = 0d0 + iratio_cut
      zeromin = 0d0 - iratio_cut
      
c     check the goodness of the gauge invariance
      do al=0,3
         jmu1(al) = q(al)
         jmu3(al) = t(al)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c     Set the middle current equal to (1,1,1,1)         
c     It can be any random current
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
         jmu2(al) = 1d0
      enddo

      tree = born(brapp,ketp,hel,
     #     p,q,l,t,pp,
     #     jmu1,jmu2,jmu3)      
      
      call  J_virtual_tri_box_pent(
     #     brapp,ketp,hel,
     #     p,q,l,t,pp,musq,
     #     jmu1,jmu3,
c     uncontracted external current. It can be 1, 2 or 3
     #     2,
c     returned current
     #     Jret)

      virt = ccdotp(Jret,jmu2)
      ratio = virt/tree/(pi**2/3-7)
      rratio = dreal(ratio)
      iratio = imag(ratio)

c      write(*,*) virt, ratio

      if ((rratio.gt.onemax).or.(rratio.lt.onemin).or.
     #     (iratio.gt.zeromax).or.(iratio.lt.zeromin)) then         
         is_planar = .true.
      else
         is_planar = .false.
      endif

      end


c     Returns the azimuthal angle between the vector and the y axis
c     in the xy plane
      function azimuth_vec_y(p)
      implicit none
      real * 8 azimuth_vec_y,p(0:3)
      real * 8 azimuth_vec_x,phi_x
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      phi_x = azimuth_vec_x(p)      
      if ((-pi.lt.phi_x).and.(phi_x.lt.-pi/2)) then
         azimuth_vec_y = 3*pi/2 + phi_x 
      else         
         azimuth_vec_y = phi_x - pi/2
      endif
      end


