c     This subroutine computes ALL the virtual corrections along the 
c     quark line depicted below and returns it as a Lorentz vector. Only 
c     TWO contractions with external currents (among jmu1, jmu2, jmu3) are done
c     
c
c      q,jmu1            t,jmu3
c        |                 |
c       \|/               \|/
c        |                 |
c -->-------------------------->---
c   p            |             p'
c               /|\
c                |
c              l,jmu2
c
c     p^2 = p'^2 = 0
c     
c     The notation for the external momenta is
c     
c     p + q + l + t = p' = pp
c     musq = mu^2 = common external scale (renormalization scale, eventually)
c     
c  
c     Only TWO external currents are passed to the subroutine
c     
c     Carlo Oleari    13/6/2005
c
      subroutine J_virtual_tri_box_pent(
c     bra(p'), ket(p) and helicity (of the ket fermion)
     #     brapp,ketp,hel,
c     momenta and squared renormalization scale
     #     p,q,l,t,pp,musq,
c     input currents (any couple among [J1,J2,J3]). ORDER IS IMPORTANT!!
c     They can be  jmu1,jmu2  or  jmu1,jmu3   or   jmu2,jmu3
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3
     #     num_ret_current,
c     returned current
     #     Jret)

      implicit none
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      real * 8 musq
      complex * 16 jal(0:3),jbe(0:3)
      integer num_ret_current
      complex * 16 jret(0:3)

      complex*16 jmu1(0:3),jmu2(0:3),jmu3(0:3)
      real * 8 psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, ppp, ppq, ppl
      real * 8 pq_sq,pql_sq,pqlt_sq,ql_sq,qlt_sq,lt_sq
      complex * 16 D04m_fin,D03m_fin,D02m_fin,C0fin,B0fin
      complex*16 E0
      complex * 16 B0_12,B0_13,B0_14,B0_15,B0_23,B0_24,B0_25,B0_34,
     #     B0_35,B0_45
      complex * 16 C0_345,C0_245,C0_235,C0_234,C0_145,C0_135,C0_134,
     #     C0_125,C0_124,C0_123
      complex * 16 Cij_345(2,4),Cij_245(2,4),Cij_235(2,4),Cij_234(2,4),
     #     Cij_145(2,4),Cij_135(2,4),Cij_134(2,4),Cij_125(2,4),
     #     Cij_124(2,4),Cij_123(2,4)
      complex * 16 D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
      complex * 16 Dij_2345(3,13),Dij_1345(3,13),Dij_1245(3,13),
     #     Dij_1235(3,13),Dij_1234(3,13)
      complex * 16 E0_12345, Eij_12345(4,46)
      common/E_functions/E0_12345, Eij_12345
      common/D_functions/D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234
      common/C_functions/
     #     C0_345,C0_245,C0_235,C0_234,C0_145,
     #     C0_135,C0_134,C0_125,C0_124,C0_123,
     #     Cij_345,Cij_245,Cij_235,Cij_234,Cij_145,
     #     Cij_135,Cij_134,Cij_125,Cij_124,Cij_123
      common/B_functions/B0_13,B0_14,B0_24,B0_34,B0_23,B0_45,B0_35
      integer compon
      complex * 16 tri_box,virt_tri_box,pent,virt_pent

      logical debug
c      parameter (debug=.true.)
      parameter (debug=.false.)

      complex * 16 jref(0:3,0:3)
      integer mu
      real * 8 dotrr

      psq = 0d0                 ! dotrr(p,p)
      qsq = dotrr(q,q)
      lsq = dotrr(l,l)
      tsq = dotrr(t,t)

      pq = dotrr(p,q)
      pl = dotrr(p,l)
      ql = dotrr(q,l)
      qt = dotrr(q,t)
      lt = dotrr(l,t)

      pt = dotrr(p,t)

      ppp = dotrr(p,pp)
      ppq = dotrr(pp,q)
      ppl = dotrr(pp,l)

c (p+q)^2
      pq_sq = qsq+2*pq+psq
c (p+q+l)^2
      pql_sq = lsq+2*ql+2*pl+qsq+2*pq+psq
c (p+q+l+t)^2
      pqlt_sq =  0d0 ! tsq+2*lt+2*qt+2*pt+lsq+2*ql+2*pl+qsq+2*pq+psq
c (q+l)^2
      ql_sq = lsq+2*ql+qsq  
c (q+l+t)^2
      qlt_sq = tsq+2*lt+2*qt+lsq+2*ql+qsq
c (l+t)^2
      lt_sq = tsq+2*lt+lsq

      if (debug) then
         write(*,*) 'more prod scal'
         write(*,*) pq_sq,pql_sq,pqlt_sq,ql_sq,qlt_sq,lt_sq
      endif

c B0(p)
      B0_12 = 0d0 ! B0fin(psq,musq)
c B0(p+q)
      B0_13 = B0fin(pq_sq,musq)
c B0(p+q+l)
      B0_14 = B0fin(pql_sq,musq)
c B0(p+q+l+t)
      B0_15 = B0fin(pqlt_sq,musq)
c B0(q)
      B0_23 = B0fin(qsq,musq)
c B0(q+l)
      B0_24 = B0fin(ql_sq,musq)
c B0(q+l+t)
      B0_25 = B0fin(qlt_sq,musq)
c B0(l)
      B0_34 = B0fin(lsq,musq)
c B0(l+t)
      B0_35 = B0fin(lt_sq,musq)
c B0(t)
      B0_45 = B0fin(tsq,musq)

c C0(l,t)
      C0_345 = C0fin(lsq,tsq,lt_sq,musq)
c C0(q+l,t)
      C0_245 = C0fin(ql_sq,tsq,qlt_sq,musq)
c C0(q,l+t)
      C0_235 = C0fin(qsq,lt_sq,qlt_sq,musq)
c C0(q,l)
      C0_234 = C0fin(qsq,lsq,ql_sq,musq)
c C0(p+q+l,t)
      C0_145 = C0fin(pql_sq,tsq,pqlt_sq,musq)
c C0(p+q,l+t)
      C0_135 = C0fin(pq_sq,lt_sq,pqlt_sq,musq)
c C0(p+q,l)
      C0_134 = C0fin(pq_sq,lsq,pql_sq,musq)
c C0(p,q+l+t)
      C0_125 = C0fin(psq,qlt_sq,pqlt_sq,musq)
c C0(p,q+l)
      C0_124 = C0fin(psq,ql_sq,pql_sq,musq)
c C0(p,q)
      C0_123 = C0fin(psq,qsq,pq_sq,musq)

c Cij_345
      call tens_red3(lsq,tsq,lt_sq, B0_45, B0_35, B0_34, 
     #     C0_345, Cij_345)
c Cij_245
      call tens_red3(ql_sq,tsq,qlt_sq, B0_45, B0_25, B0_24, 
     #     C0_245, Cij_245)
c Cij_235
      call tens_red3(qsq,lt_sq,qlt_sq, B0_35, B0_25, B0_23, 
     #     C0_235, Cij_235)
c Cij_234
      call tens_red3(qsq,lsq,ql_sq, B0_34, B0_24, B0_23, 
     #     C0_234, Cij_234)
c Cij_145
      call tens_red3(pql_sq,tsq,pqlt_sq, B0_45, B0_15, B0_14, 
     #     C0_145, Cij_145)
c Cij_135
      call tens_red3(pq_sq,lt_sq,pqlt_sq, B0_35, B0_15, B0_13, 
     #     C0_135, Cij_135)
c Cij_134
      call tens_red3(pq_sq,lsq,pql_sq, B0_34, B0_14, B0_13, 
     #     C0_134, Cij_134)
c Cij_125
      call tens_red3(psq,qlt_sq,pqlt_sq, B0_25, B0_15, B0_12, 
     #     C0_125, Cij_125)
c Cij_124
      call tens_red3(psq,ql_sq,pql_sq, B0_24, B0_14, B0_12, 
     #     C0_124, Cij_124)
c Cij_123
      call tens_red3(psq,qsq,pq_sq, B0_23, B0_13, B0_12, 
     #     C0_123, Cij_123)

c      call print_Cij(C0_123,Cij_123)
c      call print_Cij(C0_245,Cij_245)

c D0_2345 = D00(q,l,t)
c      D0_2345 = D0(qsq,lsq,tsq,ql,qt,lt)
c the following two expressions are equivalent
      D0_2345 = D04m_fin(lt_sq,ql_sq,qlt_sq,qsq,lsq,tsq,musq)
c      write(*,*) 'D0_2345',D0_2345
c      D0_2345 = D04m_fin(ql_sq,lt_sq,qsq,lsq,tsq,qlt_sq,musq)
c      write(*,*) 'D0_2345',D0_2345
      
c D0_1345 = D00(p+q,l,t)         
c      D0_1345 = D0(pq_sq,lsq,tsq,ql+pl,qt+pt,lt)
      D0_1345 = D03m_fin(lt_sq,pql_sq,pq_sq,lsq,tsq,musq)
c      write(*,*) 'D0_1345',D0_1345

c D0_1245 = D00(p,q+l,t) 
c      D0_1245 = D0(psq,ql_sq,tsq,pl+pq,pt,lt+qt)
      D0_1245 = D02m_fin(qlt_sq,pql_sq,ql_sq,tsq,musq)
c      write(*,*) 'D0_1245',D0_1245

c D0_1235 = D00(p,q,l+t)  
c      D0_1235 = D0(psq,qsq,lt_sq,pq,pt+pl,qt+ql)
      D0_1235 = D02m_fin(qlt_sq,pq_sq,qsq,lt_sq,musq)
c      write(*,*) 'D0_1235',D0_1235

c D0_1234 = D00(p,q,l)
c      D0_1234 = D0(psq,qsq,lsq,pq,pl,ql)
      D0_1234 = D03m_fin(ql_sq,pq_sq,pql_sq,lsq,qsq,musq)
c      write(*,*) 'D0_1234',D0_1234

c Dij_2345
      call tens_red4(qsq,lsq,tsq,ql,qt,lt,
     #     C0_345, C0_245, C0_235, C0_234,   
     #     Cij_345, Cij_245, Cij_235, Cij_234,   
     #     D0_2345, 
     #     Dij_2345)

c Dij_1345
      call tens_red4(pq_sq,lsq,tsq,ql+pl,qt+pt,lt,
     #     C0_345, C0_145, C0_135, C0_134,   
     #     Cij_345, Cij_145, Cij_135, Cij_134,   
     #     D0_1345, 
     #     Dij_1345)

c Dij_1245
      call tens_red4(psq,ql_sq,tsq,pl+pq,pt,lt+qt,
     #     C0_245, C0_145, C0_125, C0_124,   
     #     Cij_245, Cij_145, Cij_125, Cij_124,   
     #     D0_1245, 
     #     Dij_1245)

c Dij_1235
      call tens_red4(psq,qsq,lt_sq,pq,pt+pl,qt+ql,
     #     C0_235, C0_135, C0_125, C0_123,   
     #     Cij_235, Cij_135, Cij_125, Cij_123,   
     #     D0_1235, 
     #     Dij_1235)

c Dij_1234
      call tens_red4(psq,qsq,lsq,pq,pl,ql,
     #     C0_234, C0_134, C0_124, C0_123,   
     #     Cij_234, Cij_134, Cij_124, Cij_123,   
     #     D0_1234, 
     #     Dij_1234)

c      call print_Dij(D0_1234,Dij_1234)
c      call print_Dij(D0_2345,Dij_2345)      
c      call print_Dij(D0_1345,Dij_1345)      
c      call print_Dij(D0_1245,Dij_1245)      
c      call print_Dij(D0_1235,Dij_1235)      

c E0_12345 = E0(p,q,l,t)
      E0_12345 = E0(psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, 
     #     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234)
c      write(*,*) 'E0',E0_12345

c Eij_12345
      call tens_red5(psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, 
     #     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #     E0_12345, 
     #     Eij_12345)

      if (debug) then
         write(*,*) 'E0_12345' 
         write(*,*) E0_12345
         write(*,*) 'D0_2345,D0_1345,D0_1245,D0_1235,D0_1234'
         write(*,*) D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
         write(*,*) 'C0_345,C0_245,C0_235,C0_234,C0_145,'//
     #        'C0_135,C0_134,C0_125,C0_124,C0_123'
         write(*,*) C0_345,C0_245,C0_235,C0_234,C0_145,
     #        C0_135,C0_134,C0_125,C0_124,C0_123
         write(*,*) 'B0_12,B0_13,B0_14,B0_15,B0_23,B0_24,B0_25,B0_34,'//
     #        'B0_35,B0_45'
         write(*,*) B0_12,B0_13,B0_14,B0_15,B0_23,B0_24,B0_25,B0_34,
     #        B0_35,B0_45
      endif
      
c      call print_Cij(C0_123,Cij_123)
c      call print_Dij(D0_2345,Dij_2345)
c      call print_Eij(E0_12345,Eij_12345)            
c      write(*,*) qsq,lsq,pq,pl,ppp,ppq,ppl,ql
      
      jref(0,0)=1d0 
      jref(0,1)=0d0
      jref(0,2)=0d0
      jref(0,3)=0d0
      
      jref(1,0)=0d0
      jref(1,1)=-1d0
      jref(1,2)=0d0
      jref(1,3)=0d0
      
      jref(2,0)=0d0
      jref(2,1)=0d0
      jref(2,2)=-1d0
      jref(2,3)=0d0

      jref(3,0)=0d0
      jref(3,1)=0d0
      jref(3,2)=0d0
      jref(3,3)=-1d0
      
      if  (num_ret_current.eq.1) then
         do mu=0,3
            jmu2(mu)=jal(mu)
            jmu3(mu)=jbe(mu)
         enddo
         do compon=0,3
            do mu=0,3
               jmu1(mu) = jref(compon,mu)
            enddo
            call compute_Jprod_gammaprod(
     #           brapp,ketp,hel,
     #           p,q,l,t,pp,
     #           jmu1,jmu2,jmu3)
            tri_box = virt_tri_box(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            pent    =    virt_pent(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            Jret(compon) = tri_box + pent
         enddo
      elseif  (num_ret_current.eq.2) then
         do mu=0,3
            jmu1(mu)=jal(mu)
            jmu3(mu)=jbe(mu)
         enddo
         do compon=0,3
            do mu=0,3
               jmu2(mu) = jref(compon,mu)
            enddo
            call compute_Jprod_gammaprod(
     #           brapp,ketp,hel,
     #           p,q,l,t,pp,
     #           jmu1,jmu2,jmu3)
            tri_box = virt_tri_box(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            pent    =    virt_pent(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            Jret(compon) = tri_box + pent
         enddo
      elseif  (num_ret_current.eq.3) then
         do mu=0,3
            jmu1(mu)=jal(mu)
            jmu2(mu)=jbe(mu)
         enddo
         do compon=0,3
            do mu=0,3
               jmu3(mu) = jref(compon,mu)
            enddo
            call compute_Jprod_gammaprod(
     #           brapp,ketp,hel,
     #           p,q,l,t,pp,
     #           jmu1,jmu2,jmu3)
            tri_box = virt_tri_box(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            pent    =    virt_pent(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
            Jret(compon) = tri_box + pent
         enddo
      else
         write(*,*) 'WRONG returned current',num_ret_current
         stop
      endif
      end



c Same as J_virtual_tri_box_pent, but this subroutine 
c returns the interpolated current across the 
c unstable region, where the momenta become planar
c
c  Carlo Oleari    13/6/2005
c
      subroutine interp_J_virtual_tri_box_pent(
c     bra(p'), ket(p) and helicity of the ket
     #     brapp,ketp,hel,
c     bra ferm_type  and ket ferm_type
     #     bra_ferm_type,ket_ferm_type,
c     momenta and squared renormalization scale
     #     p,q,l,t,pp,musq,
c     input currents (any couple among [J1,J2,J3]). ORDER IS IMPORTANT!!
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3
     #     num_ret_current,
c     returned current
     #     Jret)

      implicit none
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      integer bra_ferm_type,ket_ferm_type
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      real * 8 musq
      complex * 16 jal(0:3),jbe(0:3)
      integer num_ret_current
      complex * 16 jret(0:3)

      complex * 16 brapp_new(2,-1:1),ketp_new(2,-1:1)
      complex * 16 jretvirt(0:3,3),rJret(0:3),iJret(0:3)
      real * 8 pout(0:3),ppout(0:3),qout(0:3),lout(0:3),tout(0:3)
      real * 8 ptmp(0:3),pptmp(0:3),qtmp(0:3),ltmp(0:3),ttmp(0:3)
      real * 8 azimuth_vec_y_yz
      integer n,al,mu
      real * 8 beta(1:3),phi_qq3,th_qq3,phi_p      
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      real * 8 q_new(0:3),l_new(0:3),t_new(0:3),zero(0:3)
      real * 8 interp_parab
      integer casus
      real * 8 mod_qqaux1,th_x,phi_y_yz,th_x_new(3)
      real * 8 qqaux(0:3,3),qqaux_new(0:3,3)       
      real * 8 angle_th_x,angle_th_x_step,angle_th_x_increm
      logical is_planar, planar_cond

      logical debug
      parameter (debug = .false.)
c      parameter (debug = .true.)

      logical verbose
      parameter (verbose = .false.)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCC        ADJUSTABLE PARAMETERS                CCCCCCCCCCCCCC
C     incremental step in the search for "good" angles to be used
c     during the interpolation procedure
      angle_th_x_step = 0.02d0
c     once an angle is found, find the nex one by moving by a quantity 
c     equal to:
      angle_th_x_increm = 0.08d0      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c     go to a planar configuration 
      call to_planar(p,q,l,t,pp,bra_ferm_type,ket_ferm_type,
     #     pout,qout,lout,tout,ppout,beta,phi_qq3,th_qq3,phi_p,
     #     casus,qqaux,brapp_new,ketp_new)

      if (debug) then
         call from_planar(pout,qout,lout,tout,ppout,
     #        beta,phi_qq3,th_qq3,phi_p,
     #        ptmp,qtmp,ltmp,ttmp,pptmp)

         write(*,*) 'momenta check'
         write(*,*) 'first line:  in momenta'
         write(*,*) 'second line: back from planar momenta'
         write(*,*) 'third line:  planar momenta'
         call write_mom(p)
         call write_mom(ptmp)
         call write_mom(pout)
         write(*,*)
         call write_mom(pp)
         call write_mom(pptmp)
         call write_mom(ppout)
         write(*,*)
         call write_mom(q)
         call write_mom(qtmp)
         call write_mom(qout)
         write(*,*)
         call write_mom(l)
         call write_mom(ltmp)
         call write_mom(lout)
         write(*,*)
         call write_mom(t)
         call write_mom(ttmp)
         call write_mom(tout)
      endif

c     The first two vectors of qqaux are timelike
c      
c     Compute the angle formed by the first timelike vector
c     with respect to the x axis      
      mod_qqaux1 = sqrt(qqaux(1,1)**2+qqaux(2,1)**2+qqaux(3,1)**2)
      th_x = acos(qqaux(1,1)/mod_qqaux1)
      
c     Azimuthal angle of qqaux(0,1) in the yz plane with respect to the y axis
      phi_y_yz = azimuth_vec_y_yz(qqaux(0,1))

      if (debug) then
         write(*,*) 'check angles'
         write(*,*) '1 =>',mod_qqaux1*cos(th_x)/qqaux(1,1)
         write(*,*) '1 =>',mod_qqaux1*sin(th_x)*cos(phi_y_yz)/qqaux(2,1)
         write(*,*) '1 =>',mod_qqaux1*sin(th_x)*sin(phi_y_yz)/qqaux(3,1)
      endif      

      planar_cond = is_planar(brapp_new,ketp_new,hel,
     #     pout,qout,lout,tout,ppout,musq)
c
c
      if (planar_cond) then
c     the event is close to planar ==> compute 3 angles for 
c     the interpolation
         if (verbose) then
            write(*,*) '***********  INTERPOLATING... **************'
         endif
         
c     the spacelike vector will NOT be changed
         do al=0,3
            qqaux_new(al,3) = qqaux(al,3)
         enddo
         
         if (th_x.lt.pi/2) then
            angle_th_x = th_x
            do while (planar_cond.and.(angle_th_x.gt.0d0))
c     go to smaller and smaller angle_th_x to find good interpolating points
               angle_th_x = angle_th_x - angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
c
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.le.0d0) then
               if (verbose) then
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found an angle outside the "critical region"
               th_x_new(2) = angle_th_x
            endif
c
c     to find a second angle, decrese th_x_new(2) by a fixed amount
            th_x_new(1) = th_x_new(2) - angle_th_x_increm
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  put here condition for th_x_new(1)< 0
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c            
c     now find a third angle, across the critical region
            angle_th_x = th_x
            planar_cond = .true.
            do while (planar_cond.and.(angle_th_x.lt.pi))
c     go to bigger and bigger angle_th_x to find good interpolating points
               angle_th_x = angle_th_x + angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
               
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.gt.pi) then
               if (verbose) then
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found an angle across the "critical region"
               th_x_new(3) = angle_th_x
            endif            
         else
c the initial angle th_x is bigger than pi/2
            angle_th_x = th_x
            do while (planar_cond.and.(angle_th_x.gt.0d0))
c     go to smaller and smaller angle_th_x to find good interpolating points
               angle_th_x = angle_th_x - angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.le.0d0) then
               if (verbose) then 
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found an angle across the "critical region"
               th_x_new(1) = angle_th_x
            endif
c     now find a second angle, bigger than pi/2
            angle_th_x = th_x
            planar_cond = .true.
            do while (planar_cond.and.(angle_th_x.lt.pi))
c     go to bigger and bigger angle_th_x to find good interpolating points
               angle_th_x = angle_th_x + angle_th_x_step
c
               qqaux_new(0,1) = qqaux(0,1)
               qqaux_new(1,1) = mod_qqaux1*cos(angle_th_x)
               qqaux_new(2,1) = mod_qqaux1*sin(angle_th_x)*cos(phi_y_yz)
               qqaux_new(3,1) = mod_qqaux1*sin(angle_th_x)*sin(phi_y_yz)
c     
               qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
               qqaux_new(1,2) = -qqaux_new(1,1)
               qqaux_new(2,2) = -qqaux_new(2,1)
               qqaux_new(3,2) = -qqaux_new(3,1)
c     re-assign the new momenta to the correct order         
               call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
               planar_cond = is_planar(brapp_new,ketp_new,hel,
     #              pout,q_new,l_new,t_new,ppout,musq)
            enddo
            if (angle_th_x.gt.pi) then
               if (verbose) then
                  write(*,*) 
     #                 'No good interpolating angle found! RETURN 0'
               endif
               do al=0,3
                  jret(al) = 0d0
               enddo
               RETURN
            else
c     we have found the second angle
               th_x_new(2) = angle_th_x
            endif
c
c     to find a third angle, increse th_x_new(2) by a fixed amount
            th_x_new(3) = th_x_new(2) + angle_th_x_increm
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  put here condition for th_x_new(3)> pi
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c            
         endif
c     if we arrive here, than the subroutine has found 3 angles to
c     be used for the interpolation
         if (verbose) then
            write(*,*) 'INTERPOLATION SUCCESSFUL'
         endif
c     
         do n=1,3
            qqaux_new(0,1) = qqaux(0,1)
            qqaux_new(1,1) = mod_qqaux1*cos(th_x_new(n))
            qqaux_new(2,1) = mod_qqaux1*sin(th_x_new(n))*cos(phi_y_yz)
            qqaux_new(3,1) = mod_qqaux1*sin(th_x_new(n))*sin(phi_y_yz)
c     
            qqaux_new(0,2) = qqaux(0,2)
c     the vectorial part is - qqaux_new(i,1)
            qqaux_new(1,2) = -qqaux_new(1,1)
            qqaux_new(2,2) = -qqaux_new(2,1)
            qqaux_new(3,2) = -qqaux_new(3,1)
c                       
            if (debug) then
               write(*,*) 'interpolation vector number',n
               write(*,*) 'original'
               call write_mom(qqaux(0,1))
               write(*,*) 'slightly moved'
               call write_mom(qqaux_new(0,1))
               write(*,*) 'original'
               call write_mom(qqaux(0,2))
               write(*,*) 'slightly moved'
               call write_mom(qqaux_new(0,2))
               write(*,*)
            endif            
c
c     re-assign the new momenta to the correct order         
            call from_qqaux_to_qlt(casus,qqaux_new,q_new,l_new,t_new)
c     now go back to NON planar configuration            
            call from_planar(pout,q_new,l_new,t_new,ppout,
     #           beta,phi_qq3,th_qq3,phi_p,
     #           ptmp,qtmp,ltmp,ttmp,pptmp)
c            
            if (debug) then
               write(*,*) 'NUMBER',n
               write(*,*) 'first line: old momenta'
               write(*,*) 'second line: new momenta'
               write(*,*) 'p'
               call write_mom(pout)
               call write_mom(ptmp)
               write(*,*) 'pp'
               call write_mom(ppout)
               call write_mom(pptmp)
               write(*,*) 'q'
               call write_mom(q_new)
               call write_mom(qtmp)
               write(*,*) 'l'
               call write_mom(l_new)
               call write_mom(ltmp)
               write(*,*) 't'
               call write_mom(t_new)
               call write_mom(ttmp)
               do al=0,3
                  zero(al) = 
     #                 ptmp(al)+qtmp(al)+ltmp(al)+ttmp(al)-pptmp(al)
               enddo
               write(*,*) 'p+q+l+t-pp'
               call write_mom(zero)
            endif
c     
            call J_virtual_tri_box_pent(
     #           brapp,ketp,hel,
     #           ptmp,qtmp,ltmp,ttmp,pptmp,musq,
     #           jal,jbe,
     #           num_ret_current,
     #           Jretvirt(0,n))                           
         enddo   
c         
         do mu=0,3         
            rJret(mu) = interp_parab(
     #           th_x_new(1),dreal(Jretvirt(mu,1)),
     #           th_x_new(2),dreal(Jretvirt(mu,2)),
     #           th_x_new(3),dreal(Jretvirt(mu,3)),
     #           th_x)

            iJret(mu) = interp_parab(
     #           th_x_new(1),dimag(Jretvirt(mu,1)),
     #           th_x_new(2),dimag(Jretvirt(mu,2)),
     #           th_x_new(3),dimag(Jretvirt(mu,3)),
     #           th_x)
         enddo
c
         do mu=0,3
            Jret(mu) = rJret(mu)+(0,1)*iJret(mu)
         enddo
      else
c     the event is NOT planar
         call J_virtual_tri_box_pent(
     #     brapp,ketp,hel,
     #     p,q,l,t,pp,musq,
     #     jal,jbe,
     #     num_ret_current,
     #     Jret)
      endif      
      end



c     Compute J.p_i, where J's are the external currents, and all the 
c     <bra|...|ket> gamma prods
c     The returned values are in the common blocks
c
c  Carlo Oleari    13/6/2005
c
      subroutine compute_Jprod_gammaprod(
c     bra(p'), ket(p) and helicity 
     #     bra,ket,hel,
c     momenta
     #     p,q,l,t,pp,
c     input currents
     #     jmu1,jmu2,jmu3)
      implicit none
      complex * 16 bra(2,-1:1),ket(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      complex*16 jmu1(0:3),jmu2(0:3),jmu3(0:3)

      complex * 16 cq(0:3),cl(0:3)
      complex * 16 dpmu1,dpmu2,dpmu3,dqmu1,dqmu2,dqmu3
      complex * 16 dlmu1,dlmu2,dlmu3,dppmu1,dppmu2,dppmu3
      complex * 16 dmu1mu2,dmu1mu3,dmu2mu3
      common/dotp_pol_vec/dpmu1,dpmu2,dpmu3,dqmu1,dqmu2,dqmu3,
     #     dlmu1,dlmu2,dlmu3,dppmu1,dppmu2,dppmu3,
     #     dmu1mu2,dmu1mu3,dmu2mu3
      complex * 16 gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q
      complex * 16 gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2
      complex * 16 gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3
      complex * 16 gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
      common/gammaprods/gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q,
     #     gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2,
     #     gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3,
     #     gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
c      
      complex * 16 local_mom(0:3,5), cdotp, ccdotp
      integer al
      
      logical debug
c      parameter (debug=.true.)
      parameter (debug=.false.)

      dpmu1 = cdotp(p,jmu1)
      dpmu2 = cdotp(p,jmu2)
      dpmu3 = cdotp(p,jmu3)

      dqmu1 = cdotp(q,jmu1)
      dqmu2 = cdotp(q,jmu2)
      dqmu3 = cdotp(q,jmu3)

      dlmu1 = cdotp(l,jmu1)
      dlmu2 = cdotp(l,jmu2)
      dlmu3 = cdotp(l,jmu3)

      dppmu1 = cdotp(pp,jmu1)
      dppmu2 = cdotp(pp,jmu2)
      dppmu3 = cdotp(pp,jmu3)

      dmu1mu2 = ccdotp(jmu1,jmu2)
      dmu1mu3 = ccdotp(jmu1,jmu3)
      dmu2mu3 = ccdotp(jmu2,jmu3)
      
      do al=0,3
         cl(al) = l(al)
         cq(al) = q(al)
      enddo

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = jmu2(al)
         local_mom(al,4) = cq(al)
         local_mom(al,5) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,5,gp_mu3_l_mu2_q_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_l_q)

      do al=0,3
         local_mom(al,1) = jmu2(al)
         local_mom(al,2) = cq(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu2_q_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_mu2_q)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = jmu2(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_l_mu2)

      do al=0,3
         local_mom(al,1) = cl(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_l_mu2_q)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_mu2_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cl(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_l_mu1)

      do al=0,3
         local_mom(al,1) = cl(al)
         local_mom(al,2) = cq(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_l_q_mu1)

      do al=0,3
         local_mom(al,1) = jmu3(al)
         local_mom(al,2) = cq(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_mu3_q_mu1)

      do al=0,3
         local_mom(al,1) = cl(al)
         local_mom(al,2) = jmu2(al)
         local_mom(al,3) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,3,gp_l_mu2_mu1)

      do al=0,3
         local_mom(al,1) = cl(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_l)

      do al=0,3
         local_mom(al,1) = jmu2(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_mu2)

      do al=0,3
         local_mom(al,1) = cq(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_q)

      do al=0,3
         local_mom(al,1) = jmu3(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_mu3)

      do al=0,3
         local_mom(al,1) = jmu1(al)
      enddo
      call bra_slash_ket(bra,ket,hel,local_mom,1,gp_mu1)

      if (debug) then
         write(*,*) 'gamma prods'
         write(*,*) gp_l,gp_q,gp_mu2,gp_mu3,gp_mu3_l_mu2,gp_mu3_l_q,
     #        gp_mu3_mu2_q,gp_mu3_l_mu2,gp_l_mu2_q
         
         write(*,*) 
         write(*,*) gp_mu3_l_mu2_q_mu1,gp_mu2_q_mu1,gp_mu3_mu2_mu1,
     #        gp_mu3_l_mu1,gp_l_q_mu1,gp_l_mu2_mu1,gp_mu3_q_mu1,gp_mu1
      endif

      if (debug) then
         write(*,*) gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q,
     #        gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2,
     #        gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3,
     #        gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
      endif
      end



c     Given casus and qqaux, the subroutine returns the 
c     right ordered q, l and t
c     See subroutine for details
c
c     Carlo Oleari    13/6/2005
c
      subroutine from_qqaux_to_qlt(casus,qqaux,q,l,t)
      implicit none
      integer casus
      real * 8 qqaux(0:3,3)
      real * 8 q(0:3),l(0:3),t(0:3)
      integer al

      if (casus.eq.1) then
         do al=0,3
            q(al) = qqaux(al,1)
            l(al) = qqaux(al,2)
            t(al) = qqaux(al,3)
         enddo
      elseif (casus.eq.2)then
         do al=0,3
            q(al) = qqaux(al,1)
            t(al) = qqaux(al,2)
            l(al) = qqaux(al,3)
         enddo
      elseif (casus.eq.3)then
         do al=0,3
            l(al) = qqaux(al,1)
            t(al) = qqaux(al,2)
            q(al) = qqaux(al,3)
         enddo
      endif
      end




c     This subroutine receives pin,qin,lin,tin and ppin = pin+qin+lin+tin, 
c     ("in" stands for initial) as input momenta
c     Among qin, lin and tin there are two timelike and one spacelike vector
c
c     The subroutine boosts the system in the centre of mass of the 
c     two timelike momenta.
c     Then it rotates the system so that the spacelike momentum has 
c     vector component along the z axis.
c     It rotates the system so that p belongs to the yz plane.
c     At this point, pp belongs to the yz plane too.
c
c     Carlo Oleari 13/6/2005
c
      subroutine to_planar(pin,qin,lin,tin,ppin,
c     bra ferm_type and ket ferm_type
     #     bra_ferm_type,ket_ferm_type,
c     returned momenta
     #     p,q,l,t,pp,
c     returned angles and boost vector
     #     beta,phi_qq3,th_qq3,phi_p,
c     returned case and q l t vectors ordered according their invariant mass
     #     casus, qqaux,
c     returned bra and ket for the boosted p and pp
     #     brapp_new,ketp_new)

      implicit none
      real * 8 pin(0:3),qin(0:3),lin(0:3),tin(0:3),ppin(0:3)
      integer bra_ferm_type,ket_ferm_type
      real * 8 beta(1:3),phi_qq3,th_qq3,phi_p
      real * 8 p(0:3),q(0:3),l(0:3),t(0:3),pp(0:3)
      integer casus      
      real * 8 qqaux(0:3,3)
      complex * 16 brapp_new(2,-1:1),ketp_new(2,-1:1)      

      real * 8 ptmp(0:3),pptmp(0:3)
      real * 8 paux(0:3),ppaux(0:3)
      real * 8 qq(0:3,3), qqtmp(0:3,3)
      integer i
      real * 8 dotrr,qqaux_sq(1:3)
      integer al
      real * 8 azimuth_vec_y
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)

      logical debug
      parameter (debug=.false.)
c     parameter (debug=.true.)
      
      if (debug) then
         write(*,*) 'test dot prods'
         write(*,*) dotrr(pin,pin),dotrr(ppin,ppin)
         write(*,*) dotrr(qin,qin),dotrr(lin,lin),dotrr(tin,tin)
         write(*,*) dotrr(pin,qin),dotrr(pin,lin),dotrr(pin,tin),
     #        dotrr(pin,ppin)
         write(*,*) dotrr(qin,lin),dotrr(qin,tin),dotrr(qin,ppin)
         write(*,*) dotrr(lin,tin),dotrr(lin,ppin)
         write(*,*) dotrr(tin,ppin)
      endif

      do al=0,3
         paux(al)  = pin(al)
         qqaux(al,1) = qin(al)
         qqaux(al,2) = lin(al)
         qqaux(al,3) = tin(al)
         ppaux(al) = ppin(al)
      enddo

c     find the two TIMELIKE momenta and call them qq(mu,1) and qq(mu,2)
c     Then the third SPACELIKE momentum is called qq(mu,3)
c     Three cases  for 2 timelike (T) and one spacelike (S) q l t ==>
c     1) T T S
c     2) T S T
c     3) S T T
      do i=1,3
         qqaux_sq(i) = dotrr(qqaux(0,i),qqaux(0,i))
      enddo
      if ((qqaux_sq(1).gt.0d0).and.(qqaux_sq(2).gt.0d0)) then
         casus = 1
         do al=0,3
            qq(al,1) = qqaux(al,1)
            qq(al,2) = qqaux(al,2)
            qq(al,3) = qqaux(al,3)
         enddo
      elseif ((qqaux_sq(1).gt.0d0).and.(qqaux_sq(3).gt.0d0)) then
         casus = 2
         do al=0,3
            qq(al,1) = qqaux(al,1)
            qq(al,2) = qqaux(al,3)
            qq(al,3) = qqaux(al,2)
         enddo
      elseif ((qqaux_sq(2).gt.0d0).and.(qqaux_sq(3).gt.0d0)) then
         casus = 3
         do al=0,3
            qq(al,1) = qqaux(al,2)
            qq(al,2) = qqaux(al,3)
            qq(al,3) = qqaux(al,1)
         enddo
      else
         write(*,*) 'ERROR: case NOT implemented in to_planar'
         write(*,*) qqaux_sq(1),qqaux_sq(2),qqaux_sq(3)
         stop
      endif
c
c     boost in the rest frame of qq(1) + qq(2)      
      do al=1,3
         beta(al) = - (qq(al,1)+qq(al,2))/(qq(0,1)+qq(0,2))
      enddo     
      call boost(beta,paux,ptmp)
      call boost(beta,ppaux,pptmp)
      do i=1,3
         call boost(beta,qq(0,i),qqtmp(0,i))
      enddo
c
      if (debug) then
         write(*,*) 'boosted momenta in the rest frame of the'// 
     #        ' two timelike momenta'
         call write_mom(ptmp)
         call write_mom(pptmp)
         do i=1,3
            call write_mom(qqtmp(0,i))
         enddo
      endif
c      
c     determine the angles of qq3 wrt the z axis
      phi_qq3 = azimuth_vec_y(qqtmp(0,3))
      th_qq3 = acos(qqtmp(3,3)/
     #     sqrt(qqtmp(1,3)**2+qqtmp(2,3)**2+qqtmp(3,3)**2))
c      
c     now rotate the spacelike momenta to be parallel to the z axis
c     and accordingly all the other momenta      
      call rotatez(-phi_qq3,ptmp,paux)
      call rotatex(th_qq3,paux,ptmp)
      call rotatez(-phi_qq3,pptmp,ppaux)
      call rotatex(th_qq3,ppaux,pptmp)
      do i=1,3
         call rotatez(-phi_qq3,qqtmp(0,i),qqaux(0,i))
         call rotatex(th_qq3,qqaux(0,i),qqtmp(0,i))
      enddo
c      
      if (debug) then
         write(*,*) 
     #        'The spacelike momentun must be parallel to the z axis'
         call write_mom(ptmp)
         call write_mom(pptmp)
         do i=1,3
            call write_mom(qqtmp(0,i))
         enddo
      endif
c
c     determine the azimuthal angle of p wrt the yz plane
      phi_p = azimuth_vec_y(ptmp)
c      
c     now rotate p into the yz plane
c     and accordingly all the other momenta      
      call rotatez(-phi_p,ptmp,paux)
      call rotatez(-phi_p,pptmp,ppaux)
      do i=1,3
         call rotatez(-phi_p,qqtmp(0,i),qqaux(0,i))
      enddo
      if (debug) then
         write(*,*) 
     #        'The spacelike momentun must be parallel to the z axis'
         write(*,*) 'p in the yz plane'
         call write_mom(paux)
         call write_mom(ppaux)
         do i=1,3
            call write_mom(qqaux(0,i))
         enddo
      endif
c
      do al=0,3
         p(al)  = paux(al)
         pp(al) = ppaux(al)
      enddo
c
      call from_qqaux_to_qlt(casus,qqaux,q,l,t)
c     
      if (debug) then
         write(*,*) 'test dot prods'
         write(*,*) dotrr(p,p),dotrr(pp,pp)
         write(*,*) dotrr(q,q),dotrr(l,l),dotrr(t,t)
         write(*,*) dotrr(p,q),dotrr(p,l),dotrr(p,t),dotrr(p,pp)
         write(*,*) dotrr(q,l),dotrr(q,t),dotrr(q,pp)
         write(*,*) dotrr(l,t),dotrr(l,pp)
         write(*,*) dotrr(t,pp)
      endif    
c     now compute the boosted bra and ket
      call bra(pp,bra_ferm_type,brapp_new)
      call ket(p,ket_ferm_type,ketp_new) 
      end



c     This subroutine receives pin,qin,lin,tin and ppin = pin+qin+lin+tin, 
c     ("in" stands for initial) 
c     and rotates and boosts the momenta according to the angles
c     and boost: beta,phi_qq3,th_qq3,phi_p  back to the NON planar 
c     configuration.
c     See subroutine to_planar for the meaning of these inputs
c
c     Carlo Oleari    13/6/2005
c
      subroutine from_planar(pin,qin,lin,tin,ppin,
     #     beta,phi_qq3,th_qq3,phi_p,p,q,l,t,pp)
      implicit none
      real * 8 pin(0:3),qin(0:3),lin(0:3),tin(0:3),ppin(0:3)
      real * 8 beta(1:3),phi_qq3,th_qq3,phi_p
      real * 8 p(0:3),q(0:3),l(0:3),t(0:3),pp(0:3)

      real * 8 ptmp(0:3),pptmp(0:3),qtmp(0:3),ltmp(0:3),ttmp(0:3)      
      real * 8 paux(0:3),ppaux(0:3),qaux(0:3),laux(0:3),taux(0:3)      
      real * 8 mbeta(1:3)
      real * 8 dotrr
      integer al
      
      logical debug
      parameter (debug=.false.)
      
      if (debug) then
         write(*,*) 'test dot prods in from_planar_new'
         write(*,*) dotrr(pin,pin),dotrr(ppin,ppin)
         write(*,*) dotrr(qin,qin),dotrr(lin,lin),dotrr(tin,tin)
         write(*,*) dotrr(pin,qin),dotrr(pin,lin),dotrr(pin,tin),
     #        dotrr(pin,ppin)
         write(*,*) dotrr(qin,lin),dotrr(qin,tin),dotrr(qin,ppin)
         write(*,*) dotrr(lin,tin),dotrr(lin,ppin)
         write(*,*) dotrr(tin,ppin)
      endif
      
      call rotatez(phi_p,pin,p)
      call rotatez(phi_p,ppin,pp)
      call rotatez(phi_p,qin,q)
      call rotatez(phi_p,lin,l)
      call rotatez(phi_p,tin,t)

      if (debug) then
         write(*,*) 'after rotation around z away from yz plane'
         call write_mom(p)
         call write_mom(pp)
         call write_mom(q)
         call write_mom(l)
         call write_mom(t)
      endif

      call rotatex(-th_qq3,p,paux)
      call rotatex(-th_qq3,pp,ppaux)
      call rotatex(-th_qq3,q,qaux)
      call rotatex(-th_qq3,l,laux)
      call rotatex(-th_qq3,t,taux)

      call rotatez(phi_qq3,paux,ptmp)
      call rotatez(phi_qq3,ppaux,pptmp)
      call rotatez(phi_qq3,qaux,qtmp)
      call rotatez(phi_qq3,laux,ltmp)
      call rotatez(phi_qq3,taux,ttmp)

      if (debug) then
         write(*,*) 'just before boost back...'
         call write_mom(ptmp)
         call write_mom(pptmp)
         call write_mom(qtmp)
         call write_mom(ltmp)
         call write_mom(ttmp)
      endif

      do al=1,3
         mbeta(al) = -beta(al)
      enddo
      call boost(mbeta,ptmp,p)
      call boost(mbeta,pptmp,pp)
      call boost(mbeta,qtmp,q)
      call boost(mbeta,ttmp,t)
      call boost(mbeta,ltmp,l)

      if (debug) then
         write(*,*) 'test dot prods in from_planar_new'
         write(*,*) dotrr(p,p),dotrr(pp,pp)
         write(*,*) dotrr(q,q),dotrr(l,l),dotrr(t,t)
         write(*,*) dotrr(p,q),dotrr(p,l),dotrr(p,t),dotrr(p,pp)
         write(*,*) dotrr(q,l),dotrr(q,t),dotrr(q,pp)
         write(*,*) dotrr(l,t),dotrr(l,pp)
         write(*,*) dotrr(t,pp)
      endif

      end

      
c     It returns true if the configuration is "close to" planar,
c     according to a given definition of "planar".
c
c     Carlo Oleari    13/6/2005
c     
      function is_planar(brapp,ketp,hel,p,q,l,t,pp,musq)
      implicit none
      logical is_planar
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      real * 8 musq

      integer al
      complex * 16 virt,tree,born,ratio,ccdotp      
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      real * 8 rratio,iratio,rratio_cut,iratio_cut,onemax,onemin,
     #     zeromax,zeromin
      complex * 16 jmu1(0:3),jmu2(0:3),jmu3(0:3),jret(0:3)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC      
CCCCCCCCCCCCCCC        ADJUSTABLE PARAMETERS                CCCCCCCCCCCCC
C     How close to 1 the ratio virtual/tree in the gauge check should be
      rratio_cut = 0.01d0
      iratio_cut = 0.01d0
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC      
      
      onemax = 1d0 + rratio_cut
      onemin = 1d0 - rratio_cut
      zeromax = 0d0 + iratio_cut
      zeromin = 0d0 - iratio_cut
      
c     check the goodness of the gauge invariance
      do al=0,3
         jmu1(al) = q(al)
         jmu3(al) = t(al)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c     Set the middle current equal to (1,1,1,1)         
c     It can be any random current
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
         jmu2(al) = 1d0
      enddo

      tree = born(brapp,ketp,hel,
     #     p,q,l,t,pp,
     #     jmu1,jmu2,jmu3)      
      
      call  J_virtual_tri_box_pent(
     #     brapp,ketp,hel,
     #     p,q,l,t,pp,musq,
     #     jmu1,jmu3,
c     uncontracted external current. It can be 1, 2 or 3
     #     2,
c     returned current
     #     Jret)

      virt = ccdotp(Jret,jmu2)
      ratio = virt/tree/(pi**2/3-7)
      rratio = dreal(ratio)
      iratio = imag(ratio)

c      write(*,*) virt, ratio

      if ((rratio.gt.onemax).or.(rratio.lt.onemin).or.
     #     (iratio.gt.zeromax).or.(iratio.lt.zeromin)) then         
         is_planar = .true.
      else
         is_planar = .false.
      endif

      end


c     Returns the azimuthal angle between the vector and the y axis
c     in the xy plane
      function azimuth_vec_y(p)
      implicit none
      real * 8 azimuth_vec_y,p(0:3)
      real * 8 azimuth_vec_x,phi_x
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      phi_x = azimuth_vec_x(p)      
      if ((-pi.lt.phi_x).and.(phi_x.lt.-pi/2)) then
         azimuth_vec_y = 3*pi/2 + phi_x 
      else         
         azimuth_vec_y = phi_x - pi/2
      endif
      end



CCCCCCCCCCCCC end new  CCCCCCCCCCCCCCCCCCCCCCCCCCCC

c     returns the azimuthal angle between the vector and the x axis
      function azimuth_vec_x(p)
      implicit none
      real * 8 azimuth_vec_x,p(0:3)
      real * 8 phi
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)      
      real * 8 tiny
      parameter (tiny = 1d-13)
      
      if ((abs(p(1)).lt.tiny).and.(abs(p(2)).lt.tiny)) then
         azimuth_vec_x = 0d0
         return
      endif

      if (abs(p(1)).lt.tiny) then
         if (p(2).gt.0d0) then
            phi = pi/2
         else
            phi = -pi/2
         endif
      else
         phi = atan(p(2)/p(1))
         if (p(1).lt.0d0) then
            if (phi.gt.0d0) then               
               phi = phi - pi
            else
               phi = phi + pi
            endif
         endif         
      endif
      azimuth_vec_x = phi
      end



      subroutine rotatez(theta,p,pp)
      implicit none
      real * 8 p(0:3),pp(0:3),theta
      pp(0) = p(0)
      pp(1) = p(1)*cos(theta) - p(2)*sin(theta)
      pp(2) = p(1)*sin(theta) + p(2)*cos(theta)
      pp(3) = p(3)
      end
      
      subroutine rotatex(theta,p,pp)
      implicit none
      real * 8 p(0:3),pp(0:3),theta
      pp(0) = p(0)
      pp(1) = p(1)
      pp(2) = p(2)*cos(theta) - p(3)*sin(theta)
      pp(3) = p(2)*sin(theta) + p(3)*cos(theta)
      end
      
      subroutine rotatey(theta,p,pp)
      implicit none
      real * 8 p(0:3),pp(0:3),theta
      pp(0) = p(0)
      pp(1) = p(1)*cos(theta) + p(3)*sin(theta)
      pp(2) = p(2)      
      pp(3) = -p(1)*sin(theta) + p(3)*cos(theta)
      end
      

      subroutine crotatez(theta,p,pp)
      implicit none
      real * 8 theta
      complex * 16 p(0:3),pp(0:3)
      pp(0) = p(0)
      pp(1) = p(1)*cos(theta) - p(2)*sin(theta)
      pp(2) = p(1)*sin(theta) + p(2)*cos(theta)
      pp(3) = p(3)
      end
      
      subroutine crotatex(theta,p,pp)
      implicit none
      real * 8 theta
      complex * 16 p(0:3),pp(0:3)
      pp(0) = p(0)
      pp(1) = p(1)
      pp(2) = p(2)*cos(theta) - p(3)*sin(theta)
      pp(3) = p(2)*sin(theta) + p(3)*cos(theta)
      end
      

      function theta(x)
      implicit none
      real * 8 theta,x
      if (x.gt.0d0) then
         theta = 1d0
      else
         theta = 0d0
      endif
      end

      function sgn(x)
      implicit none
      real * 8 sgn,x
      if (x.gt.0d0) then
         sgn = +1d0
      else
         sgn = -1d0
      endif
      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCC                SCALAR INTEGRALS                CCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c    ***********   D02m_fin(s,t,m3sq,m4sq,musq)   *************
c
c Scalar box with MASSLESS PROPAGATORS and with external kinematics: 
c
c                     k
c     p1 ->-----------<-------------<-- p4
c             |                 |
c             |                 |
c             |                 |  
c     p2 ->-------------------------<-- p3
c
c
c    s = (p1+p2)^2
c    t = (p2+p3)^2
c    p1^2 = 0, p2^2 = 0, p3^2 = m3sq <>0, p4^2 = m4sq <>0
c    musq = mu^2 = reference dimensional scale 
c 
c  int d^dk/(2 pi)^d 1/(k^2)/(k+p1)^2/(k+p1+p2)^2/(k+p1+p2+p3)^2 = 
c          N_ep * D02m_fin(s,t,m3sq,m4sq,musq);
c          N_ep = i/(4 pi)^2 (4 pi)^ep Gamma(1+ep) (musq)^(-ep)

      function D02m_fin(s,t,m3sq,m4sq,musq)
      implicit none
      complex * 16 D02m_fin
      real * 8 s,t,m3sq,m4sq,musq
      real * 8 ms,mt,mm3sq,mm4sq
      complex * 16 lnms,lnmt,lnmm3sq,lnmm4sq,lnsot
      complex * 16 li2arg1,li2arg2
      real * 8 arg1,arg2
      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)

      complex * 16 ris
      real * 8 prefactor, theta, dilog, im_part
      
      if (musq.lt.0d0) then
         write(*,*) 
     #        'POSSIBLE ERROR IN D02m_fin: SCALE MUSQ LESS THAN ZERO!!'
      endif

      prefactor = 1d0/(s*t)
      ms = -s/musq
      mt = -t/musq
      mm3sq = - m3sq/musq
      mm4sq = - m4sq/musq
      
      lnms = log(abs(ms)) - ipi*theta(-ms)
      lnmt = log(abs(mt)) - ipi*theta(-mt)
      lnmm3sq = log(abs(mm3sq)) - ipi*theta(-mm3sq)
      lnmm4sq = log(abs(mm4sq)) - ipi*theta(-mm4sq)
      lnsot = log(abs(s/t)) + ipi * im_part(s,t)

      arg1 = 1-m3sq/t
      arg2 = 1-m4sq/t

c     (m3sq/t.lt.0d0)
      if (arg1.gt.1d0) then
         li2arg1 = -dilog(1d0/arg1) - log(arg1)**2/2+pi2/3 
     #        - ipi*log(arg1)*im_part(m3sq,t)
      else
         li2arg1 = dilog(arg1)
      endif

c     (m4sq/t.lt.0d0)
      if (arg2.gt.1d0)  then
         li2arg2 = -dilog(1d0/arg2) - log(arg2)**2/2+pi2/3 
     #        - ipi*log(arg2)*im_part(m4sq,t)
      else
         li2arg2 = dilog(arg2)
      endif

      ris = lnms**2/2+lnmt**2-lnmm3sq**2/2-lnmm4sq**2/2
     #     +lnmm3sq*lnmm4sq-(lnmm4sq+lnmm3sq-lnms)*lnms
     #     -2*li2arg1-2*li2arg2-lnsot**2 - pi2/6
        
      D02m_fin = prefactor * ris
      end




c    ***********   D03m_fin(s,t,m2sq,m3sq,m4sq,musq)   *************
c
c Scalar box with MASSLESS PROPAGATORS and with external kinematics: 
c
c
c                     k
c     p1 ->-----------<-------------<-- p4
c             |                 |
c             |                 |
c             |                 |  
c     p2 ->-------------------------<-- p3
c
c
c    s = (p1+p2)^2
c    t = (p2+p3)^2
c    p1^2 = 0, p2^2 = m2sq <>0, p3^2 = m3sq <>0, p4^2 = m4sq <>0
c    musq = mu^2 = reference dimensional scale 
c 
c  int d^dk/(2 pi)^d 1/(k^2)/(k+p1)^2/(k+p1+p2)^2/(k+p1+p2+p3)^2 =  
c          N_ep * D03m_fin(s,t,m2sq,m3sq,m4sq,musq);
c          N_ep = i/(4 pi)^2 (4 pi)^ep Gamma(1+ep) (musq)^(-ep)

      function D03m_fin(s,t,m2sq,m3sq,m4sq,musq)
      implicit none
      complex * 16 D03m_fin
      real * 8 s,t,m2sq,m3sq,m4sq,musq
      complex * 16 D03m_fin_BDK,D03m_fin_DN
      D03m_fin = D03m_fin_BDK(s,t,m2sq,m3sq,m4sq,musq)
c      D03m_fin = D03m_fin_DN(s,t,m2sq,m3sq,m4sq,musq)
      
      end




      function D03m_fin_BDK(s,t,m2sq,m3sq,m4sq,musq)
      implicit none
      complex * 16 D03m_fin_BDK
      real * 8 s,t,m2sq,m3sq,m4sq,musq
      real * 8 ms,mt,mm2sq,mm3sq,mm4sq,sot
      complex * 16 lnms,lnmt,lnmm2sq,lnmm3sq,lnmm4sq,lnsot,lnarg3
      complex * 16 li2arg1,li2arg2,li2arg3
      real * 8 arg1,arg2,arg3
      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)

      complex * 16 ris
      real * 8 prefactor, theta, dilog, sgn, im_part
      real * 8 im_a, im_b,im_ab, eta3

      if (musq.lt.0d0) then
         write(*,*) 
     #        'POSSIBLE ERROR IN D03m_fin: SCALE MUSQ LESS THAN ZERO!!'
      endif


c  1/(ms*mt-mm2s*mm4s)*
c  (1/2*ln(ms)^2+1/2*ln(mt)^2-1/2*ln(mm2s)^2-1/2*ln(mm4s)^2-ln(ms/mt)^2
c  +ln(mm2s)*ln(mm3s)-(ln(mm3s)+ln(mm2s)-ln(mt))*ln(mt)
c  +ln(mm3s)*ln(mm4s)-(ln(mm4s)+ln(mm3s)-ln(ms))*ln(ms)
c  -2*li2(1-mm2s/ms)
c  -2*li2(1-mm4s/mt)
c  +2*li2(1-mm2s*mm4s/ms/mt))

      prefactor = 1d0/(s*t-m2sq*m4sq)
      ms = -s/musq
      mt = -t/musq
      mm2sq = - m2sq/musq
      mm3sq = - m3sq/musq
      mm4sq = - m4sq/musq
      sot = s/t
      
      lnms = log(abs(ms)) - ipi*theta(-ms)
      lnmt = log(abs(mt)) - ipi*theta(-mt)
      lnmm2sq = log(abs(mm2sq)) - ipi*theta(-mm2sq)
      lnmm3sq = log(abs(mm3sq)) - ipi*theta(-mm3sq)
      lnmm4sq = log(abs(mm4sq)) - ipi*theta(-mm4sq)
      lnsot = log(abs(sot)) + ipi * im_part(s,t)

     
      arg1 = 1-m2sq/s
      arg2 = 1-m4sq/t
      arg3 = 1-m2sq*m4sq/s/t
      

c     m2sq/s < 0 
      if (arg1.gt.1d0) then
         li2arg1 = -dilog(1d0/arg1) - log(arg1)**2/2+pi2/3 
     #        - ipi*log(arg1)*im_part(m2sq,s)
      else
         li2arg1 = dilog(arg1)
      endif

c     m4sq/t < 0
      if (arg2.gt.1d0) then
         li2arg2 = -dilog(1d0/arg2) - log(arg2)**2/2+pi2/3 
     #        - ipi*log(arg2)*im_part(m4sq,t)
      else
         li2arg2 = dilog(arg2)
      endif


      im_a = m2sq/s*(1d0/m2sq-1d0/s)
      im_b = m4sq/t*(1d0/m4sq-1d0/t)
      im_ab = m2sq*m4sq/s/t*(1d0/m2sq-1d0/s+1d0/m4sq-1d0/t)


c     m2sq*m4sq/(s*t)  < 0
      if (arg3.gt.1d0) then
         li2arg3 = -dilog(1d0/arg3) - log(arg3)**2/2+pi2/3 
     #        - ipi*log(arg3)*sgn(im_ab)
      else
         li2arg3 = dilog(arg3)
      endif

      if (arg3.lt.0d0) then
         lnarg3 = log(abs(arg3)) - ipi*sgn(im_ab)
      else
         lnarg3 = log(arg3)
      endif
      
      li2arg3 = li2arg3  + 2*ipi*eta3(im_a,im_b,im_ab)*lnarg3

      
      ris = lnms**2/2+lnmt**2/2-lnmm2sq**2/2-lnmm4sq**2/2
     #     +lnmm2sq*lnmm3sq
     #     -(lnmm3sq+lnmm2sq-lnmt)*lnmt+lnmm3sq*lnmm4sq
     #     -(lnmm4sq+lnmm3sq-lnms)*lnms      
      
      ris = ris - 2*li2arg1 - 2*li2arg2 + 2*li2arg3 
     #     - lnsot**2
        
      D03m_fin_BDK = prefactor * ris
      end






c from hep-ph/0006249 by Duplancic and Nizic
      function D03m_fin_DN(s,t,m2sq,m3sq,m4sq,musq)
      implicit none
      complex * 16 D03m_fin_DN
      real * 8 s,t,m2sq,m3sq,m4sq,musq
      real * 8 ms,mt,mm2sq,mm3sq,mm4sq
      complex * 16 lnms,lnmt,lnmm2sq,lnmm3sq,lnmm4sq
      complex * 16 li2m2ot,li2m4os,li2arg3
      real * 8 m2ot,m4os,arg3
      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)

      complex * 16 ris, li2arg(4)
      real * 8 prefactor, theta, dilog, sgn, im_part, sgnratio

      real * 8 f3m, arg(4),argom, sign
      integer i

      if (musq.lt.0d0) then
         write(*,*) 
     #        'POSSIBLE ERROR IN D03m_fin: SCALE MUSQ LESS THAN ZERO!!'
      endif

      prefactor = 1d0/(s*t-m2sq*m4sq)
      ms = -s/musq
      mt = -t/musq
      mm2sq = - m2sq/musq
      mm3sq = - m3sq/musq
      mm4sq = - m4sq/musq
      
      lnms = log(abs(ms)) - ipi*theta(-ms)
      lnmt = log(abs(mt)) - ipi*theta(-mt)
      lnmm2sq = log(abs(mm2sq)) - ipi*theta(-mm2sq)
      lnmm3sq = log(abs(mm3sq)) - ipi*theta(-mm3sq)
      lnmm4sq = log(abs(mm4sq)) - ipi*theta(-mm4sq)

     
      m2ot = 1-m2sq/t
      m4os = 1-m4sq/s
      

c     m2sq/t < 0 
      if (m2ot.gt.1d0) then
         li2m2ot = -dilog(1d0/m2ot) - log(m2ot)**2/2+pi2/3 
c     #        - ipi*log(m2ot)*im_part(m2sq,t)
     #        - ipi*log(m2ot)*sgnratio(m2sq,t)
      else
         li2m2ot = dilog(m2ot)
      endif

c     m4sq/s < 0
      if (m4os.gt.1d0) then
         li2m4os = -dilog(1d0/m4os) - log(m4os)**2/2+pi2/3 
c     #        - ipi*log(m4os)*im_part(m4sq,s)
     #        - ipi*log(m4os)*sgnratio(m4sq,s)
      else
         li2m4os = dilog(m4os)
      endif

      
      arg(1) = s
      arg(2) = t
      arg(3) = m2sq
      arg(4) = m4sq
      f3m = (s+t-m2sq-m4sq)/(s*t-m2sq*m4sq)

      
      if (f3m.gt.0d0) then
         sign = -1d0
      else
         sign = +1d0
      endif


      do i=1,4
         argom = 1-arg(i)*f3m            
         if (argom.gt.1d0) then
            li2arg(i) = -dilog(1d0/argom) - log(argom)**2/2+pi2/3 
     #           + ipi*log(argom)*sign
         else
            li2arg(i) = dilog(argom)
         endif
      enddo
      
      ris = lnms**2/2+lnmt**2/2-lnmm2sq**2/2-lnmm4sq**2/2
     #     +lnmm2sq*lnmm3sq
     #     -(lnmm3sq+lnmm2sq-lnmt)*lnmt+lnmm3sq*lnmm4sq
     #     -(lnmm4sq+lnmm3sq-lnms)*lnms      
      
      ris = ris + 2 * (li2m2ot + li2m4os + li2arg(1) + li2arg(2) 
     #     - li2arg(3) - li2arg(4))
        
      D03m_fin_DN = prefactor * ris
      end




c    ***********   D04m_fin(s,t,m1sq,m2sq,m3sq,m4sq,musq)   *************
c
c Scalar box with MASSLESS PROPAGATORS and with external kinematics: 
c
c
c                     k
c     p1 ->-----------<-------------<-- p4
c             |                 |
c             |                 |
c             |                 |  
c     p2 ->-------------------------<-- p3
c
c
c    s = (p1+p2)^2
c    t = (p2+p3)^2
c    p1^2 = m1sq <>0, p2^2 = m2sq <>0, p3^2 = m3sq <>0, p4^2 = m4sq <>0
c    musq = mu^2 = reference dimensional scale  (NOT used here, since 
c                 this box is finite 
c 
c  int d^dk/(2 pi)^d 1/(k^2)/(k+p1)^2/(k+p1+p2)^2/(k+p1+p2+p3)^2 =  
c          N_ep * D04m_fin(s,t,m1sq,m2sq,m3sq,m4sq,musq);
c          N_ep = i/(4 pi)^2 (4 pi)^ep Gamma(1+ep) (musq)^(-ep)
c

c	m2 ... arbitrary, but positive -> set m2 = 1d0


c
c
c     D0 with 4 nonzero external masses
c     and zero internal propagators      
c     use Eq.(41) from Denner, Nierste, Scharf		     
c

      function D04m_fin(s,t,m1sq,m2sq,m3sq,m4sq,musq)
      implicit none
      complex * 16 D04m_fin
      real * 8 s,t,m1sq,m2sq,m3sq,m4sq,musq

      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)

      integer k

      real * 8  dilog, eta3, theta, sgn      
      complex * 16 ln_mx(2),lnk12,lnk13,lnk14,lnk23,total

      real * 8 m2,k12,k13,k14,k23,k24,k34
      real * 8 a,b,c,d,sqrtdelta,x(2)
      real * 8 k34ok13,im_k34ok13,k24ok12,im_k24ok12
      real * 8 im_x(2),im_k34ok13_x(2),im_k24ok12_x(2),arg,logarg
      complex * 16 eta_k34ok13(2),eta_k24ok12(2)
      complex * 16 lik34ok13(2), logk34ok13(2)
      complex * 16 lik24ok12(2), logk24ok12(2)
      complex * 16 D04m_fin_real

      m2 = 1d0                  ! > 0 ALWAYS!
      
      k12 = -m1sq/m2
      k13 = -s/m2
      k14 = -m4sq/m2
      k23 = -m2sq/m2
      k24 = -t/m2
      k34 = -m3sq/m2
      

      a = k24*k34
      b = k13*k24+k12*k34-k14*k23
      c = k12*k13
      d = k23
      

      if ((b**2-4*a*c).lt.0d0) then
         write(*,*) 'ERROR: delta less than zero!!'
         write(*,*) 'RETURN ZERO'
c use BDK version of D04m, since in this case it's REAL!!!
c         D04m_fin = D04m_fin_real(s,t,m1sq,m2sq,m3sq,m4sq,musq)
         D04m_fin = 0d0
c         write(*,*) 's,t,m1sq,m2sq,m3sq,m4sq'
c         write(*,*) s,t,m1sq,m2sq,m3sq,m4sq
         return         
c         stop
      else
         sqrtdelta = sqrt(b**2-4*a*c)
      endif


      x(1) = (-b + sqrtdelta)/(2d0*a)
      x(2) = (-b - sqrtdelta)/(2d0*a)

c      write(*,*) 'x(1) x(2)',x(1),x(2)

      k34ok13 = k34/k13
      im_k34ok13 = (1d0-m3sq/s)/s
      
      k24ok12 = k24/k12
      im_k24ok12 = (1d0-t/m1sq)/m1sq

      im_x(1) = m2sq/sqrtdelta
      im_x(2) = -im_x(1)


      do k=1,2
         im_k34ok13_x(k) = im_k34ok13*x(k) + k34ok13*im_x(k)
         im_k24ok12_x(k) = im_k24ok12*x(k) + k24ok12*im_x(k)
         eta_k34ok13(k) = 
     #        2*ipi*eta3(-im_x(k),im_k34ok13,-im_k34ok13_x(k))
         eta_k24ok12(k) = 2*ipi*
     #        eta3(-im_x(k),im_k24ok12,-im_k24ok12_x(k))
         ln_mx(k) = log(abs(x(k))) - theta(x(k))*sgn(im_x(k))*ipi
      enddo

      lnk12 = log(abs(k12))-ipi*theta(-k12)
      lnk13 = log(abs(k13))-ipi*theta(-k13)
      lnk14 = log(abs(k14))-ipi*theta(-k14)
      lnk23 = log(abs(k23))-ipi*theta(-k23)

c      write(*,*) lnk12,lnk13,lnk14,lnk23

      do k=1,2
         arg = 1d0 + k34ok13*x(k)
         if (arg.gt.1d0) then
c     complex dilog
            logarg = log(arg)
            lik34ok13(k) = -dilog(1/arg) - 0.5d0 * logarg**2 + pi2/3
     #           + ipi * sgn(im_k34ok13_x(k)) * logarg
         else
            lik34ok13(k) = dilog(arg)
         endif
         logk34ok13(k) = log(abs(arg)) 
     #        + ipi*theta(-arg) * sgn(im_k34ok13_x(k))
c         write(*,*) lik34ok13(k),logk34ok13(k)
      enddo


      do k=1,2
         arg = 1d0 + k24ok12*x(k)
         if (arg.gt.1d0) then
c     complex dilog
            logarg = log(arg)
            lik24ok12(k) = -dilog(1/arg) - 0.5d0 * logarg**2 + pi2/3
     #           + ipi * sgn(im_k24ok12_x(k)) * logarg
         else
            lik24ok12(k) = dilog(arg)
         endif
         logk24ok12(k) = log(abs(arg)) 
     #        + ipi*theta(-arg) * sgn(im_k24ok12_x(k))
      enddo



c      write(*,*) '=========>', 
c     #     (lnk12+lnk13-lnk14-lnk23)*(-ln_mx(1)+ln_mx(2))

      total = 0d0
      do k=1,2
         total = total + (-1)**k*( - 0.5d0 *ln_mx(k)**2 
     #        - lik34ok13(k) - eta_k34ok13(k) * logk34ok13(k)
     #        - lik24ok12(k) - eta_k24ok12(k) * logk24ok12(k))
      enddo
      
      total = total + (lnk12+lnk13-lnk14-lnk23)*(-ln_mx(1)+ln_mx(2))

      D04m_fin = total/a/m2**2/(x(1)-x(2))
      
      end


      function E0(psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, 
     #     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234)
      implicit none
      complex * 16 E0
      real * 8 psq, qsq, lsq, tsq, pq, pl, pt, ql, qt, lt
      real * 8 q1s, q2s, ppp, pq1, ppq1, pq2, ppq2, q1q2, pps
      real * 8 det, coeff(1:5)
      complex * 16 D0_2345,D0_1345,D0_1245,D0_1235,D0_1234, J(1:5)
      integer i
      real * 8 tiny
      logical lpr
      common /e0print/lpr

      tiny = 1d-4

      if (abs(psq).gt.tiny) then
         write(*,*) 'p^2 =',psq
         write(*,*) 'E0 computed only with p^2 and pp^2 = 0'
         stop
      endif

      q1s  = qsq
      q2s  = lsq
      ppp  = psq+pq+pl+pt
      pq1  = pq
      ppq1 = pq + qsq+ql+qt
      pq2  = pl
      ppq2 = pl + ql + lsq + lt
      q1q2 = ql
      pps = lsq+2*lt+2*ql+2*pl+tsq+2*qt+2*pt+qsq+2*pq+psq
      if (abs(pps).gt.tiny) then
         write(*,*) 'pp^2 =',pps
         write(*,*) 'E0 computed only with p^2 and pp^2 = 0'
         lpr = .true.
c         stop
      endif


      det = -8*ppp*(4*q1s*pq2**2*ppq1+q1s**2*q2s*ppp-4*pq1*pq2*q1s*ppq2-
     #4*pq1*pq2*ppq1*q2s+4*q1s*pq2*q1q2*ppp+4*q1s*pq2*q2s*ppp-4*pq1*q2s*
     #q1q2*ppp+4*q1s*q2s*pq1*ppp-8*pq1*pq2*ppq1*q1q2-8*pq1*q1q2*pq2*ppp+
     #4*q1s*pq1*q1q2*ppq2+2*q1s*pq1*q2s*ppq2-8*pq1*q2s*ppq1*q1q2+2*q2s*p
     #q2*q1s*ppq1+4*q1s*pq2*ppq1*q1q2+2*q1s*q1q2*q2s*ppp-2*q1s**2*pq2*pp
     #q2+4*pq1**2*q2s*ppq2+8*pq1**2*q1q2*ppq2+4*q1s*pq2**2*ppp-8*q1q2**2
     #*pq1*ppp+4*pq1**2*q2s*ppp+q1s*q2s**2*ppp-8*q1q2**2*pq1*ppq1-2*pq1*
     #q2s**2*ppq1)

      
c      write(*,*)  'DETERMINANTE in E0',det

      coeff(1)=(-16*pq1**2*q2s*ppp-8*q1s*pq1*ppq2**2+16*q1q2**2*ppq1*ppp
     #-8*pq1**2*q2s*ppq2-16*pq1**2*q1q2*ppq2-16*q1q2*pq2*ppq1**2-8*q1s*p
     #q2**2*ppq1-16*q1s*pq2**2*ppp+16*q1q2**2*pq1*ppp+16*q1q2**2*ppp**2-
     #8*q2s*pq2*ppq1**2-16*q1s*q2s*ppp**2+8*pq1*pq2*q1s*ppq2+8*pq1*pq2*p
     #pq1*q2s-8*q1s*pq2*q1q2*ppp-8*q1s*pq2*q2s*ppp+8*pq1*q2s*q1q2*ppp-8*
     #q1s*q2s*pq1*ppp+8*q1s*pq2*ppq1*ppq2-8*q1s*q1q2*ppq2*ppp-8*q1s*q2s*
     #ppq1*ppp-8*q1s*q2s*ppq2*ppp+8*pq1*q2s*ppq1*ppq2+8*q1q2*q2s*ppq1*pp
     #p+16*pq1*pq2*ppq1*q1q2+16*q1s*pq2*ppq2*ppp+16*pq1*q2s*ppq1*ppp+32*
     #pq1*q1q2*pq2*ppp-16*pq1*q1q2*ppq2*ppp-16*q1q2*pq2*ppq1*ppp+16*pq1*
     #q1q2*ppq1*ppq2)/det


      coeff(2)=(16*pq1*q2s*ppp**2+8*q2s*pq2*ppq1*ppp+16*pq2**2*ppq1**2-1
     #6*pq1*pq2*ppq2*ppp-16*q1q2*pq2*ppp**2+16*pq1**2*ppq2**2-32*pq1*pq2
     #*ppq1*ppq2+16*pq1*q2s*ppq1*ppp-8*q1q2*q2s*ppp**2+8*q1s*q2s*ppp**2+
     #16*pq2**2*ppq1*ppp+8*pq1*q2s*ppq2*ppp-16*q1q2**2*ppp**2+8*q1s*pq1*
     #ppq2**2-16*q1q2**2*ppq1*ppp+8*q2s*pq2*ppq1**2-8*q1s*pq2*ppq1*ppq2-
     #16*pq1*q1q2*ppq1*ppq2+8*q1s*q1q2*ppq2*ppp+16*q1q2*pq2*ppq1**2+8*q1
     #s*q2s*ppq1*ppp+8*q1s*q2s*ppq2*ppp-8*pq1*q2s*ppq1*ppq2-8*q1q2*q2s*p
     #pq1*ppp)/det


      coeff(3)=8*ppp*(-2*pq1*q1q2*ppp+2*q1s*pq2*ppq2-2*pq1*q2s*ppq1-4*pq
     #1*q1q2*ppq1-2*pq1*q1q2*ppq2-2*pq1*pq2*ppq1+q1s*q1q2*ppp+2*pq1**2*p
     #pq2+2*q1s*pq2*ppp+q1s*pq1*ppq2+q1s*pq2*ppq1-pq1*q2s*ppq2-q2s*pq2*p
     #pq1+2*pq1*pq2*ppq2+2*q1q2**2*ppp+q1q2*q2s*ppp-2*pq2**2*ppq1-2*pq1*
     #q2s*ppp-2*q1q2*pq2*ppq1+2*q1q2*pq2*ppp)/det


      coeff(4)=-8*ppp*(-2*pq1*q1q2*ppp-2*pq1*q2s*ppq1-4*pq1*q1q2*ppq1-2*
     #pq1*pq2*ppq1+q1s*q2s*ppp+q1s*q1q2*ppp+2*pq1**2*ppq2+2*q1s*pq2*ppp+
     #q1s*pq1*ppq2+q1s*pq2*ppq1)/det


      coeff(5)=(16*pq1**2*q1q2*ppq2+8*pq1**2*q2s*ppq2+16*q1s*pq2**2*ppp-
     #8*pq1*pq2*q1s*ppq2-16*pq1*pq2*ppq1*q1q2-8*pq1*pq2*ppq1*q2s+8*q1s*p
     #q2*q1q2*ppp+8*q1s*pq2*q2s*ppp-8*pq1*q2s*q1q2*ppp-32*pq1*q1q2*pq2*p
     #pp-16*q1q2**2*pq1*ppp+8*q1s*pq2**2*ppq1+16*pq1**2*q2s*ppp+8*q1s*q2
     #s*pq1*ppp)/det


c D0(q1,q2,pp-p-q1-q2) 
      J(1) = D0_2345
c D0(p+q1,q2,pp-p-q1-q2)
      J(2) = D0_1345
c D0(p,q1+q2,pp-p-q1-q2)
      J(3) = D0_1245
c D0(p,q1,pp-p-q1)
      J(4) = D0_1235
c D0(p,q1,q2)
      J(5) = D0_1234
      
      E0 = 0d0
      do i=1,5
         E0 = E0 + J(i)*coeff(i)
      enddo
      end



CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


c this function compute the imaginary part of a/b, where:
c (a+I*ep)/(b+I*ep) = a/b + I*ep*a/b*(1/a-1/b)
c  and return the sign of this imaginary part
      function sgnratio(a,b)
      implicit none
      real * 8 a,b,sgnratio
      if ((1d0/b-a/b**2).gt.0d0) then
         sgnratio = +1d0
      else
         sgnratio = -1d0          
      endif

      end


c     imaginary part of a/b  
c     IF a/b > 0 then returns 0
c     IF a/b < 0 then
c        if a < 0 then return +1
c        else return -1
      function im_part(a,b)
      implicit none
      real * 8 a,b,im_part
      if ((a/b).gt.0d0) then
         im_part = 0d0
      else
         if (a.lt.0d0) then
            im_part = +1d0
         else
            im_part = -1d0
         endif
      endif
      
      end


c     compute  Li(1-a/b) 
c     IF a/b < 0 then, it attaches a POSITIVE +I*ep to a or b, 
c     according to which of them is negative
      function dilog_1_m_aob(a,b)
      implicit none
      complex * 16 dilog_1_m_aob
      real * 8 a,b
      real * 8 arg
      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)
      real * 8  dilog, sign

      arg = 1d0-a/b
      if (arg.gt.1d0) then
         if (a.lt.0d0) then
            sign = +1d0
         else
            sign = -1d0
         endif
         dilog_1_m_aob = -dilog(1d0/arg) - log(arg)**2/2+pi2/3 
     #        - ipi*log(arg)*sign
      else
         dilog_1_m_aob = dilog(arg)
      endif
      
      end


      function eta3(im_a,im_b,im_ab)
      implicit none
      real * 8 eta3,im_a,im_b,im_ab
      real * 8 theta
      eta3 = theta(-im_a)*theta(-im_b)*theta(im_ab)
     #     - theta(im_a)*theta(im_b)*theta(-im_ab)
      end


c     Parabolic interpolation thru the points (x1,y1), (x2,y2), (x3,y3)
c     It return the ordinate of point with abscissa x
      function interp_parab(x1,y1,x2,y2,x3,y3,x)
      implicit none
      real * 8 x1,y1,x2,y2,x3,y3,x,interp_parab
      real * 8 a,b,c
      a = (x1*y3-x3*y1-x2*y3-x1*y2+x2*y1+x3*y2)/(x2-x3)/(-x3+x1)/(x1-x2)
      b = -(x1**2*y3-x1**2*y2-y1*x3**2+y2*x3**2-x2**2*y3+y1*x2**2)/(x2-x
     #3)/(-x3+x1)/(x1-x2)
      c = (-y1*x2*x3**2+x2**2*x3*y1+x1**2*x2*y3-x1**2*x3*y2+y2*x1*x3**2-
     #x2**2*x1*y3)/(x2-x3)/(-x3+x1)/(x1-x2)
      interp_parab = a*x**2 + b*x + c
      end

c determinant of
c     q1(0)   q1(1)  q1(2)  q1(3)
c     q2(0)   q2(1)  ....
c     ...
c     q4(0)   ...           q4(3)
      function det_planar(q1,q2,q3,q4)
      implicit none
      real * 8 det_planar,q1(0:3),q2(0:3),q3(0:3),q4(0:3)
      real * 8 s1
      s1 = q1(0)*q2(1)*q3(2)*q4(3)-q1(0)*q2(1)*q3(3)*q4(2)-q1(0)*q3(1)*q
     #2(2)*q4(3)+q1(0)*q3(1)*q2(3)*q4(2)+q1(0)*q4(1)*q2(2)*q3(3)-q1(0)*q
     #4(1)*q2(3)*q3(2)-q2(0)*q1(1)*q3(2)*q4(3)+q2(0)*q1(1)*q3(3)*q4(2)+q
     #2(0)*q3(1)*q1(2)*q4(3)-q2(0)*q3(1)*q1(3)*q4(2)-q2(0)*q4(1)*q1(2)*q
     #3(3)+q2(0)*q4(1)*q1(3)*q3(2)
      det_planar = s1+q3(0)*q1(1)*q2(2)*q4(3)-q3(0)*q1(1)*q2(3)*q4(2)-q3
     #(0)*q2(1)*q1(2)*q4(3)+q3(0)*q2(1)*q1(3)*q4(2)+q3(0)*q4(1)*q1(2)*q2
     #(3)-q3(0)*q4(1)*q1(3)*q2(2)-q4(0)*q1(1)*q2(2)*q3(3)+q4(0)*q1(1)*q2
     #(3)*q3(2)+q4(0)*q2(1)*q1(2)*q3(3)-q4(0)*q2(1)*q1(3)*q3(2)-q4(0)*q3
     #(1)*q1(2)*q2(3)+q4(0)*q3(1)*q1(3)*q2(2)
      end

c     expression that appears in the denominator of the scalar pentagon, 
c     once it's written in terms of the 5 boxes
      function E0_determ(q1,q2,q3,q4)
      implicit none
      real * 8 E0_determ,q1(0:3),q2(0:3),q3(0:3),q4(0:3)
      real * 8 s1,s2,determin,q1s,q2s,q3s,q4s,q12,q13,q14,q23,q24,q34
      real * 8 dotrr

      q1s = dotrr(q1,q1)
      q2s = dotrr(q2,q2)
      q3s = dotrr(q3,q3)
      q4s = dotrr(q4,q4)
      q12 = dotrr(q1,q2)
      q13 = dotrr(q1,q3)
      q14 = dotrr(q1,q4)
      q23 = dotrr(q2,q3)
      q24 = dotrr(q2,q4)
      q34 = dotrr(q3,q4)

      s2 = -8*q2s*q4s**2*q13**2-8*q1s*q3s**2*q24**2-8*q3s*q14**2*q2s**2-
     #32*q4s*q12**2*q23**2-32*q2s*q34**2*q13**2-8*q2s*q34**2*q1s**2-8*q1
     #s*q4s**2*q23**2-8*q4s*q1s**2*q23**2-8*q3s*q4s**2*q12**2-8*q4s*q2s*
     #*2*q13**2-32*q1s*q34**2*q23**2-32*q3s*q12**2*q24**2+16*q1s*q2s*q3s
     #*q4s*q14-8*q2s*q3s**2*q14**2-8*q3s*q1s**2*q24**2-8*q3s**2*q4s*q12*
     #*2-8*q1s*q34**2*q2s**2+64*q23*q34**2*q12**2+8*q2s*q3s*q4s*q1s**2+3
     #2*q1s*q23*q34**2*q12-32*q1s*q2s*q34**2*q13+8*q1s*q2s*q3s*q4s**2+8*
     #q1s*q3s*q4s*q2s**2
      s1 = 8*q1s*q2s*q3s**2*q4s-32*q1s*q4s*q34*q23**2-16*q1s*q2s*q34**2*
     #q12-32*q1s*q2s*q34**2*q23+16*q23*q34*q1s**2*q24-16*q3s*q4s*q34*q12
     #**2-32*q23*q3s*q4s*q12**2-32*q24*q3s*q4s*q12**2-32*q24*q3s*q34*q12
     #**2+64*q23*q34*q12**2*q24-32*q1s*q4s*q12*q23**2-32*q1s*q34*q14*q23
     #**2-16*q1s*q4s*q14*q23**2-32*q1s*q3s*q12*q24**2-16*q1s*q3s*q13*q24
     #**2+64*q12*q23*q34**2*q13-32*q12*q2s*q34**2*q13+16*q12*q24*q3s**2*
     #q14+16*q12*q23*q4s**2*q13-64*q12*q34*q14*q23**2-32*q12*q4s*q14*q23
     #**2-32*q12*q3s*q13*q24**2+32*q23*q4s*q34*q12**2-32*q2s*q4s*q34*q13
     #**2+s2
      s2 = s1-32*q2s*q34*q13**2*q24-16*q2s*q4s*q13**2*q24+16*q13*q34*q14
     #*q2s**2-16*q2s*q3s*q14**2*q23+16*q1s*q2s*q3s*q4s*q12+16*q1s*q2s*q3
     #s*q4s*q34+16*q1s*q2s*q3s*q4s*q24+16*q1s*q2s*q3s*q4s*q13+16*q1s*q2s
     #*q3s*q4s*q23-16*q1s*q2s*q4s*q34*q13-16*q1s*q2s*q4s*q34*q23+16*q1s*
     #q2s*q34*q3s*q14+16*q1s*q2s*q34*q3s*q24+16*q1s*q23*q3s*q4s*q24+16*q
     #1s*q23*q4s*q34*q12+32*q1s*q23*q34*q3s*q24-16*q1s*q23*q3s*q4s*q12-1
     #6*q1s*q24*q3s*q4s*q12-16*q1s*q24*q3s*q34*q12-16*q1s*q2s*q34*q13*q2
     #4-16*q1s*q2s*q34*q14*q23+16*q1s*q23*q4s*q2s*q13+64*q1s*q23*q34*q12
     #*q24
      determin = 32*q1s*q23*q34*q13*q24+16*q1s*q23*q4s*q13*q24+16*q1s*q2
     #4*q3s*q14*q2s+16*q1s*q24*q3s*q14*q23+16*q12*q2s*q3s*q4s*q13-16*q12
     #*q2s*q4s*q34*q13+16*q12*q2s*q34*q3s*q14-16*q12*q23*q3s*q4s*q14+64*
     #q12*q23*q4s*q34*q13-32*q12*q23*q34*q3s*q14-32*q12*q24*q3s*q34*q13-
     #16*q12*q24*q3s*q4s*q13-32*q12*q2s*q34*q13*q24-32*q12*q2s*q34*q14*q
     #23+32*q12*q23*q4s*q2s*q13+64*q12*q23*q34*q13*q24+32*q12*q23*q4s*q1
     #3*q24+32*q12*q24*q3s*q14*q2s+32*q12*q24*q3s*q14*q23+16*q12*q2s*q3s
     #*q4s*q14+16*q13*q2s*q3s*q4s*q14+32*q13*q2s*q34*q3s*q14+32*q13*q2s*
     #q34*q14*q23+16*q13*q2s*q4s*q14*q23+16*q13*q24*q3s*q14*q2s+s2
      E0_determ = determin
      end

      subroutine print_Cij(C0,Cij)
      implicit none
      complex * 16 C0,Cij(2,4)
      integer indx_C(2),indx_D(3),indx_E(4)
      common/indexes/indx_C,indx_D,indx_E
      integer i1,i2
      write(*,*) 'C0 =',C0
      do i1=1,2
         do i2=1,indx_C(i1)
            write(*,*) 'Cij(',i1,i2,') =',Cij(i1,i2)
         enddo
      enddo
      end


      subroutine print_Dij(D0,Dij)
      implicit none
      complex * 16 D0,Dij(3,13)
      integer indx_C(2),indx_D(3),indx_E(4)
      common/indexes/indx_C,indx_D,indx_E
      integer i1,i2
      write(*,*) 'D0 =',D0
      do i1=1,3
         do i2=1,indx_D(i1)
            write(*,*) 'Dij(',i1,i2,') =',Dij(i1,i2)
         enddo
      enddo
      end

      subroutine print_Eij(E0,Eij)
      implicit none
      complex * 16 E0,Eij(4,46)
      integer indx_C(2),indx_D(3),indx_E(4)
      common/indexes/indx_C,indx_D,indx_E
      integer i1,i2
      write(*,*) 'E0 =',E0
      do i1=1,4
         do i2=1,indx_E(i1)
            write(*,*) 'Eij(',i1,i2,') =',Eij(i1,i2)
         enddo
      enddo
      end


      function rndsign()
      implicit none
      integer rndsign
      integer num,num2     
      COMMON/SEED/num,num2
      real * 8 random2
      if (random2(NUM,num2).lt.0.5d0) then
         rndsign = -1
      else
         rndsign = 1
      endif
      end







c  RANDOM B0, C0, D0 and E0 functions  (for debug purpose only)
      function B0(psq,musq)
      implicit none
      complex * 16 B0
      real * 8 psq,musq
c      complex * 16 crand_num      
      if (psq.eq.0d0) then
         B0 = 0d0
      else
         B0 = psq + 1d0/psq*(0,1)
      endif
c      B0 = 0d0
      B0 = (1d0,3d0)
      end

      function C0(psq,qsq,pq_sq,musq)
      implicit none
      complex * 16 C0
      real * 8 psq,qsq,pq_sq,musq
c      complex * 16 crand_num      
c      C0 = psq+qsq+pq_sq*(0,1)
c      C0 = 0d0
      C0 = (5d0,7d0)
      end


      function D0(psq,qsq,lsq,pq,pl,ql)
      implicit none
      complex * 16 D0
      real * 8 psq,qsq,lsq,pq,pl,ql
c      complex * 16 crand_num      
c      D0 = psq+qsq+lsq+(pq+pl+ql)*(0,1)
c      D0 = 0d0
      D0 = (11d0,13d0)
      end


c      function E0(psq, qsq, lsq, tsq,
c     #     pq, pl, pt, ql, qt, lt)
c      implicit none
c      complex * 16 E0
c      real * 8 psq, qsq, lsq, tsq,
c     #     pq, pl, pt, ql, qt, lt
c      complex * 16 crand_num      
c      E0 = psq*crand_num() + qsq*crand_num() + lsq*crand_num()
c     #     + tsq*crand_num() + pq*crand_num() + pl*crand_num()
c     #     + pt*crand_num() + ql*crand_num() + qt*crand_num() 
c     #     + lt*crand_num()
c      E0 = (17d0,19d0)
c      end


c      function E0(psq, qsq, lsq, tsq,
c     #     pq, pl, pt, ql, qt, lt, 
c     #     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234)
c      implicit none
c      complex * 16 E0      
c      real * 8 psq, qsq, lsq, tsq, pq, pl, pt, ql, qt, lt
c      real * 8 q1s, q2s, ppp, pq1, ppq1, pq2, ppq2, q1q2, pps
c      real * 8 det, coeff(1:5)
c      complex * 16 D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
c      E0 = (15d0,17d0)
c      end
      

c version of D04m_fin taken from Bern-Dixon and Kosower.
c Analytic continuation is NOT traightforward!!
c
c    ***********   D04m_fin(s,t,m1sq,m2sq,m3sq,m4sq,musq)   *************
c
c Scalar box with MASSLESS PROPAGATORS and with external kinematics: 
c
c                     
c     p2 ->-------------------------<-- p3
c             |                 |
c             |                 |
c             |       k         |  
c     p1 ->-----------<-------------<-- p4
c
c
c    s = (p1+p2)^2
c    t = (p2+p3)^2
c    p1^2 = m1sq <>0, p2^2 = m2sq <>0, p3^2 = m3sq <>0, p4^2 = m4sq <>0
c    musq = mu^2 = reference dimensional scale  (NOT used here, since 
c                 this box is finite 
c 
c  int d^dk/(2 pi)^d 1/(k^2)/(k+p1)^2/(k+p1+p2)^2/(k+p1+p2+p3)^2 =  
c          N_ep * D04m_fin(s,t,m1sq,m2sq,m3sq,m4sq,musq);
c          N_ep = i/(4 pi)^2 (4 pi)^ep Gamma(1+ep) (musq)^(-ep)
c



      function D04m_fin_wrong(s,t,m1sq,m2sq,m3sq,m4sq,musq)
      implicit none
      complex * 16 D04m_fin_wrong
      real * 8 s,t,m1sq,m2sq,m3sq,m4sq,musq

      real * 8 arg1,arg2,arg3,arg4,arg5,l1,l2,argr,r,l1tmp

      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)

      real * 8  dilog
            
c      if (musq.lt.0d0) then
c         write(*,*) 'POSSIBLE ERROR: SCALE MUSQ LESS THAN ZERO!!'
c      endif

c     the formula contains log(l1) so, to simplify the treatment of 
c     analytic continuation, we relabel the external invariants so 
c     that l1 > 0
c     
      
      l1 = m1sq*m3sq/s/t
      l2 = m2sq*m4sq/s/t
      
      if (l1.lt.0d0) then
c     exchange s and t, and relabel the external masses starting from m2, 
c     so that:
c     s <-> t,  m2sq -> m1sq,  m3sq -> m2sq,  m4sq -> m3sq,  m1sq -> m4sq 
         l1tmp = l1
         l1 = l2
         l2 = l1tmp
      endif

      argr = 1-2*l1-2*l2+(l1-l2)**2

      if (argr.le.0d0) then
         write(*,*) 'ERROR: argr < 0 in D04m_fin_wrong'
         stop
      endif

      r = sqrt(argr)
      arg1 = (1-l1+l2+r)/2
      arg2 = (1-l1+l2-r)/2
      arg3 = -(1-l1-l2-r)/(2*l1)
      arg4 = -(1-l1-l2+r)/(2*l1)
      arg5 = (1+l1-l2+r)/(1+l1-l2-r)

      write(*,*) 'r,l1,l2,arg1,arg2,arg3,arg4,arg5'
      write(*,*) r,l1,l2,arg1,arg2,arg3,arg4,arg5
      
      if ((l1.lt.0d0).or.(arg1.gt.1d0).or.(arg2.gt.1d0).or.
     #     (arg3.gt.1d0).or.(arg4.gt.1d0).or.(arg5.lt.0d0)) then
         write(*,*) 'ERROR in one of the arguments in D04m_fin_wrong'
         write(*,*) 's,t,m1sq,m2sq,m3sq,m4sq'
         write(*,*) s,t,m1sq,m2sq,m3sq,m4sq
         write(*,*) 'l1,arg1,arg2,arg3,arg4,arg5'
         write(*,*) l1,arg1,arg2,arg3,arg4,arg5
         stop
      endif
      
      D04m_fin_wrong = (dilog(arg1)-dilog(arg2)+dilog(arg3)-dilog(arg4)
     #     + log(l1/l2**2)*log(arg5)/2)/r/s/t

      end


      subroutine test_anal_cont(s,t,m2sq,m3sq,m4sq)
      implicit none
      real * 8 s,t,m2sq,m3sq,m4sq
      real * 8 ms,mt,mm2sq,mm3sq,mm4sq,sot
      complex * 16 lnms,lnmt,lnmm2sq,lnmm3sq,lnmm4sq,lnsot
      complex * 16 li2arg1,li2arg2,li2arg3
      real * 8 arg1,arg2,arg3
      complex * 16 ipi
      parameter (ipi=(0d0,3.14159265358979323846264338328d0))
      real * 8 pi,pi2
      parameter (pi=3.14159265358979323846264338328d0,
     #     pi2 = 9.86960440108935861883449099988d0)

      complex * 16 ris1,ris2
      real * 8 prefactor, theta, dilog, sgn, im_part, f3m


     
      arg1 = 1-m2sq/s
      arg2 = 1-m4sq/t
      arg3 = 1-m2sq*m4sq/s/t

      lnsot = log(s/t)
      

c     m2sq/s < 0 
      if (arg1.gt.1d0) then
         li2arg1 = -dilog(1d0/arg1) - log(arg1)**2/2+pi2/3 
     #        - ipi*log(arg1)*im_part(m2sq,s)
      else
         li2arg1 = dilog(arg1)
      endif

c     m4sq/t < 0
      if (arg2.gt.1d0) then
         li2arg2 = -dilog(1d0/arg2) - log(arg2)**2/2+pi2/3 
     #        - ipi*log(arg2)*im_part(m4sq,t)
      else
         li2arg2 = dilog(arg2)
      endif

c     m2sq*m4sq/(s*t)  < 0
      if (arg3.gt.1d0) then
         li2arg3 = -dilog(1d0/arg3) - log(arg3)**2/2+pi2/3 
     #        - ipi*log(arg3)*im_part(m2sq*m4sq,s*t)
      else
         li2arg3 = dilog(arg3)
      endif
      
      ris1 =  - 2*li2arg1 - 2*li2arg2 + 2*li2arg3 
     #     - lnsot**2
        
      f3m = (s+t-m2sq-m4sq)/(s*t-m2sq*m4sq)

      ris2 = 2*(dilog(1-m2sq/t) + dilog(1-m4sq/s) 
     #     + dilog(1-s*f3m) + dilog(1-t*f3m)  
     #     - dilog(1-m2sq*f3m) - dilog(1-m4sq*f3m))

      write(*,*) ris1
      write(*,*) ris2

      end





      subroutine test_D0
      implicit none

      complex * 16 D0t,D04m_fin,D04,ris1,ris2,ris3,ris4,
     #     D03m_fin,D0fin, D02m_fin,
     #     D03m_fin_DN, D03m_fin_BDK
      real * 8 invs,invt,m1sq,m2sq,m3sq,m4sq,m
      
      real * 8 energ,musq,ratioimag,ratioreal

      integer num,num2     
      COMMON/SEED/num,num2
      real * 8 random2
      integer rndsign
      integer i
      
      energ = 100
      m = 0.0001d0

      do i=1,1000
         invs = energ*random2(NUM,num2)*rndsign()
         invt = energ*random2(NUM,num2)*rndsign()
         m1sq = energ*random2(NUM,num2)*rndsign()
         m2sq = energ*random2(NUM,num2)*rndsign()
         m3sq = energ*random2(NUM,num2)*rndsign()
         m4sq = energ*random2(NUM,num2)*rndsign()
         musq = energ*random2(NUM,num2)  ! > 0 always


c     TEST D04m
         goto 727
         ris1 = D04m_fin(invs,invt,m1sq,m2sq,m3sq,m4sq,musq)
         ris2 = D0t(m1sq,m2sq,m3sq,m4sq,invt,invs)
         ris3 = D04(m1sq,m2sq,m3sq,m4sq,invs,invt,0d0,m,m,m)
         write(*,*) ris1
         write(*,*) ris2
         write(*,*) ris3
 727     continue


c     TEST D03m
c         goto 456
         ris1 = D03m_fin_DN(invs,invt,m2sq,m3sq,m4sq,musq)
         ris2 = D03m_fin_BDK(invs,invt,m2sq,m3sq,m4sq,musq)

c         ris1 = D02m_fin(invs,invt,m3sq,m4sq,musq)
c         ris2 = D0fin(invs,invt,m3sq,m4sq,musq)
c         ris3 = D0t(0d0,0d0,m3sq,m4sq,invt,invs)


c         write(*,*) 'invs,invt,m2sq,m3sq,m4sq,musq'
c         write(*,*) invs,invt,m2sq,m3sq,m4sq,musq
c         write(*,*) ris1

c         write(*,*) ris1
c         write(*,*) ris2
c         write(*,*) ris3

         if ((abs(dimag(ris1)).lt.1d-14).and.
     #        (abs(dimag(ris2)).lt.1d-14)) then
            ratioimag = 1d0
         else
            ratioimag = dimag(ris1)/dimag(ris2)
         endif
         
         ratioreal = dreal(ris1)/dreal(ris2)
         
         if ((abs(ratioimag-1d0).gt.1d-10).or.
     #        (abs(ratioreal-1d0).gt.1d-10)) then
            write(*,*) '=========>',ratioreal,ratioimag
            write(*,*) ris1
            write(*,*) ris2
            write(*,*) ''
         endif

 456     continue


      enddo


      end


c generate n ligh-like momenta
c  p1 + p2 = p3 + p4 + ... + pn
      subroutine fill_momenta(n,p)
      implicit none
      integer n
      real * 8 softness
      real * 8 p(0:3,n)
      
      real * 8 E(n),th(n),ph(n),Etot
      integer mu
      real * 8 random2
      integer num,num2     
      COMMON/SEED/num,num2
      double precision pi
      parameter (pi=3.141592653589793238462643383279502884197D0)
      real * 8 Emax(n) 
      integer i
      real * 8 dotrr
      
      do i=3,n
         Emax(i) = 2000.d0         
         E(i)  = Emax(i)*random2(NUM,num2)
         th(i) = pi*random2(NUM,num2)
         ph(i) = 2*pi*random2(NUM,num2)
      enddo


      th(1) = 0d0
      ph(1) = 0d0

      th(2) = pi
      ph(2) = 0d0

      do i=3,n
         p(0,i) = E(i)
         p(1,i) = E(i)*sin(th(i))*cos(ph(i))
         p(2,i) = E(i)*sin(th(i))*sin(ph(i))
         p(3,i) = E(i)*cos(th(i))
      enddo

c  compute px, py and pz of the (n-1)th vector: Sum P_vec_i = 0            
      do mu=1,3
         p(mu,n-1) = 0d0
         do i=3,n
            if (i.ne.n-1) then 
               p(mu,n-1) = p(mu,n-1) - p(mu,i)
            endif
         enddo
      enddo

C p(n-1)^2=0 ==>
      p(0,n-1) = sqrt(p(1,n-1)**2 + p(2,n-1)**2 + p(3,n-1)**2)

           
c  conservation p1 + p2 = p3 + p4 + ... + pn
      Etot = 0d0
      do i=3,n
         Etot = Etot + p(0,i)
      enddo
      
      E(1) = Etot/2
      E(2) = Etot/2    
      
      p(0,1) = E(1)
      p(1,1) = 0.d0
      p(2,1) = 0.d0
      p(3,1) = E(1)

      p(0,2) = E(2)
      p(1,2) = 0.d0
      p(2,2) = 0.d0
      p(3,2) = -E(2)


      end


      
      function born(
c     bra(p'), ket(p) and helicity 
     #     brapp,ketp,hel,
c     momenta and squared renormalization scale
     #     p,q,l,t,pp,
c     input currents 
     #     jmu1,jmu2,jmu3)

      implicit none
      complex * 16 born
      complex * 16 brapp(2,-1:1),ketp(2,-1:1)
      integer hel
      real * 8 p(0:3),pp(0:3),q(0:3),l(0:3),t(0:3)
      complex * 16 jmu1(0:3),jmu2(0:3),jmu3(0:3)

      complex * 16 tree

      real * 8 psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt, ppp, ppq, ppl

      real * 8 dotrr

      psq = dotrr(p,p)
      qsq = dotrr(q,q)
      lsq = dotrr(l,l)
      tsq = dotrr(t,t)

      pq = dotrr(p,q)
      pl = dotrr(p,l)
      ql = dotrr(q,l)
      qt = dotrr(q,t)
      lt = dotrr(l,t)

      pt = dotrr(p,t)

      ppp = dotrr(p,pp)
      ppq = dotrr(pp,q)
      ppl = dotrr(pp,l)
      
      call compute_Jprod_gammaprod(
     #     brapp,ketp,hel,
     #     p,q,l,t,pp,
     #     jmu1,jmu2,jmu3)
      
      born = tree(qsq,lsq,pq,pl,ppp,ppq,ppl,ql)
      
      end


      function tree(q1s,q2s,pq1,pq2,ppp,ppq1,ppq2,q1q2)
      implicit none
      complex * 16 tree
      real * 8 q1s,q2s,pq1,pq2,ppp,ppq1,ppq2,q1q2
      complex * 16 dpmu1,dpmu2,dpmu3,dqmu1,dqmu2,dqmu3
      complex * 16 dlmu1,dlmu2,dlmu3,dppmu1,dppmu2,dppmu3
      complex * 16 dmu1mu2,dmu1mu3,dmu2mu3
      common/dotp_pol_vec/dpmu1,dpmu2,dpmu3,dqmu1,dqmu2,dqmu3,
     #     dlmu1,dlmu2,dlmu3,dppmu1,dppmu2,dppmu3,
     #     dmu1mu2,dmu1mu3,dmu2mu3
      complex * 16 gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q
      complex * 16 gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2
      complex * 16 gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3
      complex * 16 gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
      common/gammaprods/gp_mu3_l_mu2_q_mu1,gp_l,gp_mu2,gp_mu3_l_q,
     #     gp_mu2_q_mu1,gp_mu3_mu2_q,gp_q,gp_mu3_l_mu2,
     #     gp_l_mu2_q,gp_mu3_mu2_mu1,gp_mu3_l_mu1,gp_mu3,
     #     gp_mu1,gp_l_q_mu1,gp_mu3_q_mu1,gp_l_mu2_mu1
      complex * 16 t1,t2,t3,t4,t5,t6,t7,t8,t9,t10
      complex * 16 t11,t12,t13,t14,t15,t16,t17
      t2 = 2*pq1
      t5 = 1/(q1s+2*q1q2+t2+q2s+2*pq2)
      t7 = dqmu2+dpmu2
      t10 = 1/(q1s+t2)
      t11 = t5*t10
      tree = -t5*gp_mu3_mu2_mu1+4*dpmu1*t7*t11*gp_mu3+t11*gp_mu3_l_mu2_q
     #_mu1+2*t7*t5*t10*gp_mu3_q_mu1+2*dpmu1*t5*t10*gp_mu3_l_mu2
      end

c     returns the azimuthal angle between the vector and the y axis
c     in the yz plane
      function azimuth_vec_y_yz(p)
      implicit none
      real * 8 azimuth_vec_y_yz,p(0:3)
      real * 8 phi
      real * 8 pi
      parameter (pi=3.141592653589793238462643383279502884197D0)      
      real * 8 tiny
      parameter (tiny = 1d-13)
      
      if ((abs(p(2)).lt.tiny).and.(abs(p(3)).lt.tiny)) then
         azimuth_vec_y_yz = 0d0
         return
      endif

      if (abs(p(2)).lt.tiny) then
         if (p(3).gt.0d0) then
            phi = pi/2
         else
            phi = -pi/2
         endif
      else
         phi = atan(p(3)/p(2))
         if (p(2).lt.0d0) then
            if (phi.gt.0d0) then               
               phi = phi - pi
            else
               phi = phi + pi
            endif
         endif         
      endif
      azimuth_vec_y_yz = phi
      end



