c*************************************************************************
      FUNCTION Amplitude(rand, p, x, v, nlo)
c*************************************************************************
c     This is the manager of all matrixelement routines and the (only) 
c     interface of those to the main program.  Depending on the process ID
c     the right matrix element is called with the correct calling 
c     convention, which might well be individually different.
c     Once many processes are included in the program, this subroutine
c     will be quite complex, but it should actually be not much more than a
c     big select-case statement.
c
c*************************************************************************
c     INPUT
c     rand      : Array with random numbers driven by monaco. 
c     p         : 4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c                 The first 2 entries are the incoming partons.
c     x         : the feynman x of the incoming partons 
c     v         : The 4-momenta of the other particles
c     nlo       : switch for NLO processes
c     
c     OUTPUT
c     Amplitude : The amplitude square as returned by the matrix element code.
c*************************************************************************
      implicit none

#include "global.inc"

      real*8 Amplitude
      real*8 x(nx) 
      real*8 rand(max_PS_dim)
      real*8 p(0:3,max_p,max_ptilde)
      real*8 v(0:3,max_v,max_vtilde)
      real*8 rn, rn2(2), xuz(2,2:3), m2s(0:3), m2s_dec
      logical lokt(3)
      integer NLO

c     to convert 'NLO' to vbfnlo matrixelement notation (this is m2s_wbfvv - specific)
      integer VBFNLOswitch(0:3)
      data VBFNLOswitch / 0, 4,-4,-5 /

c     masses and widths
      real*8 XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs

      Double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                 BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,BHGAM
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM


c     external matrixelement functions
      real*8 m2s_ubtd
      real*8 m2s_singletop
      real*8 m2s_stpw
      real*8 m2s_wbfvv
      real*8 m2s_wbfw
      real*8 m2s_ggfh
      real*8 m2s_QCDww
      real*8 m2s_z2j
      real*8 RandomNumber
      real*8 higgs2tau
     
      integer mu
      external m2s_ubtd
      external m2s_singletop
      external m2s_wbfvv
      external m2s_wbfw
      external m2s_stpw
      external m2s_ggfh
      external m2s_QCDww
      external m2s_z2j
      external RandomNumber
      external higgs2tau
c     auxiliary 
      real*8 amp, Q(0:3)
      integer lflavr(2)
      real*8 mass2
      external mass2

      rn = RandomNumber()

      SELECT CASE(procID)
#ifdef WITH_VVJJ 
c     pp -> Hjj
      CASE(100)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> Hjj -> gamma gamma jj
      CASE(101)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         AMplitude = Amplitude*BHGAM
c     pp -> Hjj -> mu+mu- jj
      CASE(102)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHMU
c     pp -> Hjj -> tau+tau- jj
      CASE(103)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjj -> b bbar jj
      CASE(104)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = AMplitude*BHBB
c     pp -> Hjj -> W W jj
      CASE(111:119)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v,3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjj -> Z Z jj 
      CASE(141:147)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v,2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Zjj
      CASE(201:204) 
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(2,0,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> W+jj
      CASE(301:303) 
         rn = RandomNumber()
         Amplitude = m2s_wbfw(3,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> W-jj
      CASE(304:306) 
         rn = RandomNumber()
         Amplitude = m2s_wbfw(4,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> WWjj 
      CASE(411:419)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(34,0,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> ZZjj 
      CASE(501:503)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(22,0,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
      CASE(504:506)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(21,0,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
         

#ifdef WITH_VVJJJ
c     pp -> Hjjj
      CASE(2100)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> Hjjj -> gamma gamma jjj
      CASE(2101)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         AMplitude = Amplitude*BHGAM
c     pp -> Hjjj -> mu+mu- jjj
      CASE(2102)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjj -> tau+tau- jjj
      CASE(2103)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjj -> b bbar jjj
      CASE(2104)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = AMplitude*BHBB
c     pp -> Hjjj -> W W jjj
      CASE(2111:2119)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v,3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjj -> Z Z jjj
      CASE(2141:2147)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v,2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Zjj
      CASE(2201:2204) 
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 2,0, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W+jjj
      CASE(2301:2303)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 3, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W-jjj
      CASE(2304:2306)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 4, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> WW jjj
      CASE(2411:2419)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 34,0,  VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> ZZ jjj
      CASE(2501:2503)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 22,0,  VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
      CASE(2504:2506)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 21,0,  VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)

#endif
#endif
#ifdef WITH_GGF
c     GF pp -> Hjj
*      CASE(4100)
*         Amplitude = m2s_ggfh( x, p(0,1,1),v(0,1,1),rand )
c     GF pp -> Hjj -> gamma gamma jj
*      CASE(4101)
*         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
*            q(mu) = v(mu,1,1)+v(mu,2,1)
*         enddo
*         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
*         AMplitude = Amplitude*BHGAM
c     GF pp -> Hjj -> mu+ mu- jj
*      CASE(4102)  
*         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
*            q(mu) = v(mu,1,1)+v(mu,2,1)
*         enddo
*         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
*         Amplitude = Amplitude*BHMU
c     GF pp -> Hjj -> tau+ tau- jj
*      CASE(4103)
*         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
*            q(mu) = v(mu,1,1)+v(mu,2,1)
*         enddo
*         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
*         rn = RandomNumber()
*         Amplitude = amplitude * higgs2tau(q, rn)
c     GF pp -> Hjj -> WWjj -> llnunu jj
*      CASE(4111:4119)
*         do mu = 0,3 !     sum the 4 lepton momenta up to Higgs momentum
*            q(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)+v(mu,4,1)
*         enddo
*         call m2s_WW(v(0,1,1),amp) ! decay H -> WW -> llll
*         amp = amp / ( (xm2s(6) - mass2(q) )**2  +  xmgs(6)**2 ) ! propagator factor
*         Amplitude = amp * m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
#endif
c     QCD pp -> Zjj -> lljj
      CASE(4201:4203)
         rn2(1) = RandomNumber()
         rn2(2) = RandomNumber()
         lflavr(1) = 22 ! mu mubar
         lflavr(2) = 22
         Amplitude = m2s_z2j( 2, lflavr, x, p(0,1,1), v(0,1,1) ,rn2)
c     QCD pp -> Zjj -> nunujj
      CASE(4204)
         rn2(1) = RandomNumber()
         rn2(2) = RandomNumber()
         lflavr(1) = 11 ! numu numubar
         lflavr(2) = 11
         Amplitude = m2s_z2j( 2, lflavr, x, p(0,1,1), v(0,1,1) ,rn2)
c     QCD pp -> WWjj -> llnunu jj
*      CASE(4411:4419) 
*         Amplitude = m2s_QCDww( x, p(0,1,1), v(0,1,1) )

c     pp -> t W j -> WW jj
*      CASE(9414)
*         Amplitude = m2s_stpw( x, p(0,1,1),v(0,1,1))
c     pp -> t j -> W jj
*      CASE(9415)
*         Amplitude = m2s_singletop(x,p(0,1,1),v(0,1,1))

      CASE DEFAULT
         print *,'Error: Amplitude for process not implemented, ID = ',procID
         print *,'Check process-ID and compiler flags'
         STOP
      END SELECT
      
      RETURN
      END
       
c*************************************************************************
      SUBROUTINE RE_Amplitude(rand, p, x, v, lokt, xuz, m2s)
c*************************************************************************
c     Amplitude for real emissions.
c*************************************************************************
      implicit none

#include "global.inc"
      real*8 x(nx) 
      real*8 rand(max_PS_dim)
      real*8 p(0:3,max_p,max_ptilde)
      real*8 v(0:3,max_v,max_vtilde)
      real*8 rn, xuz(2,2:3), m2s(0:3), m2s_dec
      logical lokt(3)
      integer L

c     masses and widths
      real*8 XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs

      Double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                 BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,BHGAM
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM

      real*8 RandomNumber
      external RandomNumber
      SELECT CASE(procID)
#ifdef WITH_VVJJJ
c     pp -> Hjjj
      CASE(0100)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> Hjjj -> gamma gamma jjj
      CASE(0101)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> Hjjj -> mu+mu- jjj
      CASE(0102)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> Hjjj -> tau+tau- jjj
      CASE(0103)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> Hjjj -> b bbar jjj
      CASE(0104)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> Hjjj -> W W jjj
      CASE(0111:0119)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v,3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjjj -> Z Z jjj
      CASE(0141:0147)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v,2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Zjjj 
      CASE(201:204) 
         rn = RandomNumber()
         call m2s_wbfzhg( 2,0, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W+ jjj
      CASE(301:303) 
         rn = RandomNumber()
         call m2s_wbfw3j( 3, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W- jjj
      CASE(304:306)
         rn = RandomNumber()
         call m2s_wbfw3j( 4, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> WW jjj
      CASE(411:419)
         rn = RandomNumber()
         call m2s_wbfzhg( 34,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
c     pp -> ZZ jjj
      CASE(501:503)
         rn = RandomNumber()
         call m2s_wbfzhg( 22,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
      CASE(504:506)
         rn = RandomNumber()
         call m2s_wbfzhg( 21,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 


#endif
      END SELECT

!!!!!!!!!!!!!!!!
c the old processes
      SELECT CASE(procID)
#ifdef WITH_VVJJJ
      CASE(1,7)
         rn = RandomNumber()
         call m2s_wbfzhg( 34,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
        
      CASE(2)
         rn = RandomNumber()
         call m2s_wbfzhg( 22,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
        
      CASE(3)
         rn = RandomNumber()
         call m2s_wbfzhg( 21,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
        
      CASE(14)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,3, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
      CASE(15)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
      CASE(16) 
         rn = RandomNumber()
         call m2s_wbfzhg( 2,0, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
      CASE(17)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,6, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
      CASE(22) 
         rn = RandomNumber()
         call m2s_wbfw3j( 3, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
      CASE(23)
         rn = RandomNumber()
         call m2s_wbfw3j( 4, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)

c     pp -> Hjj (j)
      CASE(50)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> Hjj (j) -> gamma gamma jj (j)
      CASE(51)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> Hjj (j) -> Z Z jj (j) -> 4l jj (j)
      CASE(52)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v,2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjj (j) -> W W jj (j) -> 4l jj (j)
      CASE(53)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v,3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjj (j) -> tau+tau- jj (j)
      CASE(54)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> Hjj (j) -> b bbar jj (j)
      CASE(55)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> Hjj (j) -> mu+mu- jj (j)
      CASE(56)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHMU
         enddo


#endif
      CASE DEFAULT
c         print *,"Error: Real emissions Amplitude ",
c     &           "for process not implemented, ID = ",procID
c         STOP
      END SELECT

      RETURN
      END
