****************************************************************************
* This file contains the subroutines for the histogram handling.
* There is an internal representation of the histograms, that is filled
* by calls of "FillHist". In the end of the program, this data is transvert
* to other histogram representations (e.g. hbook, top-drawer).
* Each histogram has an ID, which is used for the filling.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     SUBROUTINE InitHistograms
c     SUBROUTINE HistogramEvent(weight,dsig,dps, rand, p, x, v, jets, n_jets, NLO)
c     SUBROUTINE FillHist(ID, value, weight)
c     SUBROUTINE CreateHist(ID, title, bins, min, max)
c     SUBROUTINE WriteHistograms
c     SUBROUTINE WriteRootHists
c     SUBROUTINE WritePawHists
c     SUBROUTINE WriteTopHists
c     SUBROUTINE WriteGnuHists
c     SUBROUTINE SelectFile
c
c***************************************************************************
c
c      to add a histogram
c     --------------------
c     1: put a 'CreateHist' in 'InitHistograms'. Use unique IDs for each histogram
c     2: put a proper 'FillHist' in 'HistogramEvents'
c
c***************************************************************************

c*************************************************************************
      SUBROUTINE InitHistograms
c*************************************************************************
c     Set up histograms
c*************************************************************************
      implicit none
#include "hist.inc"
#include "global.inc"
#include "lha.inc"
      integer id, j 
      real*8 hwid
      character*5 ext

c
c reset histogram contents to zero     
c
      do id = 1,HISTMAX
         nbin(ID) = 100
         hlo(ID) = 0d0
         hup(ID) = 1d0
         htitles(ID) = ""
         do j = 1,BINMAX
            sum(id,j) = 0
            square(id,j) = 0
            count(id,j) = 0
         enddo
         hstored(ID) = .false.
      enddo


***********************************************************************
*                        *
*    HISTOGRAM SETUP     *
*                        *
**************************
*

*     call CreateHist(ID, title, #bins, min, max)
      call CreateHist(29, "ds/deta_jj (fb)", 100,0d0, 10d0)
      call CreateHist(30, "dS/dR_jj (fb)", 100, 0d0, 10d0)
      call CreateHist(31, "dS/dPhi_jj (fb)", 100, 0d0, 200d0)
      call CreateHist(40, "dS/dR_ll (fb)", 100, 0d0, 6d0)
      call CreateHist(41, "dS/dpT_j (fb/GeV)",100,0d0,250d0)
      call CreateHist(42, "dS/dpTmax_j (fb/GeV)",100,0d0,500d0)
      call CreateHist(43, "dS/dpTmin_j (fb/GeV)",100,0d0,200d0)
      call CreateHist(44, "dS/deta_j (fb)",100,-5d0,5d0)
      call CreateHist(45, "dS/deta_j1 (fb)",100,-5d0,5d0)
      call CreateHist(46, "dS/deta_j2 (fb)",100,-5d0,5d0)
      call CreateHist(47, "dS/dmjj (fb/GeV)",100,0d0,2500d0)
      call CreateHist(48, "dS/d|eta|max_j (fb/GeV)",100,0d0,5d0)
      call CreateHist(49, "dS/d|eta|min_j (fb/GeV)",100,0d0,5d0)
      call CreateHist(50, "dS/dpTmax_l (fb)",100,0d0,500d0)
      call CreateHist(51, "dS/dpTmin_l (fb)",100,0d0,200d0)
      call CreateHist(52, "dS/d|eta|max_l (fb/GeV)",100,0d0,5d0)
      call CreateHist(53, "dS/d|eta|min_l (fb/GeV)",100,0d0,5d0)

      
* select the histograms which are stored in a file
      hstored(29) = .true. 
      hstored(30) = .true. 
      hstored(31) = .true. 
      hstored(40) = .true. 
      hstored(41) = .true.
      hstored(42) = .true.
      hstored(43) = .true.
      hstored(44) = .true.
      hstored(45) = .true.
      hstored(46) = .true.
      hstored(47) = .true.
      hstored(48) = .true.
      hstored(49) = .true.
      hstored(50) = .true.
      hstored(51) = .true.
      hstored(52) = .true.
      hstored(53) = .true.

***********************************************************************

c     
c select also the NLO histograms to be stored in a file    
c
      if (hasNLO.and.doNLO) then
         do id = 1, histmax/2
            hstored(id+NLOoffset)=hstored(id)
         enddo

      endif
c   
c calculate inverse bin widths for each histogram
c
      do id = 1, histmax
         hwid = hup(id) - hlo(id)
         
         if ( nbin(id).gt.0 .and. hwid.ne.0d0 ) then
            rwid(id) = real( nbin(id) ) / hwid
            dbin(id) = 1/rwid(id)
         else
            dbin(id) = 0d0
            rwid(id) = 0d0
         end if
      end do

c
c read input file from "hist.dat" / "vbfnlo.dat"
c
!!    call loadfile("hist.dat")
      call loadfile("vbfnlo.dat",.false.)
      call read_logical("ROOT",writeROOT,.false.)
      call read_logical("HBOOK",writePAW,.false.)
      call read_logical("TOP",writeTOP,.false.)
      call read_logical("GNU",writeGNU,.false.)
      call read_logical("REPLACE",replaceFiles,.false.)
      call read_string("ROOTFILE",rootfile,"histograms")
      call read_string("PAWFILE",PAWfile,"histograms")
      call read_string("TOPFILE",TOPfile,"histograms")
      call read_string("GNUFILE",GNUfile,"histograms")
      call closefile
      if (.not.replacefiles) then
         ext = '.root'
         call selectfile(rootfile,ext)
         ext = '.paw'
         call selectfile(pawfile,ext)
         ext = '.top'
         call selectfile(topfile,ext)
         ext = '.gp'
         call selectfile(gnufile,ext)
      else
        rootfile= rootfile(1:index(rootfile,' ')-1)//".root"
        pawfile= pawfile(1:index(pawfile,' ')-1)//".paw"
        topfile= topfile(1:index(topfile,' ')-1)//".top"        
        gnufile= gnufile(1:index(gnufile,' ')-1)//".gp"        
      endif
     
      RETURN
      END


c*************************************************************************
      SUBROUTINE HistogramEvent(weight,dsig,dps, rand, p, x, v,
     &                           jets, n_jets, leptons, nl, photons, nphotons, NLO)
c*************************************************************************
c     Fill histograms
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 weight, dsig, dps
      integer NLO 
      real*8 p(0:3,max_p), v(0:3,max_v), x(nx)
      real*8 rand(max_PS_dim)
      real*8 jets(0:7,max_jets), leptons(0:7,max_v), photons(0:7,max_v)
      integer n_jets, nl, nphotons

      real*8 dw
      real*8 deltaR, deltaY, deltaPhi, mjj2, mjj
      
      double precision RSEPS
      external RSEPS

      if ((NLO.ne.0).and.(NLO.ne.1)) return
      
      if (dsig.eq.0d0) return

      dw = dsig * weight

******************
*  Delta eta_jj  *
******************
      if (n_jets.ge.2) then
         deltay = abs(jets(6,1)-jets(6,2))
         call FillHist(29, deltay, dw, NLO)
      endif

****************
*  Delta R_jj  *
****************
      if (n_jets.ge.2) then
         deltaR = RSEPS(jets(6,1),jets(7,1),jets(6,2),jets(7,2))
         call FillHist(30, deltaR, dw, NLO)
      endif

******************
*  Delta Phi_jj  *
******************
      if (n_jets.ge.2) then
         if (jets(6,1).gt.jets(6,2)) then
            deltaPhi = jets(7,1) - jets(7,2)
         else
            deltaPhi = jets(7,2) - jets(7,1)
         endif
         if (deltaPhi.gt.Pi) then
            deltaPhi = deltaPhi - 2d0*pi
         else if (deltaPhi.lt.-Pi) then
            deltaPhi = deltaPhi + 2d0*pi
         endif
         deltaPhi = deltaPhi * 180d0 / Pi
         call FillHist(31, deltaPhi, dw, NLO)
      endif

****************
*  Delta R_ll  *
****************      
      if (nl.ge.2) then       
         deltaR = RSEPS(leptons(6,1),leptons(7,1),leptons(6,2),leptons(7,2))
         call FillHist(40, deltaR, dw, NLO)
      endif

**********************
* pT of tagging jets *
**********************
      if (n_jets.ge.2) then
         call FillHist(41, jets(5,1), dw*0.5d0, NLO)
         call FillHist(41, jets(5,2), dw*0.5d0, NLO)
         call FillHist(42, jets(5,1), dw, NLO)
         call FillHist(43, jets(5,2), dw, NLO)
      endif

***********************
* eta of tagging jets *
***********************
      if (n_jets.ge.2) then
         call FillHist(44, jets(6,1), dw*0.5d0, NLO)
         call FillHist(44, jets(6,2), dw*0.5d0, NLO)
         call FillHist(45, jets(6,1), dw, NLO)
         call FillHist(46, jets(6,2), dw, NLO)
      endif

****************************
* inv mass of tagging jets *
****************************
      if (n_jets.ge.2) then
         mjj=sqrt(mjj2(jets(0,1),jets(0,2)))
         call FillHist(47, mjj, dw, NLO)
      endif

*************************
* |eta| of tagging jets *
*************************
      if (n_jets.ge.2) then
         call FillHist(48, abs(jets(6,1)), dw, NLO)
         call FillHist(49, abs(jets(6,2)), dw, NLO)
      endif

**********************
* pT of leptons      *
**********************
      if (nl.ge.2) then
         call FillHist(50, max(leptons(5,1),leptons(5,2)), dw, NLO)
         call FillHist(51, min(leptons(5,1),leptons(5,2)), dw, NLO)
      endif

*************************
* |eta| of leptons      *
*************************
      if (nl.ge.2) then
         call FillHist(52, abs(max(jets(6,1),jets(6,2))), dw, NLO)
         call FillHist(53, abs(min(jets(6,1),jets(6,2))), dw, NLO)
      endif

      RETURN
      END


c*************************************************************************
      SUBROUTINE FillHist(ID, value, weight, NLO)
c*************************************************************************
c     Fill value in histogram. 
c     Smear over 2 bins if value is within fractio dxb of bin boundary
c*************************************************************************
      implicit none
#include "hist.inc"
      Integer ID, NLO
      real*8 value, weight
      Integer hist_in, BIN, binl, binh
      real*8 X,Y 
      real*8 xbin, yl, yh
      real*8 drb, tdrbi, rat
      parameter (drb=0.49d0, tdrbi = 0.5d0/drb)

      hist_in = ID
      if (NLO.eq.1) hist_in=hist_in+NLOoffset
      x = value
      y = weight*rwid(hist_in)
      
      xbin = (X-HLO(hist_in)) * RWID(hist_in)
      if (xbin.gt.drb) then
         bin = min( INT(xbin), nbin(hist_in) ) 
         if ( (xbin-bin).lt.drb ) then ! x in low end of bin
            binl = bin+1
            binh = bin+2
            rat = (xbin-bin)*tdrbi + 0.5
            yh = y*rat
            yl = y-yh
            bin = 0
         elseif( (xbin-bin).lt.(1.0-drb) ) then ! x in main part of bin
            bin = bin+2
C            elseif( (xbin-bin).lt.1.0 ) then   ! x in high part of bin
         elseif( (xbin-bin).lt.1.0.and.xbin.lt.nbin(hist_in) ) then ! x in high part of bin
            binl = bin+2
            binh = bin+3
            rat = (xbin-bin-1.0)*tdrbi + 0.5
            yh = y*rat
            yl = y-yh
            bin = 0
         else                   ! x more than drp above upper bin
            bin = nbin(hist_in)+2
         endif
      elseif(xbin.lt.-drb) then
         bin = 1
      else                      ! xbin in [-drb,drb]
         binl = 1
         binh = 2
         rat = xbin * tdrbi + 0.5 ! in [0,1]
         yh = y*rat
         yl = y-yh
         bin = 0
      endif
      if (hist_in.eq.-1) then
         print*,' '
         print*,' input to hf1dp: x,y = ',value,weight
         print*,' bin = ',bin
         if (bin.eq.0) then
            print*,' bin_or. = ',min( INT(xbin), nbin(hist_in) ),
     1           ' binl,h = ',binl,binh
            print*,' xbin = ',xbin,' rat = ',rat
            print*,' yl = ',yl,' yh = ',yh
         endif
      endif
      if (bin.eq.0) then
         SUM(HIST_IN,BINL) = SUM(HIST_IN,BINL) + YL
         SQUARE(HIST_in,BINL) = SQUARE(HIST_in,BINL) + YL*YL
         COUNT(HIST_in,BINL) = COUNT(HIST_in,BINL) + rat
         SUM(HIST_IN,BINH) = SUM(HIST_IN,BINH) + YH
         SQUARE(HIST_in,BINH) = SQUARE(HIST_in,BINH) + YH*YH
         COUNT(HIST_in,BINH) = COUNT(HIST_in,BINH) + 1d0-rat
      else
         SUM(HIST_IN,BIN) = SUM(HIST_IN,BIN) + Y
         SQUARE(HIST_in,BIN) = SQUARE(HIST_in,BIN) + Y*Y
         COUNT(HIST_in,BIN) = COUNT(HIST_in,BIN) + 1
      endif
      
      RETURN
      END

c*************************************************************************
      SUBROUTINE CreateHist(ID, title, bins, min, max)
c*************************************************************************
c     Set up the histogram for the internal representation.
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID
      character*(*) title
      integer bins
      real*8 min, max

      nbin(ID) = bins
      hlo(ID) = min
      hup(ID) = max
      htitles(ID) = title

      nbin(ID+NLOoffset) = bins
      hlo(ID+NLOoffset) = min
      hup(ID+NLOoffset) = max
c      htitles(ID+NLOoffset) = title(1:index(title,'  ')-1)//" (NLO)"
      write(htitles(ID+NLOoffset),*) title," (NLO)"
      htitles(ID+NLOoffset) = htitles(ID+NLOoffset)(1:index(htitles(ID+NLOoffset),'  '))
     
      RETURN
      END

c*************************************************************************
      SUBROUTINE WriteHistograms
c*************************************************************************
c     write out histograms in various formats
c*************************************************************************
      implicit none
#include "hist.inc"      
#ifdef WITH_ROOT
      if (writeRoot) call WriteRootHists
#endif

#ifdef WITH_CERN
      if (writePaw) call WritePawHists
#endif

      if (writeTop) call WriteTopHists

      if (writegnu) call WriteGnuHists

      RETURN
      END

#ifdef WITH_ROOT
c*************************************************************************
      SUBROUTINE WriteRootHists
c*************************************************************************
c     Transfers the histogram data to the Root format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin
      double precision x,y
      
      call openrootsession(rootfile)
      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
           
            call createroothisto(ID, htitles(ID),nbin(ID), hlo(ID), hup(ID))
            histdp = ID 
            do bin = 1,nbin(ID)+2
               if (count(histdp,bin).gt.1d-3) then
                  x = hlo(ID) + dbin(ID)*(bin-1.5d0)
                  y = sum(histdp,bin)
                  call fillroothisto(ID, x, y)
                  
               endif
            enddo
         ENDIF
      ENDDO

      call closerootsession
      print *," created root file for histograms : ", rootfile
      
      RETURN
      END
#endif

#ifdef WITH_CERN
c*************************************************************************
      SUBROUTINE WritePawHists
c*************************************************************************
c     Transfers the histogram data to the HBOOK format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin
      real*4 x,y, lo, up
      integer  nhmem
      parameter( nhmem=262144 )
      real  hmem(nhmem)
      common /pawc/ hmem

      call hlimit( nhmem )

      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            lo = hlo(id)
            up = hup(id)
            call hbook1( ID,htitles(ID), nbin(id), lo, up, 0.0)
            call hidopt( ID, 'inte' )
            histdp = ID 
            do bin = 1,nbin(ID)+2
               if (count(histdp,bin).gt.1d-3) then
                  x = lo + dbin(ID)*(real(bin)-1.5)
                  y = sum(histdp,bin)
                  call hf1(ID,x,y)
               endif
            enddo
         ENDIF
      ENDDO

      call hrput( 0, pawfile, 'N' )
       print *," created paw file for histograms : ", pawfile
       

      RETURN
      END
#endif      


c*************************************************************************
      SUBROUTINE WriteTopHists
c*************************************************************************
c     Transfers the histogram data to the top-drawer format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin, ibin, i1
      double precision x,y
      double precision delx,xres(0:1000,2), xmin,xmax,ymin,ymax,ymaxK,yminK

      OPEN(UNIT=20,FILE=topfile,STATUS='REPLACE')

      write (20,'(A)') ' set size 11 by 8.5 '

      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            write (20,*)
            write (20,'(a)') ' new frame '

            write (20,'(A)') ' set intensity 4 '
            write (20,'(A)') ' set font duplex ' 
            write (20,'(A)') ' set tick size .05 '
            write (20,'(A)') ' set title size 2.0 '
            
            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            XRES(1,1) = hlo(id) + DELX/2.
            DO  IBIN = 2,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO
            xmin = hlo(id)
            xmax = hup(id)
            ymin = 1d33
            ymax = -1d33
            yminK = 0
            ymaxK = 2
            do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
               xres(ibin,2) = sum(id,ibin+1)               
               ymin = min( ymin, xres(ibin,2) )
               ymax = max( ymax, xres(ibin,2) )              
            enddo
            do i1 = 0,nbin(id)+1
               if (xres(i1,2).eq.0.) xres(i1,2) = 1e-20
            end do
            write (20,'(A,i3,a)') ' title 2.0 7.5 ''Fig.',
     1           id,': '//htitles(id)
            write (20,'(A)') ' set window  x 1.2 to 5.2 y 3 to 7.0 '
            write (20,202) ' set limits x ',xmin,' to ',xmax,
     1                             ' y ',ymin,' to ',ymax
 201        format(2(a,f8.2),2(a,f12.2))
 202        format(2(a,f8.2),2(a,f12.5)) 
            write(20,*) ' ( '
            DO  IBIN = 1,NBIN(ID)
               WRITE (20,'(2g20.6)')  XRES(ibin,1), xres(ibin,2)
            END DO
            write (20,'(A)') ' join solid '
                       
         ENDIF
      ENDDO

      CLOSE(20)

      print *," created top-drawer file for histograms : ", topfile
       

      RETURN
      END


c*************************************************************************
      SUBROUTINE WriteGnuHists
c*************************************************************************
c     Transfers the histogram data to the gnuplot format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin, ibin, i1
      double precision x,y
      double precision delx,xres(0:1000,2), xmin,xmax,ymin,ymax,ymaxK,yminK

      OPEN(UNIT=20,FILE=gnufile,STATUS='REPLACE')

      write (20,'(AAA)') 'set output "',gnufile(1:index(gnufile,' ')-4),'.ps"'
      write (20,'(A)') 'set terminal postscript'
      write (20,'(A)') 'set style data lines'
      write (20,'(A)') 'set key off'

      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            write (20,*)
            write (20,*) '#',ID,': ',
     &           htitles(id)(1:index(htitles(id),'   ')-1)
            write (20,'(A)') 'set title "'//
     &           htitles(id)(1:index(htitles(id),'   ')-1)//'"'
            write (20,'(A)') 'plot "-" '
            
            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            XRES(1,1) = hlo(id) + DELX/2.
            DO  IBIN = 2,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO
            xmin = hlo(id)
            xmax = hup(id)
            ymin = 1d33
            ymax = -1d33
            yminK = 0
            ymaxK = 2
            do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
               xres(ibin,2) = sum(id,ibin+1)               
               ymin = min( ymin, xres(ibin,2) )
               ymax = max( ymax, xres(ibin,2) )              
            enddo
            do i1 = 0,nbin(id)+1
               if (xres(i1,2).eq.0.) xres(i1,2) = 1e-20
            end do

            DO  IBIN = 1,NBIN(ID)
               WRITE (20,'(2g20.6)')  XRES(ibin,1), xres(ibin,2)
            END DO
            
            write (20,'(A)') 'e'
          ENDIF
      ENDDO

      CLOSE(20)
      print *," created GNUplot file for histograms : ", gnufile
      
      RETURN
      END



ccccccccccccccccccccc begin subroutine SelectFile ccccccccccccccc
c
c  This subroutine selects a file name which does not exist yet
c  Terrance Figy <terrance@pheno.physics.wisc.edu
c  August 17, 2001  
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine SelectFile(name,extens)
      implicit none
      character*50 name,lname,lname1
      character*1 digit(0:9)
      character*5 extens
      integer ii,i,j
      logical FileAlreadyExists
      data DIGIT /'0','1','2','3','4','5','6','7','8','9'/
 
      lname1 = name
      do 20 ii = 1,99
     
         if((ii.ge.1).and.(ii.le.9)) then
            
            lname = 
     &           lname1(1:index(lname1,'  ')-1)//'.'//DIGIT(ii)//extens
            inquire(file=lname,exist=FileAlreadyExists)
            if(FileAlreadyExists) then
               goto 20
            else
               name = lname(1:index(lname,' ')-1)
               return
            endif
         elseif(ii.gt.9) then
            i = (ii/10)
            j = mod(ii,10)
            lname = lname1(1:index(lname1,'  ')-1)//'.'//
     &              DIGIT(i)//DIGIT(j)//extens
            inquire(file=lname,exist=FileAlreadyExists)
            if(FileAlreadyExists) then
               goto 20
            else
               name = lname(1:index(lname,' ')-1)
               return
            endif
         endif

         name = name(1:index(name,' ')-1)
         
 20      continue
         write(*,*) '-------- fatal crash in SelectFile ----------'
         stop
         end
