c*************************************************************************  
c     This file contains subroutines for reading input files. 
c     The input files are in the format: key = value 
c*************************************************************************  
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     SUBROUTINE loadfile(filename)
c     SUBROUTINE closefile
c     SUBROUTINE read_Real(key, value, default_value)
c     SUBROUTINE read_Int(key, value, default_value)
c     SUBROUTINE read_Logical(key, value, default_value)
c     SUBROUTINE read_String(key, value, default_value)
c
c*************************************************************************  



c*************************************************************************  
      subroutine loadfile(filename,show_input)
c*************************************************************************  
c     Opens a textfile and reads the data in an internal buffer.
c     End-of-line comments, starting with ! or //, are removed.
c     Do only process one single file at a time.
c     DON'T FORGET TO CALL CLOSEFILE AFTERWARDS.
c*************************************************************************  
      implicit none
      character*(*) filename
     
      include 'readinput.inc'
      
      integer i,le,k,j
      character*100 line
      logical show_input
      
      iunit=10
      usedefaults=.false.
      showinput=show_input
c Open file
      open (unit=iunit,
     &      file=filename,
     &      status='old',err=77)

c read text lines
      n_lines=0
 50   read(iunit,'(a100)',end=99,err=98) lines(n_lines+1)
      n_lines=n_lines+1
      if (n_lines.lt.maxlines) GOTO 50
 98   CONTINUE
      print *," Read error on file",filename
      print *," Using default values."
      usedefaults=.true.
 99   CONTINUE
      n_lines=n_lines+1

c remove comments
      do i=1,n_lines
         line=lines(i)
         if (INDEX(line,'!').ne.0) then
            line=line(1:INDEX(line,'!')-1)
         endif
         if (INDEX(line,'//').ne.0) then
            line=line(1:INDEX(line,'//')-1)
         endif
c upper case everything in front of a '='
         le = index(line,'=')
         if (le.gt.0) then
            do j=1,le-1
               k=ichar(line(j:j))
               if(k.ge.97.and.k.le.122) then
                  k=ichar(line(j:j))-32   
                  line(j:j)=char(k)
               endif
            enddo
         endif
         lines(i)=line
      enddo
cc     if (showinput) write(*,'(A,A,A)'), " reading input from file ",filename," : "
      
      return
 77   CONTINUE
      print *," Open error on file ",filename
      print *," Using default values."
      usedefaults=.true.
      return
      end

c*************************************************************************  
      subroutine closefile
c*************************************************************************  
c     Closes the current file.
c*************************************************************************  
      implicit none

      include 'readinput.inc'
      
      close(iunit)
      return
      end

c*************************************************************************  
      SUBROUTINE read_Real(key, value, default_value)
c*************************************************************************  
      implicit none
     
      include 'readinput.inc'

      character*(*) key
      real*8 value, default_value
      character*100 line
      integer i,le,lk,lt
     
      value=default_value
      if (usedefaults) RETURN
      do i=1,n_lines
         line = lines(i)
         lk=INDEX(line,key)
         if (lk.ne.0) then
            le=INDEX(line,'=')
            lt = lk+LEN(key) 
            if (((le.ne.0).and.(le.gt.lk))
     &       .and.(line(lt:lt).eq.' '.or.line(lt:lt).eq.'='.or.line(lt:lt).eq.achar(9) )) then
               line=" "//line(le+1:len(line))
               read(line,*,ERR=10) value       
               if (showinput) write(*,'(T4,A,T25,A,G11.5)') key," = ",value
               RETURN
            endif
         endif
 10      CONTINUE
      enddo

      print *,"Warning: cannot read value for key = ",key,
     &      ". Using default value : ",default_value
      RETURN
      END

c*************************************************************************  
      SUBROUTINE read_Int(key, value, default_value)
c*************************************************************************  
      implicit none
     
      include 'readinput.inc'

      character*(*) key
      
      integer value, default_value
      character*100 line
      integer i,le,lk,lt
     
      value=default_value
      if (usedefaults) RETURN
      do i=1,n_lines
         line = lines(i)
         lk=INDEX(line,key)
         if (lk.ne.0) then
            le=INDEX(line,'=')
            lt = lk+LEN(key) 
            if (((le.ne.0).and.(le.gt.lk))
     &           .and.(line(lt:lt).eq.' '.or.line(lt:lt).eq.'='.or.line(lt:lt).eq.achar(9))) then
               line=" "//line(le+1:len(line))
               read(line,*,ERR=10) value
               if (showinput) write(*,'(T4,A,T25,A,I5)') key," = ",value
               RETURN
            endif
         endif
 10      CONTINUE
      enddo

      print *,"Warning: cannot read value for key = ",key,
     &      ". Using default value : ",default_value
      RETURN
      END

c*************************************************************************  
      SUBROUTINE read_Logical(key, value, default_value)
c*************************************************************************  
      implicit none
     
      include 'readinput.inc'
      
      character*(*) key
      Logical value, default_value
      character*100 line
      integer i,le,lk,lt
     
      value=default_value
      if (usedefaults) RETURN
      do i=1,n_lines
         line = lines(i)
         lk=INDEX(line,key)
         if (lk.ne.0) then
            le=INDEX(line,'=')
            lt = lk+LEN(key) 
            if (((le.ne.0).and.(le.gt.lk))
     &           .and.(line(lt:lt).eq.' '.or.line(lt:lt).eq.'='.or.line(lt:lt).eq.achar(9))) then
               line=" "//line(le+1:len(line))
               read(line,*,ERR=10) value
               if (showinput) write(*,'(T4,A,T25,A,L5)') key," = ",value
               RETURN
            endif
         endif
 10      CONTINUE
      enddo

      print *,"Warning: cannot read value for key = ",key,
     &      ". Using default value : ",default_value
      RETURN
      END

c*************************************************************************  
      SUBROUTINE read_String(key, value, default_value)
c*************************************************************************  
      implicit none
     
      include 'readinput.inc'

      character*(*) key
      character*(*) value, default_value
      character*100 line
      integer i,le,lk,lt
     
      value=default_value
      if (usedefaults) RETURN
      do i=1,n_lines
         line = lines(i)
         lk=INDEX(line,key)
         if (lk.ne.0) then
            le=INDEX(line,'=')
            lt = lk+LEN(key) 
            if (((le.ne.0).and.(le.gt.lk))
     &           .and.(line(lt:lt).eq.' '.or.line(lt:lt).eq.'='.or.line(lt:lt).eq.achar(9))) then           
               line=" "//line(le+1:len(line))
               read(line,*,ERR=10) value
               if (showinput) write(*,'(T4,A,T25,A,A)') key," = ",value
               RETURN
            endif
         endif
 10      CONTINUE
      enddo

      print *,"Warning: cannot read value for key = ",key,
     &      ". Using default value : ",default_value
      RETURN
      END


