****************************************************************************
*
*  This is the VBFNLO main program. 
*
****************************************************************************

****************************************************************************
*  Last modified: 04.10.2006
****************************************************************************

      PROGRAM vbfnlo
      implicit none

c include global variables via include-files, e.g. "procID"
#include "global.inc"
#include "lha.inc"
#include "tau_hel.inc"

c define local variables
c--------------------------------------------------------
c     variables for the integration loops
      integer iteration, i, seed
      integer*8 Ncall

c     digit characters for numbering the grid output file
      character*1 digit(9)
      data digit /'1','2','3','4','5','6','7','8','9'/
c     the filenames for the grid output
      character*50 file_name_out, file_name_in

c     random number array controlled by monaco
      real*8 rand(max_PS_dim),rn

c     4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c     The partons are the incoming quarks/gluons + the 2 tagging jets + 
c     the real emission (for NLO only)
      real*8 p(0:3,max_p,max_ptilde) 

c     Additional particles like a higgs boson, or any decay products 
c     (also hadronic ones). in short: everything which is not a parton 
c     involved in the basic QCD 2->2(+1)+X process.
      real*8 v(0:3,max_v,max_vtilde) 

c     Jets after jet-defining function
      real*8 jets(0:7,max_jets,max_jtilde) 
      integer n_jets(max_jtilde)
c     Leptons, i.e. electrons, muons, taus? no neutrinos
      real*8 leptons(0:7,max_v,max_vtilde)
      integer n_leptons(max_vtilde)
c     photons
      real*8 photons(0:7,max_v,max_vtilde)
      integer n_photons(max_vtilde)
c     event weights, jacobi-factors, amplitude-squares
      real*8 weight, dps, m2s(0:3), dsig(0:3)

c     the result of the integration: crossection, error, chisquare
      real*8 chi2
      real*8 NLOxsection(-2:5), NLOsdev(-2:5), NLOchi2(-2:5)

c     other variables
      real*8 x(nx)   ! feynman x parameters
      logical cut_ok ! whether event passes cuts
      real*8 xuz(2,2:3)
      integer gnlo, L
      logical lokt(3)

c     external functions
      logical Cuts     ! general cut-function
      real*8 amplitude ! general squared-amplitude function
      external Cuts, amplitude

      real*8 RandomNumber
      external RandomNumber     

c     debugging and testing variables and switches
      logical showtiming
      parameter (showtiming=.false.)

      if (showtiming) call mtime

c     first, do all the initialization
      
      call InitGlobalParameters

      call PrintProcInfo
      
      call InitCouplings

      call InitProcess

      call InitScales
      
      call InitCuts
      
      call InitPhaseSpace
      
      call InitRandomNumbers(seed)

      call InitHistograms
      
!      call InitEventOutput

c...Les Houches interface
      if (lha.and..not.doNLO) then 
         if(procID.ge.4100.and.procID.le.4419)then
            lha=.false.
            print*,''
            print*,'LHA file is not available for the process',procID
            print*,''
         else
            lha=.true.
            call lha_file(-1)
         endif
      endif
      
c==========================
c    The LO Calculation
c==========================
    
      if (doBorn) then

         call InitPDFs(0)

         gnlo = 0
         file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)


         DO iteration = 1, N_iterations1
            Ncall = 2**( N_points(0) - N_iterations1 + iteration )
           
            file_name_out = gridname1(1:INDEX(gridname1,'  ')-1)//
     &                   '.out.'//digit(iteration)
            if ( iteration .eq. 1 ) then
               call monaco_init( PS_dimension, Ncall, seed )
               call monaco_read(file_name_in)
            else
               call monaco_init2( PS_dimension, Ncall )
            end if       
            if (iteration.eq.N_iterations1) then
               call monran_set(1) ! save random number info
            endif

            call ResetWeights

            do i = 1, Ncall
c     retrieve a set of random numbers and the corresponding weight
               call monaco_get( rand, weight)
                       
c     transform the random numbers to a physical phase space point
               call phasespace(rand, p, x, v, dps  )
                
               if ( dps .gt. 0d0 ) then

c     define jets, leptons, photons
                  call defineJets(p, v, 1, jets, n_jets)
                  call DefineLeptons(v, 1, leptons, n_leptons)
                  call DefinePhotons(v, 1, photons, n_photons)
c     apply cuts
                  cut_ok = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 1)
               
                  if (cut_ok) then
c     call the matrix element code and get an amplitude square
                     m2s(0) = Amplitude(rand, p, x, v, 0)                 
                  else
                     dps = 0d0
                     m2s(0) = 0d0
                  end if 

                  dsig(0) = m2s(0)*dps

                  if (cut_ok) then
                     eventweight=dsig(0)*weight

c     in the last iteration : 
                     if (iteration.eq.N_iterations1) then

c     pass event information to the histogram routine
                        call HistogramEvent(weight,dsig(0),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 0)

c     the unweighting of events : 
                        
c                        if (unweighting) then 
                           call Reweight(eventweight) ! reweight                         
c                        endif

                        if (eventweight.gt.0d0) then ! event is to be kept

                           if(lha.and..not.doNLO) call lha_put(p,v) !  Les Houches interface


#ifdef WITH_ROOT  
!                          call writerootevent(n_jets(1),jets(0,1,1),n_leptons(1),leptons(0,1,1), eventweight)
    !                      call writerootevent2(n_p,p(0,1,1),n_v,v(0,1,1), eventweight)
    !                      call writerootevent3(n_jets(1),jets(0,1,1),n_leptons(1),leptons(0,1,1), eventweight, tauchi(1) )
#endif

                        endif
                     endif
                     
                  endif ! cutok

               else     ! dps > 0d0
                  m2s(0) = 0d0
                  dsig(0) = 0d0
               end if

c     pass weight to monaco
               call monaco_put(rand, weight, dsig(0))

            end do
        
            call monaco_write( file_name_out )

         END DO

         call monaco_result(xsection, sdev, chi2)
     
c     print out the result. 
         print *,""
         print *," result (LO): ",xsection," +- ",sdev," fb  "
     &        ,sdev/xsection*100d0,'%'
         print *,""
      else
         print *," No LO calculation performed."
      endif

c...Les Houches interface
      if(lha.and..not.doNLO) then
         call lha_file(0)
         call lha_file(1)
      endif

      if(.not.doNLO) then 
         print*,'number of events in the last iteration =',ncall
         call reweightinginfo
      endif

c============================
c    The NLO Calculation
c============================     
c  The virtual corrections      
c----------------------------
      if (doVirtuals.and.hasNLO) then

         if (N_iterations1.eq.1) then
            file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)
         else
            file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)//
     &           '.out.'//digit(N_iterations1-1)
         endif

         call InitPDFs(1)

         NLOxsection(0) = 0d0
         NLOsdev(0) = 0d0

         do gnlo = 1, NLO_Loops ! loop over different virtual contributions

            Ncall = 2**( N_points(gnlo) )
            file_name_out = gridname1(1:INDEX(gridname1,'  ')-1)//
     &                               '.out.'//digit(1)
            
            call monaco_init( PS_dimension, Ncall, seed )
            call monaco_read(file_name_in)
!            call monran_set(2)  
         
            do i = 1, Ncall
               call monaco_get( rand, weight)
         
               call phasespace(rand, p, x, v, dps  )
              
               if ( dps .gt. 0d0 ) then
               
                  call defineJets(p, v, 1, jets, n_jets)
                  call DefineLeptons(v, 1, leptons, n_leptons)
                  call DefinePhotons(v, 1, photons, n_photons)

                  cut_ok = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 1)
           
                  if (cut_ok) then


                     m2s(0) = Amplitude(rand, p, x, v, gnlo)                 
                  else
                     dps = 0d0
                     m2s(0) = 0d0
                  end if 

                  dsig(0) = m2s(0)*dps

                  call HistogramEvent(weight,dsig(0),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 1)

               else
                  m2s(0) = 0d0
                  dsig(0) = 0d0
               end if

               call monaco_put(rand, weight, dsig(0))
            end do

            call monaco_write(file_name_out)
     
            call monaco_result(NLOxsection(gnlo), 
     &                         NLOsdev(gnlo), NLOchi2(gnlo))

            print *," "
            print *," result for virtual contribution, no. ",gnlo
            print *," sigma = ",NLOxsection(gnlo)," +- ",
     &              NLOsdev(gnlo)," fb"
            print *," "
            NLOxsection(0) = NLOxsection(0) + NLOxsection(gnlo)
            NLOsdev(0) = sqrt(NLOsdev(0)**2 + NLOsdev(gnlo)**2)

            print *,' total result for virtual contributions so far '

            print *,' sigma = ',NLOxsection(0),' +- ',NLOsdev(0),' fb'
            print *,' '
         enddo

      endif

c----------------------------
c    The real emissions      
c----------------------------
      if (doEmissions.and.hasNLO) then
         call InitPDFs(1)

         call InitRealEmissions

         file_name_in  = gridname2(1:INDEX(gridname2,'  ')-1)

         DO iteration = 1, N_iterations2
            Ncall = 2**( N_points(-1) - N_iterations2 + iteration )
           
            file_name_out = gridname2(1:INDEX(gridname2,'  ')-1)//
     &                   '.out.'//digit(iteration)
            if ( iteration .eq. 1 ) then
               call monaco_init( PS_dimension, Ncall, seed )
               call monaco_read(file_name_in)
            else
               call monaco_init2( PS_dimension, Ncall )
            end if       
           
            do i = 1, Ncall
               call monaco_get( rand, weight)
                       
               call phasespace(rand, p, x, v, dps  )
            
               if ( dps .gt. 0d0 ) then

! I don't know how this can be generalized, so I put it in here in a dirty fashion                  
                  
                  call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
                 
                  call defineJets(p, v, 3, jets, n_jets)
                  call DefineLeptons(v, 1, leptons, n_leptons)
                  call DefinePhotons(v, 1, photons, n_photons)
                  
                  lokt(1) = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 1)
                  lokt(2) = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 2)
                  lokt(3) = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 3)
                  cut_ok =  lokt(1).or.lokt(2).or.lokt(3)
               
                  if (cut_ok) then
                     call RE_Amplitude(rand, p, x, v, lokt, xuz, m2s)                 
                  else
                     dps = 0d0
                     do L = 0,3
                        m2s(L) = 0d0
                     enddo                     
                  end if 


                  do L = 0,3
                     dsig(L) = m2s(L)*dps
                  enddo   

                  if (iteration.eq.N_iterations2) then

                     if (dsig(0).ne.0d0.or.dsig(1).ne.0) then
                        call HistogramEvent(weight,dsig(1),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 1)
                     endif
                     do L = 2,3
                        if (dsig(L).ne.0d0) then
                           call HistogramEvent(weight,dsig(L),dps,
     &                                rand, p(0,1,L), x, v(0,1,1), 
     &                                jets(0,1,L), n_jets(L), 
     &                                leptons(0,1,1), n_leptons(1),
     &                                photons(0,1,1), n_photons(1), 1)

                        endif
                     enddo


                  endif

               else
                  do L = 0,3
                     m2s(L) = 0d0
                     dsig(L) = 0d0
                  enddo
               end if
               

               call monaco_put(rand, weight, dsig(0))
               
            end do
        
            call monaco_write( file_name_out )

         END DO

         call monaco_result(NLOxsection(-1), NLOsdev(-1), NLOchi2(-1))
     
c     print out the result. 
         print *,""
         print *," result for real emissions "
         print *," sigma = ",NLOxsection(-1)," +- ",NLOsdev(-1)," fb"
         print *,""

      endif
      
      if (dovirtuals.and.doEmissions.and.hasNLO) then
         NLOxsection(-2) = NLOxsection(-1)+NLOxsection(0)
         NLOsdev(-2) = sqrt(NLOsdev(-1)**2+NLOsdev(0)**2)
         print *," "
         print *," final result at NLO "         
         print *," sigma = ",NLOxsection(-2)," +- ",NLOsdev(-2)," fb"
     &   ,NLOsdev(-2)/NLOxsection(-2)*100d0,'%'
      endif

      call WriteHistograms

!      call CloseEventOutput
     
c print out results in a file also
      open (20,
     &      file="xsection.out",
     &      status='REPLACE')
      write(20,*) xsection, sdev
      write(20,*) NLOxsection(-2), NLOsdev(-2)
      close(20)

      if (showtiming) call mtime
   
      STOP
      END


c-------------------------------------------------------------------------
c-------------------------------------------------------------------------

c the following subroutines should be moved into appropriate extra files somewhen

c*************************************************************************
      SUBROUTINE InitRandomNumbers(seed)
c*************************************************************************
c     Select and initialize random number generator.
c*************************************************************************
      implicit none

c     So far we have only one random number generator here: The one from
c     monaco. If it is meaningful, other generators might be included later, 
c     being interfaced by this routine and the 'RandomNumber' function.

      integer seed, seed1, seed2
      
      call loadfile("random.dat",.false.)
      call read_int("SEED",seed,0)
      call closefile

      print *," Random number generator initialized with seed = ",seed
      
c set the two seeds
      if (mod(seed,2).eq.0) then
         seed1 = seed / 2
         seed2 = seed / 2
      else
         seed1 = (seed + 1) / 2
         seed2 = (seed - 1) / 2
      endif

      call iranmr( 1, seed1, seed2 )
      
      RETURN
      END

c*************************************************************************
      FUNCTION RandomNumber()
c*************************************************************************
c     Return a random number
c*************************************************************************
      implicit none
      real*8 R, RandomNumber
      
      call ranmar( R )

      RandomNumber=R

      RETURN
      END
      

c*************************************************************************
      SUBROUTINE InitGlobalParameters
c*************************************************************************
c     Initialize the global parameters which are defined in "global.inc"
c*************************************************************************

      implicit none
#include "global.inc"
#include "scales.inc"
#include "lha.inc"
#include "koppln.inc"
     
c Assign values to the global parameters here.
c Read in values from 'vbfnlo.dat'.

      doBorn=.true.
      doVirtuals=.false.
      doEmissions=.false.

      call loadfile("vbfnlo.dat",.true.)
      print *," "
      print *,"       Information on global parameters"
      print *,"-----------------------------------------------"
      call read_int("PROCESS",procID,100)
      call read_int("LO_ITERATIONS",N_iterations1,4)
      call read_int("NLO_ITERATIONS",N_iterations2,4)
      call read_logical("LHA_SWITCH",lha,.false.)
      call read_logical("UNWEIGHTING_SWITCH",unweighting,.true.)
      call read_int("PRENEVUNW",prenevunw,1000)
      call read_logical("TAUMASS",taumass,.false.)
      call read_int("LO_POINTS",N_points(0),15)
      call read_int("NLO_POINTS",N_points(-1),15)
      call read_logical("NLO_SWITCH",doNLO,.false.)
      call read_string("LO_GRID",gridname1,"grid2")
      call read_string("NLO_GRID",gridname2,"grid3")
     
      call read_Real("ECM",ecm,14000d0)
      call read_int("BEAM1",sign1,1)
      call read_int("BEAM2",sign2,1)

      call read_int("ID_MUF",id_muf,0)
      call read_int("ID_MUR",id_mur,0)
      call read_Real("MUF_USER",muf_user,100d0)
      call read_Real("MUR_USER",mur_user,100d0)
      call read_Real("XIF",xif,1d0)
      call read_Real("XIR",xir,1d0)

      if (donlo) then
         doVirtuals=.true.
         doEmissions=.true.
      endif

      call closefile

      RETURN
      END

c*************************************************************************
      SUBROUTINE InitProcess
c*************************************************************************
c     This routine is intended to do the initialization of parameters, that
c     are process dependend. However process dependend coupling parameters
c     should be set in InitCouplings, or shouldn't they?
c*************************************************************************
      implicit none
#include "global.inc"
      integer lflavr(5:6)

c first set phase space dimension & NLO parameters
      SELECT CASE(procID)
#ifdef WITH_VVJJ 
***********************
* VBF Higgs processes *
***********************
      CASE(100:104) ! VBF pp -> Hjj -> xxjj
         n_p = 4
         n_v = 2
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.true.  ! process at nlo
         NLO_Loops = 1  ! number of different virtual contributions
         N_points(1) = N_points(0)        ! #points for virtual corr.
         call qqhqqgi(0)
      CASE(111:119,141:147) ! VBF pp -> Hjj -> VVjj
         n_p = 4
         n_v = 4
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.true.  ! process at nlo
         NLO_Loops = 1  ! number of different virtual contributions
         N_points(1) = N_points(0)        ! #points for virtual corr. 
         call qqhqqgi(0)
      CASE(2100:2104) ! VBF pp -> Hjjj -> xxjjj
         n_p = 5
         n_v = 2
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.false.  ! process at lo
         NLO_Loops = 0  
         call qqhqqgi(0)
      CASE(2111:2119,2141:2147) ! VBF pp -> Hjjj -> VVjjj
         n_p = 5
         n_v = 4
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.false.  ! process at lo
         NLO_Loops = 0
         call qqhqqgi(0)
*******************
* VBF Z processes *
*******************
      CASE(201:203) ! VBF pp->Zjj->lljj
         n_p = 4
         n_v = 2  
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.true.
         NLO_Loops = 1 
         N_points(1) = N_points(0) 
         lflavr(5) = 2          
         lflavr(6) = 2          
         call qqbqqi(2,lflavr)
      CASE(204) ! VBF pp->Zjj->nunujj
         n_p = 4
         n_v = 2  
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.true.
         NLO_Loops = 1 
         N_points(1) = N_points(0) 
         lflavr(5) = 1          
         lflavr(6) = 1          
         call qqbqqi(2,lflavr)
      CASE(2201:2203) ! VBF pp->Zjjj->lljjj
         n_p = 5
         n_v = 2  
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.false.
         lflavr(5) = 2          
         lflavr(6) = 2          
         call qqbqqi(2,lflavr)
      CASE(2204) ! VBF pp->Zjjj->nunujjj
         n_p = 5
         n_v = 2  
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.false.
         lflavr(5) = 1          
         lflavr(6) = 1          
         call qqbqqi(2,lflavr)
***********************
* VBF W+/W- processes *
***********************
      CASE(301:303) ! VBF pp->W+jj->l+nu
         n_p = 4
         n_v = 2  
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.true.
         NLO_Loops = 1 
         N_points(1) = N_points(0) 
         lflavr(5) = 2          
         lflavr(6) = 1          
         call qqbqqi(3,lflavr)
      CASE(304:306) ! VBF pp->W-jj->l-nu
         n_p = 4
         n_v = 2
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.true.
         NLO_Loops = 1
         N_points(1) = N_points(0)
         lflavr(5) = 1
         lflavr(6) = 2
         call qqbqqi(4,lflavr)
      CASE(2301:2303)
         n_p = 5
         n_v = 2  
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.false.
         NLO_Loops = 0 
         lflavr(5) = 2          
         lflavr(6) = 1          
         call qqbqqi(3,lflavr)
      CASE(2304:2306)
         n_p = 5
         n_v = 2
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.false.
         NLO_Loops = 0
         lflavr(5) = 1
         lflavr(6) = 2
         call qqbqqi(4,lflavr)
********************
* VBF WW processes *
********************
      CASE(411:419)
         n_p = 4  ! actual number of partons
         n_v = 4  ! actual number of other particles
         PS_dimension = 3*(n_p+n_v-nx)-3  ! phasespace dimension
         hasNLO=.true.  ! process at nlo
         NLO_Loops = 3  ! number of different virtual contributions
         N_points(1) = N_points(0)        ! #points for virtual corr. 
         N_points(2) = N_points(0) - 4    ! -||- for boxes
         N_points(3) = N_points(0) - 7    ! -||- for pentagons
         call qqhqqi(3)
      CASE(2411:2419)
         n_p = 5
         n_v = 4
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.false.
         NLO_Loops = 0
         call qqhqqi(3)
********************
* VBF ZZ processes *
********************
      CASE(501:506)
         n_p = 4
         n_v = 4
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.true.
         NLO_Loops = 3
         N_points(1) = N_points(0)
         N_points(2) = N_points(0) - 4
         N_points(3) = N_points(0) - 7
      CASE(2501:2506)  
         n_p = 5
         n_v = 4
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.false.
         NLO_Loops = 0
#endif

#ifdef WITH_GGF
**********************
* GF Higgs processes *
**********************
      CASE(4100)
         n_p = 4
         n_v = 1
         PS_dimension = 3*(n_p-nx)
         hasNLO=.false.
         NLO_Loops = 0
         call init_gluonfusion    
      CASE(4101:4103) ! GF pp -> Hjj -> xxjj
         n_p = 4
         n_v = 2
         PS_dimension = 3*(n_p+n_v-nx)-3
         hasNLO=.false.  ! process at lo
         NLO_Loops = 0  
         call init_gluonfusion
      CASE(4111:4119)
         n_p = 4
         n_v = 4
         PS_dimension = 3*(n_p+n_v-nx)-3
         particle_IDs(1)=12  ! nue
         particle_IDs(2)=-11 ! e+
         particle_IDs(3)=13  ! mu-
         particle_IDs(4)=-14 ! numubar    
         hasNLO=.false.
         NLO_Loops = 0
         call init_gluonfusion      
#endif
*******************
* QCD Z processes *
*******************
      CASE(4201:4204)
         n_p = 4
         n_v = 2
         PS_dimension = 3*(n_p+n_v-nx)-3 
         hasNLO=.false.
         NLO_Loops = 0
********************
* QCD WW processes *
********************         
c$$$      CASE(4411:4419)
c$$$         n_p = 4
c$$$         n_v = 4
c$$$         PS_dimension = 3*(n_p+n_v-nx)-3 
c$$$         hasNLO=.false.
c$$$         NLO_Loops = 0
************************
* Single top processes *
************************
*      CASE(9414)
*         n_p = 4  
*         n_v = 4  
*         PS_dimension = 3*(n_p+n_v-nx)-3 - 1 
*         hasNLO=.false.         ! process at nlo
*         NLO_Loops = 0
*         print *,''
*         print *,"Warning: b-quark mass set to 0"
*         print *,''
*      CASE(9415)
*         n_p = 4
*         n_v = 2
*         PS_dimension = 3*(n_p-nx+n_v-1) - 1 !!    
*         hasNLO=.false.
*         NLO_Loops = 0
*         print *,''
*         print *,"Warning: b-quark mass set to 0"
*         print *,''
      CASE DEFAULT
         print *,'Error: Amplitude for process not implemented, ID = ',procID
         STOP
      END SELECT

c now set particle ids
      SELECT CASE(procID)
************************************
* VBF/QCD Higgs, WW & ZZ processes *
************************************
      CASE(100,2100) ! VBF pp -> Hjj 
         particle_IDs(1)= 100  ! no real particle
         particle_IDs(2)= -100 ! no real particle
      CASE(101,2101) ! VBF pp -> Hjj -> gammagammajj
         particle_IDs(1)= 22 ! photon
         particle_IDs(2)= 22 ! photon
      CASE(102,2102) ! VBF pp -> Hjj -> mu+mu-jj
         particle_IDs(1)= -13  ! mu+
         particle_IDs(2)= 13 ! mu-
      CASE(103,2103) ! VBF pp -> Hjj -> tau+tau-jj 
         particle_IDs(1)= -15 ! tau+
         particle_IDs(2)= 15  ! tau-
      CASE(104,2104) ! VBF pp -> Hjj -> bbjj
         particle_IDs(1)= 5  ! b
         particle_IDs(2)= -5 ! bbar
      CASE(111,2111,411,2411)   ! pp -> WWjj
         particle_IDs(1)= 12  ! nue
         particle_IDs(2)= -11 ! e+
         particle_IDs(3)= -12 ! nuebar
         particle_IDs(4)= 11  ! e-
      CASE(112,2112,412,2412)   ! pp -> WWjj
         particle_IDs(1)= 12  ! nue
         particle_IDs(2)= -11 ! e+
         particle_IDs(3)= -14 ! numubar
         particle_IDs(4)= 13  ! mu-
      CASE(113,2113,413,2413) ! pp ->  WWjj
         particle_IDs(1)= 12  ! nue
         particle_IDs(2)= -11 ! e+
         particle_IDs(3)= -16 ! nutaubar
         particle_IDs(4)= 15  ! tau-
      CASE(114,2114,414,2414) ! pp -> WWjj
         particle_IDs(1)= 14  ! numu
         particle_IDs(2)= -13 ! mu+
         particle_IDs(3)= -12 ! nuebar
         particle_IDs(4)= 11  ! e-
      CASE(115,2115,415,2415) !  pp -> WWjj
         particle_IDs(1)= 14  ! numu
         particle_IDs(2)= -13 ! mu+
         particle_IDs(3)= -14 ! numubar
         particle_IDs(4)= 13  ! mu-
      CASE(116,2116,416,2416) ! pp -> WWjj
         particle_IDs(1)= 14  ! numu
         particle_IDs(2)= -13 ! mu+
         particle_IDs(3)= -16 ! nutaubar
         particle_IDs(4)= 15  ! tau-
      CASE(117,2117,417,2417) !  pp -> WWjj
         particle_IDs(1)= 16  ! nutau
         particle_IDs(2)= -15 ! tau+
         particle_IDs(3)= -12 ! nuebar
         particle_IDs(4)= 11  ! e-
      CASE(118,2118,418,2418) !  pp -> WWjj
         particle_IDs(1)= 16  ! nutau
         particle_IDs(2)= -15 ! tau+
         particle_IDs(3)= -14 ! numubar
         particle_IDs(4)= 13  ! mu-
      CASE(119,2119,419,2419) ! pp -> WWjj
         particle_IDs(1)= 16  ! nutau
         particle_IDs(2)= -15 ! tau+
         particle_IDs(3)= -16 ! nutaubar
         particle_IDs(4)= 15  ! tau-         
      CASE(141,2141,501,2501) ! VBF pp  -> ZZjj
         particle_IDs(1)= -13 ! mu+ 
         particle_IDs(2)= 13  ! mu- 
         particle_IDs(3)= -11 ! e+
         particle_IDs(4)= 11  ! e-
      CASE(142,2142,502,2502) ! VBF pp -> ZZjj
         particle_IDs(1)= -15 ! tau+ 
         particle_IDs(2)= 15  ! tau- 
         particle_IDs(3)= -11 ! e+
         particle_IDs(4)= 11  ! e-
      CASE(143,2143,503,2503) ! VBF pp -> ZZjj
         particle_IDs(1)= -15 ! tau+ 
         particle_IDs(2)= 15  ! tau- 
         particle_IDs(3)= -13 ! mu+
         particle_IDs(4)= 13  ! mu-
      CASE(144,2144,504,2504) ! VBF pp -> ZZjj
         particle_IDs(1)= -11 ! e+
         particle_IDs(2)= 11  ! e-
         particle_IDs(3)= 14  ! numu 
         particle_IDs(4)= -14 ! numubar
      CASE(145,2145,505,2505)   ! VBF pp -> ZZjj
         particle_IDs(1)= -13   ! mu+
         particle_IDs(2)= 13  ! mu-
         particle_IDs(3)= 14  ! numu 
         particle_IDs(4)= -14 ! numubar
      CASE(146,2146,506,2506)   ! VBF pp -> ZZjj
         particle_IDs(1)= -15 ! tau+
         particle_IDs(2)= 15  ! tau-
         particle_IDs(3)= 14    ! numu 
         particle_IDs(4)= -14   ! numubar
!      CASE(147,2147,507,2507) ! VBF pp -> ZZjj
!         particle_IDs(1)= -12   ! nuebar
!         particle_IDs(2)= 12  ! nue
!         particle_IDs(3)= 14  ! numu 
!         particle_IDs(4)= -14 ! numubar
        
********************
* VBF  Z processes *
********************
      CASE(201,2201)            ! VBF pp -> Z jj 
         particle_IDs(1)= -11   ! e+
         particle_IDs(2)= 11    ! e-
      CASE(202,2202)            ! VBF pp -> Z jj 
         particle_IDs(1)= -13   ! mu+
         particle_IDs(2)= 13    ! mu-
      CASE(203,2203)            ! VBF pp -> Z jj 
         particle_IDs(1)= -15   ! tau+
         particle_IDs(2)= 15    ! tau-
      CASE(204,2204)            ! VBF pp -> Z jj 
         particle_IDs(1)= -12   ! nuebar
         particle_IDs(2)= 12    ! nue
***********************
* VBF W+/W- processes *
***********************
      CASE(301,2301)            ! VBF pp -> W+ jj
         particle_IDS(1) =  12  !nue
         particle_IDs(2) = -11  !e+ 
      CASE(302,2302)            ! VBF pp -> W+ jj
         particle_IDS(1) =  14  !numu
         particle_IDs(2) = -13  !mu+ 
      CASE(303,2303)            ! VBF pp -> W+ jj
         particle_IDS(1) =  16  !nutau
         particle_IDs(2) = -15  !tau+ 
      CASE(304,2304)            ! VBF pp -> W- jj
         particle_IDs(1) =-12   !nuebar
         particle_IDS(2) = 11   !e-
      CASE(305,2305)            ! VBF pp -> W- jj
         particle_IDs(1) =-14   !numubar
         particle_IDS(2) = 13   !mu-
      CASE(306,2306)            ! VBF pp -> W- jj
         particle_IDs(1) =-16   !nutaubar
         particle_IDS(2) = 15   !tau-
**********************
* GF Higgs processes *
**********************
      CASE(4100)
         particle_IDs(1) = 25
      CASE(4101)                ! GF pp -> Hjj -> gammagammajj
         particle_IDs(1)= 22 ! photon
         particle_IDs(2)= 22 ! photon
      CASE(4102) ! GF pp -> Hjj -> mu+mu-jj
         particle_IDs(1)= -13  ! mu+
         particle_IDs(2)= 13 ! mu-
      CASE(4103) ! GF pp -> Hjj -> tau+tau-jj 
         particle_IDs(1)= -15 ! tau+
         particle_IDs(2)= 15  ! tau-
      CASE(4111)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=12  ! nue
         particle_IDs(2)=-11 ! e+
         particle_IDs(3)=11  ! e-
         particle_IDs(4)=-12 ! nuebar    
      CASE(4112)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=12  ! nue
         particle_IDs(2)=-11 ! e+
         particle_IDs(3)=13  ! mu-
         particle_IDs(4)=-14 ! numubar    
      CASE(4113)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=12  ! nue
         particle_IDs(2)=-11 ! e+
         particle_IDs(3)=15  ! tau-
         particle_IDs(4)=-16 ! nutaubar    
      CASE(4114)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=14  ! numu
         particle_IDs(2)=-13 ! mu+
         particle_IDs(3)=11  ! e-
         particle_IDs(4)=-12 ! nuebar    
      CASE(4115)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=14  ! numu
         particle_IDs(2)=-13 ! mu+
         particle_IDs(3)=13  ! mu-
         particle_IDs(4)=-14 ! numubar    
      CASE(4116)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=14  ! numu
         particle_IDs(2)=-13 ! mu+
         particle_IDs(3)=15  ! tau-
         particle_IDs(4)=-16 ! nutaubar    
      CASE(4117)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=16  ! nutau
         particle_IDs(2)=-15 ! tau+
         particle_IDs(3)=11  ! e-
         particle_IDs(4)=-12 ! nuebar    
      CASE(4118)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=16  ! nutau
         particle_IDs(2)=-15 ! tau+
         particle_IDs(3)=13  ! mu-
         particle_IDs(4)=-14 ! numubar    
      CASE(4119)                ! GF pp -> Hjj -> WWjj
         particle_IDs(1)=16  ! nutau
         particle_IDs(2)=-15 ! tau+
         particle_IDs(3)=15  ! tau-
         particle_IDs(4)=-16 ! nutaubar    
********************
* QCD  Z processes *
********************
      CASE(4201)            ! QCD pp -> Z jj 
         particle_IDs(2)= -11   ! e+
         particle_IDs(1)= 11    ! e-
      CASE(4202)            ! QCD pp -> Z jj 
         particle_IDs(2)= -13   ! mu+
         particle_IDs(1)= 13    ! mu-
      CASE(4203)            ! QCD pp -> Z jj 
         particle_IDs(2)= -15   ! tau+
         particle_IDs(1)= 15    ! tau-
      CASE(4204)            ! QCD pp -> Z jj 
         particle_IDs(2)= -12   ! nuebar
         particle_IDs(1)= 12    ! nue      
************************
* Single top processes *
************************
*      CASE(9414)
*         particle_IDs(1)=11     ! e-
*         particle_IDs(2)=-12 ! nuebar
*         particle_IDs(3)=-13 ! mu+
*         particle_IDs(4)=14  ! numu  
*      CASE(9415)
*         particle_IDs(1)=11  ! e-
*         particle_IDs(2)=-12 ! nuebar
         
      END SELECT
     
      RETURN
      END

c*************************************************************************
      SUBROUTINE PrintProcInfo
c*************************************************************************
c     Prints out the information in 'procinfo.dat' depending on which
c     process is selected.
c*************************************************************************
      implicit none
#include "global.inc"

      character*50 filename
      character*7 istr 
      character*100 line
      integer iunit, i1, id, i2
      logical doprint
      filename = "procinfo.dat"
      iunit=10
      doprint = .false.

c Open file
      open (unit=iunit,
     &      file=filename,
     &      status='old',err=77)     


      write (istr,'(a2,I4.4)') "##",procid
      
      print *,""
      print *,"      information on the selected process "
      print *,"-----------------------------------------------"
      
c read lines and print infos
 50   CONTINUE
      read(iunit,'(a100)',end=99,err=99) line    
      if (index(line,"##").ne.0) doprint = .false.
      if (INDEX(line,istr).ne.0) then 
         doprint = .true.
         goto 50
      endif
      if (doprint) print *, line
      GOTO 50
 99   CONTINUE
      
      close(iunit)
      RETURN
 77   CONTINUE
      print *," Error in reading ",filename
      RETURN
      END

c*************************************************************************
      SUBROUTINE InitRealEmissions
c*************************************************************************
c     This reinitializes certain parameters that change for the calculation
c     of the real emissions (e.g. the phase-space dimension)
c*************************************************************************
      implicit none
#include "global.inc"

#ifdef WITH_VVJJ       
      SELECT CASE(procID)
      CASE(100:104,111:119,141:147)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
         call qqhqqgi(0)
      CASE(201:204,301:306)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3   
      CASE(411:419)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
         call qqhqqgi(3)
      CASE(501:506)   
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
      END SELECT

!!!!!!!!!!!!!
c the old processes:
      SELECT CASE(procID)
      CASE(1,7)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
         call qqhqqgi(3)
      CASE(2)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
      CASE(3)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
      CASE(14)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3  
         call qqhqqgi(3)
      CASE(15)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3  
         call qqhqqgi(0)
      CASE(16)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3   
      CASE(17)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3  
         call qqhqqgi(6)
      CASE(22)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3 
      CASE(23)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3

      CASE(50:56)
         n_p = 5
         PS_dimension = 3*(n_p+n_v-nx)-3
         call qqhqqgi(0)
 
      END SELECT
#endif
      RETURN
      END

c*************************************************************************
      SUBROUTINE InitCouplings
c*************************************************************************
c     This is the interface to the koppln-routine for the initialization
c     of coupling constants. Somewhen one has to implement a way to read
c     in the parameters from a file.
c*************************************************************************
      implicit none
#include "global.inc"
#include "koppln.inc"

      logical anom
      integer ewscheme

      call loadfile("vbfnlo.dat",.true.)

      print *," "
      print *,"              Physics parameters"
      print *,"-----------------------------------------------"
      call read_Real("HMASS",xmh,120d0)
      call read_ReaL("TOPMASS",xmt,174.2d0)
      call read_ReaL("ALFA_S",alfas,0.118d0)
      call read_Int("EWSCHEME",ewscheme,3)

      Select Case (EWSCHEME)
      Case(1)
         call read_Real("FERMI_CONST",gf,1.16639d-5)
         call read_Real("ALFA",alfa,7.7579519d-3)
         call read_Real("ZMASS",xmz,91.188d0)
         xmw = -1.d0
         sin2w = -1.d0
      CASE(2)
         call read_Real("FERMI_CONST",gf,1.16639d-5)
         call read_Real("SIN2W",sin2w,0.2312d0)
         call read_Real("ZMASS",xmz,91.188d0)
         xmw = -1.d0
         alfa = -1.d0
      CASE(3)
         call read_Real("FERMI_CONST",gf,1.16639d-5)
         call read_Real("WMASS",xmw,80.423d0)
         call read_Real("ZMASS",xmz,91.188d0)
         alfa = -1.d0
         sin2w = -1.d0
      CASE(4)
         call read_Real("FERMI_CONST",gf,1.16639d-5)
         call read_Real("ALFA",alfa,7.7579519d-3)
         call read_Real("SIN2W",sin2w,0.2312d0)
         call read_Real("WMASS",xmw,80.423d0)
         call read_Real("ZMASS",xmz,91.188d0)
      Case Default
         print*,"Invalid choice for EWSCHEME"
      End Select

      call read_logical("ANOM_CPL",anom,.false.)

      call closefile

      with_anom = .false.
      with_anomHiggs = .false.

#ifdef WITH_VVJJ       

      if (anom) then
         Select Case (procID)
         Case(0411:0419,2411:2419)
            with_anom = anom
c         call read_an_couplings
            call read_anWWcouplings
         Case(0100:0147,2100:2147)
            with_anomHiggs=anom
            call read_anomHVV
         END SELECT
      endif

#endif

      CALL KOPPLN(2)
      call ctrans


      RETURN
      END 

c*************************************************************************
      SUBROUTINE InitEventOutput
c*************************************************************************
c     Initialize the output of events.
c*************************************************************************
      implicit none
      character*20 rootfile 
#include "global.inc"
#ifdef WITH_ROOT
      rootfile = "rootevents.root" ! read this from input file later
      call initrootoutput(max_jets,max_v,rootfile)
#endif
      RETURN
      END

c*************************************************************************
      SUBROUTINE CloseEventOutput
c*************************************************************************
c     finish event output
c*************************************************************************
      implicit none
#ifdef WITH_ROOT         
      call closerootoutput()
#endif
      RETURN
      END

c*************************************************************************
      SUBROUTINE InitPhaseSpace
c*************************************************************************
c     Setup phasespace generators.
c*************************************************************************
      implicit none
#include "global.inc"
      SELECT CASE(procID)
      CASE(100:104,2100:2104)
         call def_ps_vnj(6)
      CASE(111:119,2111:2119)
         call def_ps_vvnj(6,3,4,1,1,1)
      CASE(141:147,2141:2147)
         call def_ps_vvnj(6,2,2,1,1,1)
      CASE(201:204,2201:2204,4201:4204)
         call def_ps_vnj(2)
      CASE(301:303,2301:2303)
         call def_ps_vnj(3)
      CASE(304:306,2304:2306)
         call def_ps_vnj(4)
      CASE(411:419,2411:2419)
         call def_ps_vvnj(-6,3,4,1,1,1)
      CASE(501:506,2501:2506) 
         call def_ps_vvnj(-6,2,2,1,1,1)
      CASE(4100:4103)
         call def_ps_vnj(6) 
      CASE(4111:4119)
         call def_ps_vvnj(6,3,4,1,1,1)
c      CASE(4411:4419)   
c         call def_ps_vvnj(0,3,4,1,1,1)         
      END SELECT
      
!!!!!!!!!!!!!
c the old processes:
      SELECT CASE(procID)
      CASE(1,4,7,8)
         call def_ps_vvnj(-6,3,4,1,1,1)
      CASE(2,3,5,6)
         call def_ps_vvnj(-6,2,2,1,1,1)
      CASE(12)
         call def_ps_vnj(6)
      CASE(13)
         call def_ps_vvnj(6,3,4,1,1,1)
      CASE(14) 
         call def_ps_vvnj(6,3,4,1,1,1)
      CASE(9,15,17,18) 
         call def_ps_vnj(6) 
      CASE(16,19)
         call def_ps_vnj(2) 
      CASE(20)
         call def_ps_vvnj(0,3,4,1,1,1)
      CASE(21)
         call def_ps_vnj(2)
      CASE(22,24)
         call def_ps_vnj(3)
      CASE(23,25)
         call def_ps_vnj(4)
       CASE(50,51,54,55,56)
         call def_ps_vnj(6)
      CASE(52)
         call def_ps_vvnj(6,2,2,1,1,1)
      CASE(53)
         call def_ps_vvnj(6,3,4,1,1,1)
      CASE(60,61,64,65,66)
         call def_ps_vnj(6)
      CASE(62)
         call def_ps_vvnj(6,2,2,1,1,1)
      CASE(63)
         call def_ps_vvnj(6,3,4,1,1,1)

      END SELECT

      RETURN
      END

c*************************************************************************      
      SUBROUTINE InitPDFs(nlo)
c*************************************************************************
c     Initialize parton distribution functions.
c     There should be two pdfsets defined. One for the LO and one for the
c     NLO calculation. 
c*************************************************************************
      implicit none
      
      character*32 pdfname1, pdfname2, pdfname
      integer pdflib
      integer pdfnumber1, pdfnumber2, pdfnumber
      COMMON/PDFparameters/pdflib

      integer nlo
      
c      call loadfile("pdfs.dat")
      call loadfile("vbfnlo.dat",.false.)
      call read_int("PDF_SWITCH",pdflib,0)
      SELECT CASE(pdflib)
      CASE(0)
         pdfname1=''
         pdfname2=''
         pdfnumber1=4
         pdfnumber2=1
      CASE(1)
         call read_string("LO_PDFSET",pdfname1,"PDFsets/cteq6l.LHpdf")
         call read_string("NLO_PDFSET",pdfname2,"PDFsets/cteq6l.LHpdf")
         call read_int("LO_PDFMEMBER",pdfnumber1,0)
         call read_int("NLO_PDFMEMBER",pdfnumber2,0)
      CASE DEFAULT
         print *,"Error: invalid choice of pdf : PDF_SWITCH = ",pdflib
         STOP
      END SELECT
     
      call closefile
      
      if (nlo.eq.0) then
         pdfnumber = pdfnumber1
         pdfname = pdfname1
      else
         pdfnumber = pdfnumber2
         pdfname = pdfname2
      endif

      SELECT CASE(pdflib)
      CASE(0)
         Call SetCtq6(pdfnumber)
#ifdef WITH_LHA
      CASE(1)
         call InitPDFset(pdfname)
         call InitPDF(pdfnumber)
#endif
      END SELECT

      RETURN
      END

c*************************************************************************  
      SUBROUTINE pdfproton(x,q,pdf)
c*************************************************************************
C     This should be the interface for all PDFs, i.e. this subroutine is
c     called inside the matrix elements. 
c*************************************************************************
      implicit none
      real*8 x,q,pdf(-6:6)

      integer pdflib
      COMMON/PDFparameters/pdflib
    
      SELECT CASE(pdflib)
      CASE(0)
         call pftopdg_cteq(x,q,pdf)
#ifdef WITH_LHA
      CASE(1)
         call evolvePDF(x,q,pdf)
#endif
      END SELECT
    

      RETURN
      END
