CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                tens_red5 = 5-point tensors
C    (Dieter Zeppenfeld and Giuseppe Bozzi - November 2006)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine tens_red5(psq, qsq, lsq, tsq,
     #                     pq, pl, pt, ql, qt, lt, 
     #                     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #                     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #                     E0, 
     #                     Eij)
      implicit none
      double precision psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt
      double complex D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
      double complex Dij_2345(3,13),Dij_1345(3,13),Dij_1245(3,13),
     #     Dij_1235(3,13),Dij_1234(3,13)
      double complex E0, Eij(4,46), Eijp(4,46), Eijpp(4,46), Eijo(4,46)
c
c  Determine the Passarino-Veltman(PV) tensor decomposition for the four-point
c  tensor integrals.
c  Denner-Dittmaier(DD) reduction formalism (hep-ph/0509141), overcoming the
c  problem of small Gram determinants, has been used.
c  Step 1) express DD D-coefficients in function of PV D-coefficients
c          (this is done in d2pvtodd subroutine) 
c  Step 2) use DD reduction formalism to obtain the E-coefficients
c          (this is done in this subroutine)
c  Step 3) go back from DD E-coefficients to PV E-coefficients
c          (this is done in this subroutine)
c                                                             d^4k
c   E0; E_mu; E_mu,nu; E_mu,nu,rho; E_mu,nu,rho_sig  =  Int -------
c                                                           (2pi)^4
c
c   1; k_mu; k_mu k_nu; k_mu k_nu k_rho; k_mu k_nu k_rho k_sig
c --------------------------------------------------------------------------
c   k^2  (k+p)^2  (k+p+q)^2  (k+p+q+l)^2  (k+p+q+l+t)^2
c
c  with
c
c   E_mu = p_mu E11  +  q_mu E12  +  l_mu E13  +  t_mu E14
c
c   E_munu = p_mu p_nu E21 + q_mu q_nu E22 + ...
c
c
C INPUT:  psq, pq,...                     kinematics invariants
c         D0_1345 = D00(p+q,l,t)          scalar 4-point functions
c         D0_1245 = D00(p,q+l,t)          in the PV notation
c         D0_1234 = D00(p,q,l)
c         D0_1235 = D00(p,q,l+t)      
c         D0_2345 = D00(q,l,t)

C         Dij_1235(n,m) = D_nm(1,2,3,5) ....    higher D_nm form factors
C                                               as in tens_red3 and tens_red4
c         E0 = E0(p,q,l,t)                      scalar five point function
c
c OUTPUT: Eij(n,m) = E_nm                    form factors in the tensor 
c                                            integrals a la PV
c         nm = 11, 12, 13, 14                ff's for E_mu
c         nm = 21, 22, 23, ..., 211    ff's for E_munu
c         nm = 31, 32, 33, ..., 323, 324     ff's for E_mu,nu,rho
c         nm = 41, 42, 43, ..., 445, 446
c
      double complex R(1:4),res(1:4)

      double precision s02,s03,s12,s13,s14,s23,s24,s34,  s01,s04,
     1       detx4,detx01t,detx02t,detx03t,detx04t,detx00t
      double precision x4(0:4,0:4), x4t(0:4,0:4), x4tt(0:4,0:4,0:4,0:4)
      double complex dd1(4,0:4), dd1n(4,0:4), ed2(4,4), dd2(4,4,0:4), 
     1           ed3(4,4,4), xed3(4,4,4), dd300(4,0:4), ed300(4),
     1           dd3(4,4,4,0:4), dd400(4,4,0:4), dd40000(0:4),
     1           xed4(4,4,4,4), ed4(4,4,4,4), ed400(4,4), ed40000,
     1           xed400(4,4), yed400(4,4)
      double complex zz(4,3)
      integer i,j,k,l
      logical debug, lwr
      parameter (debug=.false.,lwr=.false.)
      character*20 text

c Quantities defined in Section 2 of DD paper

c p1 = p          with p.p = 0
c p2 = p+q
c p3 = p+q+l
c p4 = p+q+l+t    with p4.p4=0
      if(psq.ne.0d0) then
         print*,' psq must be zero. Here psq = ',psq
      endif
      s04 = psq+qsq+lsq+tsq+2*(pq+pl+pt+ql+qt+lt)
c      if (lwr) print*,' p4.p4 = s04 =',s04
c s02:=p2.p2;    
      s02=2*pq+psq+qsq
c s03:=p3.p3;
      s03=lsq+2*pl+2*pq+psq+2*ql+qsq
c s12:=(p2-p1).(p2-p1);
      s12=qsq
c s13:=(p3-p1).(p3-p1);
      s13=lsq+2*ql+qsq
c s14:=(p4-p1).(p4-p1);
      s14=lsq+2*lt+2*ql+qsq+2*qt+tsq
c s23:=(p2-p3).(p2-p3);
      s23=lsq
c s24:=(p2-p4).(p2-p4);
      s24=lsq+2*lt+tsq
c s34:=(p3-p4).(p3-p4);
      s34=tsq
c detx4:=det(x5);
      detx4=2*s14*(s02**2*s13*s34-s02*s03*s12*s34-s02*s03*s13*s24+
     . s02*s03*s14*s23+s03**2*s12*s24)
c detx01t:= -det(xt1);
      detx01t=s02**2*s13*s34+s02**2*s14*s34-s02**2*s34**2-s02*s03*s12
     . *s34-s02*s03*s13*s24+2*s02*s03*s14*s23-s02*s03*s14*s24-s02*
     . s03*s14*s34+2*s02*s03*s24*s34+s02*s12*s34**2-s02*s13*s24*s34
     . -s02*s14*s23*s34+s03**2*s12*s24+s03**2*s14*s24-s03**2*s24**2-
     . s03*s12*s24*s34+s03*s13*s24**2-s03*s14*s23*s24
c detx02t:= det(xt2);
      detx02t=s14*(-s02*s03*s13+s02*s03*s14-s02*s03*s34+2*s02*s13*
     . s34+s03**2*s12-s03**2*s14+s03**2*s24-s03*s12*s34-s03*s13*s24+
     . s03*s14*s23)
c detx03t:= -det(xt3);
      detx03t=s14*(s02**2*s13-s02**2*s14+s02**2*s34-s02*s03*s12+s02*
     . s03*s14-s02*s03*s24-s02*s12*s34-s02*s13*s24+s02*s14*s23+2*
     . s03*s12*s24)
c detx04t:= det(xt4);
      detx04t=-s02**2*s13**2+s02**2*s13*s14+s02**2*s13*s34+2*s02*
     . s03*s12*s13-s02*s03*s12*s14-s02*s03*s12*s34-s02*s03*s13*s14-
     . s02*s03*s13*s24+2*s02*s03*s14*s23-s02*s12*s13*s34+s02*s13**2
     . *s24-s02*s13*s14*s23-s03**2*s12**2+s03**2*s12*s14+s03**2*s12*
     . s24+s03*s12**2*s34-s03*s12*s13*s24-s03*s12*s14*s23

      detx00t=s02**2*s13**2-2*s02**2*s13*s14-2*s02**2*s13*s34+s02
     . **2*s14**2-2*s02**2*s14*s34+s02**2*s34**2-2*s02*s03*s12*
     . s13+2*s02*s03*s12*s14+2*s02*s03*s12*s34+2*s02*s03*s13*
     . s14+2*s02*s03*s13*s24-2*s02*s03*s14**2-4*s02*s03*s14*s23
     . +2*s02*s03*s14*s24+2*s02*s03*s14*s34-2*s02*s03*s24*s34+
     . 2*s02*s12*s13*s34+2*s02*s12*s14*s34-2*s02*s12*s34**2-2
     . *s02*s13**2*s24+2*s02*s13*s14*s23+2*s02*s13*s14*s24-4*
     . s02*s13*s14*s34+2*s02*s13*s24*s34-2*s02*s14**2*s23+2*s02
     . *s14*s23*s34+s03**2*s12**2-2*s03**2*s12*s14-2*s03**2*s12*
     . s24+s03**2*s14**2-2*s03**2*s14*s24+s03**2*s24**2-2*s03*s12
     . **2*s34+2*s03*s12*s13*s24+2*s03*s12*s14*s23-4*s03*s12*
     . s14*s24+2*s03*s12*s14*s34+2*s03*s12*s24*s34+2*s03*s13*
     . s14*s24-2*s03*s13*s24**2-2*s03*s14**2*s23+2*s03*s14*s23*
     . s24+s12**2*s34**2-2*s12*s13*s24*s34-2*s12*s14*s23*s34+s13
     . **2*s24**2-2*s13*s14*s23*s24+s14**2*s23**2
c
      x4t(0,0)=detx00t
      x4t(0,1)=detx01t
      x4t(0,2)=detx02t
      x4t(0,3)=detx03t
      x4t(0,4)=detx04t
      x4t(1,1)=s02**2*s34**2-2*s02*s03*s24*s34+s03**2*s24**2
      x4t(1,2)=s03*s14*(s02*s34-s03*s24)
      x4t(1,3)=s02*s14*(-s02*s34+s03*s24)
      x4t(1,4)=-s02**2*s13*s34+s02*s03*s12*s34+s02*s03*s13*s24-
     &       2*s02*s03*s14*s23-s03**2*s12*s24
      x4t(2,2)=s03**2*s14**2
      x4t(2,3)=-s02*s03*s14**2
      x4t(2,4)=s03*s14*(s02*s13-s03*s12)
      x4t(3,3)=s02**2*s14**2
      x4t(3,4)=s02*s14*(-s02*s13+s03*s12)
      x4t(4,4)=s02**2*s13**2-2*s02*s03*s12*s13+s03**2*s12**2

      do i = 0,3
         do j = i+1,4
            x4t(j,i) = x4t(i,j)
         enddo
      enddo
      do i = 1,4
         do j = 1,4
            x4tt(0,i,0,j) = -(x4t(0,0)*x4t(i,j)-x4t(i,0)*x4t(0,j))/detx4
            do k = 1,4
               x4tt(0,i,j,k) = (x4t(0,k)*x4t(i,j)-x4t(i,k)*x4t(0,j))
     &                         /detx4
               x4tt(j,k,0,i) = x4tt(0,i,j,k)
            enddo
         enddo
      enddo

c  Rank-1

      if(lwr) then
         print*,'--------------------------------------------'
         print*,' enter tensor redux Eij '
      endif

      do i = 1,4
         r(i) = (x4t(i,1)*(d0_1345-d0_2345) +
     &           x4t(i,2)*(d0_1245-d0_2345) +
     &           x4t(i,3)*(d0_1235-d0_2345) +
     &           x4t(i,4)*(d0_1234-d0_2345) - 
     &           x4t(i,0)*d0_2345)/detx4
      enddo

      eij(1,4) = r(4)
      do i = 3,1,-1
         eij(1,i) = eij(1,i+1)+r(i)
      enddo

c  Rank-2

      Eij(2,11) = (detx01t*(dij_1345(2,7)-dij_2345(2,7)) +
     &              detx02t*(dij_1245(2,7)-dij_2345(2,7)) +
     &              detx03t*(dij_1235(2,7)-dij_2345(2,7)) +
     &              detx04t*(dij_1234(2,7)-dij_2345(2,7)) )/detx4

      call d2pvtodd(4,d0_1234,dij_1234,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(3,d0_1235,dij_1235,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(2,d0_1245,dij_1245,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(1,d0_1345,dij_1345,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(0,d0_2345,dij_2345,dd1,dd2,dd3,dd300,dd400,dd40000)

      do i = 1,4
         do j = 1,4
            ed2(i,j) = x4t(i,1)*(dd1(j,1)-dd1(j,0)) +
     &                 x4t(i,2)*(dd1(j,2)-dd1(j,0)) +
     &                 x4t(i,3)*(dd1(j,3)-dd1(j,0)) +
     &                 x4t(i,4)*(dd1(j,4)-dd1(j,0)) -
     &                 x4t(i,0)*dd1(j,0) 
            ed2(i,j) = ed2(i,j) - 2*(
     &        x4tt(i,1,0,j)*(dij_1345(2,7)-dij_2345(2,7)) +
     &        x4tt(i,2,0,j)*(dij_1245(2,7)-dij_2345(2,7)) +
     &        x4tt(i,3,0,j)*(dij_1235(2,7)-dij_2345(2,7)) +
     &        x4tt(i,4,0,j)*(dij_1234(2,7)-dij_2345(2,7)) )
         enddo
      enddo
      do i = 1,4
         do j = i,4
            ed2(i,j) = (ed2(i,j)+ed2(j,i))/(2*detx4)
         enddo
      enddo

      eij(2,4) = ed2(4,4)
      eij(2,10) = ed2(4,4) + ed2(3,4)
      eij(2,9) = eij(2,10)+ ed2(2,4)
      eij(2,7) = eij(2,9) + ed2(1,4)
      eij(2,3) = ed2(4,4) + 2*ed2(3,4) + ed2(3,3)
      eij(2,8) = eij(2,3) + ed2(2,4) + ed2(2,3)
      eij(2,6) = eij(2,8) + ed2(1,4) + ed2(1,3)
      eij(2,2) = eij(2,8) + ed2(2,4) + ed2(2,3) + ed2(2,2)
      eij(2,5) = eij(2,2) + ed2(1,4) + ed2(1,3) + ed2(1,2)
      eij(2,1) = eij(2,5) + ed2(1,4) + ed2(1,3) + ed2(1,2) + ed2(1,1)

c  Rank-3

      do i = 1,4
         ED300(i) = (2*(x4t(1,0)*(dd300(i,1)-dd300(i,0)) +
     &                  x4t(2,0)*(dd300(i,2)-dd300(i,0)) +
     &                  x4t(3,0)*(dd300(i,3)-dd300(i,0)) +
     &                  x4t(4,0)*(dd300(i,4)-dd300(i,0)) ) +
     &         x4t(i,1)*(dij_1345(2,7)-dij_2345(2,7)) +
     &         x4t(i,2)*(dij_1245(2,7)-dij_2345(2,7)) +
     &         x4t(i,3)*(dij_1235(2,7)-dij_2345(2,7)) +
     &         x4t(i,4)*(dij_1234(2,7)-dij_2345(2,7)) -
     &               x4t(i,0)*dij_2345(2,7))/(3*detx4)
         do j = 1,4
            do k = 1,4
               xed3(i,j,k) =
     &                       x4t(i,1)*(dd2(j,k,1)-dd2(j,k,0)) +
     &                       x4t(i,2)*(dd2(j,k,2)-dd2(j,k,0)) +
     &                       x4t(i,3)*(dd2(j,k,3)-dd2(j,k,0)) +
     &                       x4t(i,4)*(dd2(j,k,4)-dd2(j,k,0)) 
     &               - x4t(i,0)*dd2(j,k,0) 
     &        - 2*( x4tt(i,1,0,j)*(dd300(k,1)-dd300(k,0)) + 
     &              x4tt(i,2,0,j)*(dd300(k,2)-dd300(k,0)) +
     &              x4tt(i,3,0,j)*(dd300(k,3)-dd300(k,0)) +
     &              x4tt(i,4,0,j)*(dd300(k,4)-dd300(k,0)) +
     &              x4tt(i,1,0,k)*(dd300(j,1)-dd300(j,0)) + 
     &              x4tt(i,2,0,k)*(dd300(j,2)-dd300(j,0)) +
     &              x4tt(i,3,0,k)*(dd300(j,3)-dd300(j,0)) +
     &              x4tt(i,4,0,k)*(dd300(j,4)-dd300(j,0)) )
            enddo
         enddo
      enddo
      do i = 1,4
         do j = i,4
            do k = j,4
         ed3(i,j,k) = (xed3(i,j,k)+xed3(j,i,k)+xed3(k,i,j))/(3*detx4)
            enddo
         enddo
      enddo

      eij(3,1) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3) + 
     &         3*ed3(2,4,4) + 6*ed3(2,3,4) + 3*ed3(2,3,3) +3*ed3(2,2,4)+
     &         3*ed3(2,2,3) + ed3(2,2,2) + 3*ed3(1,4,4) + 6*ed3(1,3,4) + 
     &         3*ed3(1,3,3) + 6*ed3(1,2,4) + 6*ed3(1,2,3)+3*ed3(1,2,2) + 
     &         3*ed3(1,1,4) + 3*ed3(1,1,3) + 3*ed3(1,1,2) + ed3(1,1,1)
      eij(3,2) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3) + 
     &     3*ed3(2,4,4) + 6*ed3(2,3,4) + 3*ed3(2,3,3) + 3*ed3(2,2,4) + 
     &     3*ed3(2,2,3) + ed3(2,2,2)
      eij(3,3) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3)
      eij(3,4) = ed3(4,4,4)
      eij(3,5) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3) + 
     &         3*ed3(2,4,4) + 6*ed3(2,3,4) + 3*ed3(2,3,3) +3*ed3(2,2,4)+ 
     &         3*ed3(2,2,3) + ed3(2,2,2) + 2*ed3(1,4,4) + 4*ed3(1,3,4) + 
     &         2*ed3(1,3,3) + 4*ed3(1,2,4) + 4*ed3(1,2,3)+2*ed3(1,2,2) + 
     &           ed3(1,1,4) + ed3(1,1,3) + ed3(1,1,2)
      eij(3,6) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3) + 
     &         2*ed3(2,4,4) + 4*ed3(2,3,4) + 2*ed3(2,3,3) + ed3(2,2,4) + 
     &           ed3(2,2,3) + 2*ed3(1,4,4) + 4*ed3(1,3,4) +2*ed3(1,3,3)+ 
     &         2*ed3(1,2,4) + 2*ed3(1,2,3) + ed3(1,1,4) + ed3(1,1,3)
      eij(3,7) = ed3(4,4,4) + 2*ed3(3,4,4) + ed3(3,3,4) + 2*ed3(2,4,4) + 
     &         2*ed3(2,3,4) + ed3(2,2,4) + 2*ed3(1,4,4) + 2*ed3(1,3,4) + 
     &         2*ed3(1,2,4) + ed3(1,1,4) 
      eij(3,8) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3) + 
     &     3*ed3(2,4,4) + 6*ed3(2,3,4) + 3*ed3(2,3,3) + 3*ed3(2,2,4) + 
     &     3*ed3(2,2,3) + ed3(2,2,2) + ed3(1,4,4) + 2*ed3(1,3,4) + 
     &       ed3(1,3,3) + 2*ed3(1,2,4) + 2*ed3(1,2,3) + ed3(1,2,2)
      eij(3,9) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3) + 
     &     2*ed3(2,4,4) + 4*ed3(2,3,4) + 2*ed3(2,3,3) + ed3(2,2,4) + 
     &       ed3(2,2,3)
      eij(3,10) = ed3(4,4,4) + 2*ed3(3,4,4) + ed3(3,3,4) + 2*ed3(2,4,4)+ 
     &          2*ed3(2,3,4) + ed3(2,2,4)
      eij(3,11) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3)+ 
     &     ed3(2,4,4)+ 2*ed3(2,3,4) + ed3(2,3,3) + ed3(1,4,4) + 
     &   2*ed3(1,3,4) + ed3(1,3,3)
      eij(3,12) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3)+ 
     &            ed3(2,4,4) + 2*ed3(2,3,4) + ed3(2,3,3)
      eij(3,13) = ed3(4,4,4) + 2*ed3(3,4,4) + ed3(3,3,4)
      eij(3,14) = ed3(4,4,4) + ed3(3,4,4) + ed3(2,4,4) + ed3(1,4,4)
      eij(3,15) = ed3(4,4,4) + ed3(3,4,4) + ed3(2,4,4)
      eij(3,16) = ed3(4,4,4) + ed3(3,4,4)
      eij(3,17) = ed3(4,4,4) + 3*ed3(3,4,4) + 3*ed3(3,3,4) + ed3(3,3,3)+ 
     &     2*ed3(2,4,4) + 4*ed3(2,3,4) + 2*ed3(2,3,3) + ed3(2,2,4) + 
     &       ed3(2,2,3) + ed3(1,4,4) + 2*ed3(1,3,4) + ed3(1,3,3) + 
     &       ed3(1,2,4) + ed3(1,2,3)
      eij(3,18) = ed3(4,4,4) + 2*ed3(3,4,4) + ed3(3,3,4) + 2*ed3(2,4,4)+ 
     &  2*ed3(2,3,4) + ed3(2,2,4) + ed3(1,4,4) + ed3(1,3,4) + ed3(1,2,4)
      eij(3,19) = ed3(4,4,4) + 2*ed3(3,4,4) + ed3(3,3,4) + ed3(2,4,4) + 
     &     ed3(2,3,4) + ed3(1,4,4) + ed3(1,3,4)
      eij(3,20) = ed3(4,4,4) + 2*ed3(3,4,4) + ed3(3,3,4) + ed3(2,4,4) + 
     &            ed3(2,3,4)

      eij(3,24) = ed300(4)
      eij(3,23) = eij(3,24)+ed300(3)
      eij(3,22) = eij(3,23)+ed300(2)
      eij(3,21) = eij(3,22)+ed300(1)

c  Rank-4 (giuseppe)

      ed40000=(
     &     x4t(1,0)*(dd40000(1)-dd40000(0))+
     &     x4t(2,0)*(dd40000(2)-dd40000(0))+
     &     x4t(3,0)*(dd40000(3)-dd40000(0))+
     &     x4t(4,0)*(dd40000(4)-dd40000(0))
     &     )/detx4

      do i = 1,4
         do j = 1,4
            xed400(i,j) = 2*(
     &           x4t(1,0)*(dd400(i,j,1)-dd400(i,j,0)) +
     &           x4t(2,0)*(dd400(i,j,2)-dd400(i,j,0)) +
     &           x4t(3,0)*(dd400(i,j,3)-dd400(i,j,0)) +
     &           x4t(4,0)*(dd400(i,j,4)-dd400(i,j,0)) )
            yed400(i,j) =
     &           x4t(i,1)*(dd300(j,1)-dd300(j,0)) +
     &           x4t(i,2)*(dd300(j,2)-dd300(j,0)) +
     &           x4t(i,3)*(dd300(j,3)-dd300(j,0)) +
     &           x4t(i,4)*(dd300(j,4)-dd300(j,0)) -
     &           x4t(i,0)*dd300(j,0) -
     &           2*(
     &           x4tt(i,1,0,j)*(dd40000(1)-dd40000(0)) +
     &           x4tt(i,2,0,j)*(dd40000(2)-dd40000(0)) +
     &           x4tt(i,3,0,j)*(dd40000(3)-dd40000(0)) +
     &           x4tt(i,4,0,j)*(dd40000(4)-dd40000(0)) )
            do k = 1,4
               do l = 1,4
                  xed4(i,j,k,l) =
     &                 x4t(i,1)*(dd3(j,k,l,1)-dd3(j,k,l,0)) +
     &                 x4t(i,2)*(dd3(j,k,l,2)-dd3(j,k,l,0)) +
     &                 x4t(i,3)*(dd3(j,k,l,3)-dd3(j,k,l,0)) +
     &                 x4t(i,4)*(dd3(j,k,l,4)-dd3(j,k,l,0)) - 
     &                 x4t(i,0)*dd3(j,k,l,0) -
     &                 2*(
     &                 x4tt(i,1,0,j)*(dd400(k,l,1)-dd400(k,l,0)) + 
     &                 x4tt(i,2,0,j)*(dd400(k,l,2)-dd400(k,l,0)) +
     &                 x4tt(i,3,0,j)*(dd400(k,l,3)-dd400(k,l,0)) +
     &                 x4tt(i,4,0,j)*(dd400(k,l,4)-dd400(k,l,0)) +
     &                 x4tt(i,1,0,k)*(dd400(j,l,1)-dd400(j,l,0)) + 
     &                 x4tt(i,2,0,k)*(dd400(j,l,2)-dd400(j,l,0)) +
     &                 x4tt(i,3,0,k)*(dd400(j,l,3)-dd400(j,l,0)) +
     &                 x4tt(i,4,0,k)*(dd400(j,l,4)-dd400(j,l,0)) +
     &                 x4tt(i,1,0,l)*(dd400(j,k,1)-dd400(j,k,0)) + 
     &                 x4tt(i,2,0,l)*(dd400(j,k,2)-dd400(j,k,0)) +
     &                 x4tt(i,3,0,l)*(dd400(j,k,3)-dd400(j,k,0)) +
     &                 x4tt(i,4,0,l)*(dd400(j,k,4)-dd400(j,k,0)) )
               enddo
            enddo
         enddo
      enddo
      do i = 1,4
         do j = i,4
            yed400(i,j)=yed400(i,j)+yed400(j,i)
            do k = j,4
               do l = k,4
                  ed4(i,j,k,l) = (xed4(i,j,k,l)+xed4(j,i,k,l)+
     &                            xed4(k,j,i,l)+xed4(l,j,k,i))/(4*detx4)
               enddo
            enddo
         enddo
      enddo

      do i=1,4
         do j=1,4
            ed400(i,j)=(xed400(i,j)+yed400(i,j))/(4*detx4)
         enddo
      enddo
      
         eij(4,1) = 
     /      ed4(1,1,1,1) + 4*ed4(1,1,1,2) + 4*ed4(1,1,1,3) + 
     /      4*ed4(1,1,1,4) + 6*ed4(1,1,2,2) + 12*ed4(1,1,2,3) + 
     /      12*ed4(1,1,2,4) + 6*ed4(1,1,3,3) + 12*ed4(1,1,3,4) + 
     /      6*ed4(1,1,4,4) + 4*ed4(1,2,2,2) + 12*ed4(1,2,2,3) + 
     /      12*ed4(1,2,2,4) + 12*ed4(1,2,3,3) + 24*ed4(1,2,3,4) + 
     /      12*ed4(1,2,4,4) + 4*ed4(1,3,3,3) + 12*ed4(1,3,3,4) + 
     /      12*ed4(1,3,4,4) + 4*ed4(1,4,4,4) + ed4(2,2,2,2) + 
     /      4*ed4(2,2,2,3) + 4*ed4(2,2,2,4) + 6*ed4(2,2,3,3) + 
     /      12*ed4(2,2,3,4) + 6*ed4(2,2,4,4) + 4*ed4(2,3,3,3) + 
     /      12*ed4(2,3,3,4) + 12*ed4(2,3,4,4) + 4*ed4(2,4,4,4) + 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 
     /      4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,5) = 
     /      ed4(1,1,1,2) + ed4(1,1,1,3) + 
     /      ed4(1,1,1,4) + 3*ed4(1,1,2,2) + 6*ed4(1,1,2,3) + 
     /      6*ed4(1,1,2,4) + 3*ed4(1,1,3,3) + 6*ed4(1,1,3,4) + 
     /      3*ed4(1,1,4,4) + 3*ed4(1,2,2,2) + 9*ed4(1,2,2,3) + 
     /      9*ed4(1,2,2,4) + 9*ed4(1,2,3,3) + 18*ed4(1,2,3,4) + 
     /      9*ed4(1,2,4,4) + 3*ed4(1,3,3,3) + 9*ed4(1,3,3,4) + 
     /      9*ed4(1,3,4,4) + 3*ed4(1,4,4,4) + ed4(2,2,2,2) + 
     /      4*ed4(2,2,2,3) + 4*ed4(2,2,2,4) + 6*ed4(2,2,3,3) + 
     /      12*ed4(2,2,3,4) + 6*ed4(2,2,4,4) + 4*ed4(2,3,3,3) + 
     /      12*ed4(2,3,3,4) + 12*ed4(2,3,4,4) + 4*ed4(2,4,4,4) + 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 
     /      4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,6) = 
     /      ed4(1,1,1,3) + ed4(1,1,1,4) + 
     /      3*ed4(1,1,2,3) + 3*ed4(1,1,2,4) + 3*ed4(1,1,3,3) + 
     /      6*ed4(1,1,3,4) + 3*ed4(1,1,4,4) + 3*ed4(1,2,2,3) + 
     /      3*ed4(1,2,2,4) + 6*ed4(1,2,3,3) + 12*ed4(1,2,3,4) + 
     /      6*ed4(1,2,4,4) + 3*ed4(1,3,3,3) + 9*ed4(1,3,3,4) + 
     /      9*ed4(1,3,4,4) + 3*ed4(1,4,4,4) + ed4(2,2,2,3) + 
     /      ed4(2,2,2,4) + 3*ed4(2,2,3,3) + 6*ed4(2,2,3,4) + 
     /      3*ed4(2,2,4,4) + 3*ed4(2,3,3,3) + 9*ed4(2,3,3,4) + 
     /      9*ed4(2,3,4,4) + 3*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,17) = 
     /      ed4(1,1,2,2) + 2*ed4(1,1,2,3) + 2*ed4(1,1,2,4) + 
     /      ed4(1,1,3,3) + 2*ed4(1,1,3,4) + ed4(1,1,4,4) + 
     /      2*ed4(1,2,2,2) + 6*ed4(1,2,2,3) + 6*ed4(1,2,2,4) + 
     /      6*ed4(1,2,3,3) + 12*ed4(1,2,3,4) + 6*ed4(1,2,4,4) + 
     /      2*ed4(1,3,3,3) + 6*ed4(1,3,3,4) + 6*ed4(1,3,4,4) + 
     /      2*ed4(1,4,4,4) + ed4(2,2,2,2) + 4*ed4(2,2,2,3) + 
     /      4*ed4(2,2,2,4) + 6*ed4(2,2,3,3) + 12*ed4(2,2,3,4) + 
     /      6*ed4(2,2,4,4) + 4*ed4(2,3,3,3) + 12*ed4(2,3,3,4) + 
     /      12*ed4(2,3,4,4) + 4*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,23) = 
     /      ed4(1,1,2,3) + ed4(1,1,2,4) + ed4(1,1,3,3) + 
     /      2*ed4(1,1,3,4) + ed4(1,1,4,4) + 2*ed4(1,2,2,3) + 
     /      2*ed4(1,2,2,4) + 4*ed4(1,2,3,3) + 8*ed4(1,2,3,4) + 
     /      4*ed4(1,2,4,4) + 2*ed4(1,3,3,3) + 6*ed4(1,3,3,4) + 
     /      6*ed4(1,3,4,4) + 2*ed4(1,4,4,4) + ed4(2,2,2,3) + 
     /      ed4(2,2,2,4) + 3*ed4(2,2,3,3) + 6*ed4(2,2,3,4) + 
     /      3*ed4(2,2,4,4) + 3*ed4(2,3,3,3) + 9*ed4(2,3,3,4) + 
     /      9*ed4(2,3,4,4) + 3*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,18) = 
     /      ed4(1,1,3,3) + 2*ed4(1,1,3,4) + ed4(1,1,4,4) + 
     /      2*ed4(1,2,3,3) + 4*ed4(1,2,3,4) + 2*ed4(1,2,4,4) + 
     /      2*ed4(1,3,3,3) + 6*ed4(1,3,3,4) + 6*ed4(1,3,4,4) + 
     /      2*ed4(1,4,4,4) + ed4(2,2,3,3) + 2*ed4(2,2,3,4) + 
     /      ed4(2,2,4,4) + 2*ed4(2,3,3,3) + 6*ed4(2,3,3,4) + 
     /      6*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,8) = 
     /      ed4(1,2,2,2) + 3*ed4(1,2,2,3) + 3*ed4(1,2,2,4) + 
     /      3*ed4(1,2,3,3) + 6*ed4(1,2,3,4) + 3*ed4(1,2,4,4) + 
     /      ed4(1,3,3,3) + 3*ed4(1,3,3,4) + 3*ed4(1,3,4,4) + 
     /      ed4(1,4,4,4) + ed4(2,2,2,2) + 4*ed4(2,2,2,3) + 
     /      4*ed4(2,2,2,4) + 6*ed4(2,2,3,3) + 12*ed4(2,2,3,4) + 
     /      6*ed4(2,2,4,4) + 4*ed4(2,3,3,3) + 12*ed4(2,3,3,4) + 
     /      12*ed4(2,3,4,4) + 4*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,26) = 
     /     ed4(1,2,2,3) + ed4(1,2,2,4) + 2*ed4(1,2,3,3) + 
     /      4*ed4(1,2,3,4) + 2*ed4(1,2,4,4) + ed4(1,3,3,3) + 
     /      3*ed4(1,3,3,4) + 3*ed4(1,3,4,4) + ed4(1,4,4,4) + 
     /      ed4(2,2,2,3) + ed4(2,2,2,4) + 3*ed4(2,2,3,3) + 
     /      6*ed4(2,2,3,4) + 3*ed4(2,2,4,4) + 3*ed4(2,3,3,3) + 
     /      9*ed4(2,3,3,4) + 9*ed4(2,3,4,4) + 3*ed4(2,4,4,4) + 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 
     /      4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,29) = 
     /      ed4(1,2,3,3) + 2*ed4(1,2,3,4) + 
     /      ed4(1,2,4,4) + ed4(1,3,3,3) + 3*ed4(1,3,3,4) + 
     /      3*ed4(1,3,4,4) + ed4(1,4,4,4) + ed4(2,2,3,3) + 
     /      2*ed4(2,2,3,4) + ed4(2,2,4,4) + 2*ed4(2,3,3,3) + 
     /      6*ed4(2,3,3,4) + 6*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 
     /      4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,11) = 
     /      ed4(1,3,3,3) + 3*ed4(1,3,3,4) + 
     /      3*ed4(1,3,4,4) + ed4(1,4,4,4) + ed4(2,3,3,3) + 
     /      3*ed4(2,3,3,4) + 3*ed4(2,3,4,4) + ed4(2,4,4,4) + 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 
     /      4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,2) = 
     /      ed4(2,2,2,2) + 4*ed4(2,2,2,3) + 
     /      4*ed4(2,2,2,4) + 6*ed4(2,2,3,3) + 12*ed4(2,2,3,4) + 
     /      6*ed4(2,2,4,4) + 4*ed4(2,3,3,3) + 12*ed4(2,3,3,4) + 
     /      12*ed4(2,3,4,4) + 4*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,9) = 
     /     ed4(2,2,2,3) + ed4(2,2,2,4) + 3*ed4(2,2,3,3) + 
     /      6*ed4(2,2,3,4) + 3*ed4(2,2,4,4) + 3*ed4(2,3,3,3) + 
     /      9*ed4(2,3,3,4) + 9*ed4(2,3,4,4) + 3*ed4(2,4,4,4) + 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 
     /      4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,20) = 
     /      ed4(2,2,3,3) + 2*ed4(2,2,3,4) + 
     /      ed4(2,2,4,4) + 2*ed4(2,3,3,3) + 6*ed4(2,3,3,4) + 
     /      6*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + ed4(3,3,3,3) + 
     /      4*ed4(3,3,3,4) + 6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,12) = 
     /      ed4(2,3,3,3) + 3*ed4(2,3,3,4) + 3*ed4(2,3,4,4) + 
     /      ed4(2,4,4,4) + ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 
     /      6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,3) = 
     /      ed4(3,3,3,3) + 4*ed4(3,3,3,4) + 
     /      6*ed4(3,3,4,4) + 4*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,7) = 
     /      ed4(1,1,1,4) + 3*ed4(1,1,2,4) + 
     /      3*ed4(1,1,3,4) + 3*ed4(1,1,4,4) + 3*ed4(1,2,2,4) + 
     /      6*ed4(1,2,3,4) + 6*ed4(1,2,4,4) + 3*ed4(1,3,3,4) + 
     /      6*ed4(1,3,4,4) + 3*ed4(1,4,4,4) + ed4(2,2,2,4) + 
     /      3*ed4(2,2,3,4) + 3*ed4(2,2,4,4) + 3*ed4(2,3,3,4) + 
     /      6*ed4(2,3,4,4) + 3*ed4(2,4,4,4) + ed4(3,3,3,4) + 
     /      3*ed4(3,3,4,4) + 3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,24) = 
     /      ed4(1,1,2,4) + ed4(1,1,3,4) + 
     /      ed4(1,1,4,4) + 2*ed4(1,2,2,4) + 4*ed4(1,2,3,4) + 
     /      4*ed4(1,2,4,4) + 2*ed4(1,3,3,4) + 4*ed4(1,3,4,4) + 
     /      2*ed4(1,4,4,4) + ed4(2,2,2,4) + 3*ed4(2,2,3,4) + 
     /      3*ed4(2,2,4,4) + 3*ed4(2,3,3,4) + 6*ed4(2,3,4,4) + 
     /      3*ed4(2,4,4,4) + ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 
     /      3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,25) = 
     /      ed4(1,1,3,4) + ed4(1,1,4,4) + 
     /      2*ed4(1,2,3,4) + 2*ed4(1,2,4,4) + 2*ed4(1,3,3,4) + 
     /      4*ed4(1,3,4,4) + 2*ed4(1,4,4,4) + ed4(2,2,3,4) + 
     /      ed4(2,2,4,4) + 2*ed4(2,3,3,4) + 4*ed4(2,3,4,4) + 
     /      2*ed4(2,4,4,4) + ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 
     /      3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,19) = 
     /      ed4(1,1,4,4) + 2*ed4(1,2,4,4) + 
     /      2*ed4(1,3,4,4) + 2*ed4(1,4,4,4) + ed4(2,2,4,4) + 
     /      2*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + ed4(3,3,4,4) + 
     /      2*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,27) = 
     /      ed4(1,2,2,4) + 2*ed4(1,2,3,4) + 
     /      2*ed4(1,2,4,4) + ed4(1,3,3,4) + 2*ed4(1,3,4,4) + 
     /      ed4(1,4,4,4) + ed4(2,2,2,4) + 3*ed4(2,2,3,4) + 
     /      3*ed4(2,2,4,4) + 3*ed4(2,3,3,4) + 6*ed4(2,3,4,4) + 
     /      3*ed4(2,4,4,4) + ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 
     /      3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,35) = 
     /      ed4(1,2,3,4) + ed4(1,2,4,4) + 
     /      ed4(1,3,3,4) + 2*ed4(1,3,4,4) + ed4(1,4,4,4) + 
     /      ed4(2,2,3,4) + ed4(2,2,4,4) + 2*ed4(2,3,3,4) + 
     /      4*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + ed4(3,3,3,4) + 
     /      3*ed4(3,3,4,4) + 3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,32) = 
     /      ed4(1,2,4,4) + ed4(1,3,4,4) + 
     /      ed4(1,4,4,4) + ed4(2,2,4,4) + 2*ed4(2,3,4,4) + 
     /      2*ed4(2,4,4,4) + ed4(3,3,4,4) + 2*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,30) = 
     /     ed4(1,3,3,4) + 2*ed4(1,3,4,4) + ed4(1,4,4,4) + 
     /      ed4(2,3,3,4) + 2*ed4(2,3,4,4) + ed4(2,4,4,4) + 
     /      ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 3*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,33) = 
     /      ed4(1,3,4,4) + ed4(1,4,4,4) + ed4(2,3,4,4) + ed4(2,4,4,4) +
     /      ed4(3,3,4,4) + 2*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,14) = 
     /      ed4(1,4,4,4) + ed4(2,4,4,4) + 
     /      ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,10) = 
     /      ed4(2,2,2,4) + 3*ed4(2,2,3,4) + 
     /      3*ed4(2,2,4,4) + 3*ed4(2,3,3,4) + 6*ed4(2,3,4,4) + 
     /      3*ed4(2,4,4,4) + ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 
     /      3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,28) = 
     /      ed4(2,2,3,4) + ed4(2,2,4,4) + 
     /      2*ed4(2,3,3,4) + 4*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + 
     /      ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 3*ed4(3,4,4,4) + 
     /      ed4(4,4,4,4)
         eij(4,21) = 
     /      ed4(2,2,4,4) + 2*ed4(2,3,4,4) + 2*ed4(2,4,4,4) + 
     /      ed4(3,3,4,4) + 2*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,31) = 
     /      ed4(2,3,3,4) + 2*ed4(2,3,4,4) + 
     /      ed4(2,4,4,4) + ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 
     /      3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,34) = 
     /      ed4(2,3,4,4) + ed4(2,4,4,4) + 
     /      ed4(3,3,4,4) + 2*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,15) = 
     /      ed4(2,4,4,4) + ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,13) = 
     /      ed4(3,3,3,4) + 3*ed4(3,3,4,4) + 
     /      3*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,22) = 
     /      ed4(3,3,4,4) + 2*ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,16) = 
     /      ed4(3,4,4,4) + ed4(4,4,4,4)
         eij(4,4) = 
     /      ed4(4,4,4,4)
         eij(4,36) = 
     /      ed400(1,1) + 2*ed400(1,2) + 
     /      2*ed400(1,3) + 2*ed400(1,4) + ed400(2,2) + 
     /      2*ed400(2,3) + 2*ed400(2,4) + ed400(3,3) + 
     /      2*ed400(3,4) + ed400(4,4)
         eij(4,40) = 
     /      ed400(1,2) + ed400(1,3) + 
     /      ed400(1,4) + ed400(2,2) + 2*ed400(2,3) + 
     /      2*ed400(2,4) + ed400(3,3) + 2*ed400(3,4) + 
     /      ed400(4,4)
	 eij(4,41) = 
     /     ed400(1,3) + ed400(1,4) + ed400(2,3) + ed400(2,4) + 
     /      ed400(3,3) + 2*ed400(3,4) + ed400(4,4)
         eij(4,42) = 
     /      ed400(1,4) + ed400(2,4) + 
     /      ed400(3,4) + ed400(4,4)
         eij(4,37) = 
     /      ed400(2,2) + 2*ed400(2,3) + 
     /      2*ed400(2,4) + ed400(3,3) + 2*ed400(3,4) + 
     /      ed400(4,4)
	 eij(4,43) = 
     /     ed400(2,3) + ed400(2,4) + ed400(3,3) + 
     /      2*ed400(3,4) + ed400(4,4)
         eij(4,44) = 
     /      ed400(2,4) + ed400(3,4) + ed400(4,4)
         eij(4,38) = 
     /      ed400(3,3) + ed400(4,4) + 2*ed400(3,4)
         eij(4,45) = 
     /      ed400(3,4) + ed400(4,4)
         eij(4,39) = 
     /      ed400(4,4)
         eij(4,46) = 
     /      ed40000

      if(lwr) then
         call test_tensor_pent(psq, qsq, lsq, tsq,
     #                     pq, pl, pt, ql, qt, lt, 
     #                     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #                     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #                     E0, 
     #                     Eij)
         read(*,'(a)') text
      endif
      end


      subroutine prod_mat_col4(M,col,res)
      implicit none
      real * 8 M(1:4,1:4)
      complex * 16 col(1:4), res(1:4)
      integer i,j
      do i=1,4 
         res(i) = 0d0
      enddo
      do i=1,4
         do j=1,4
            res(i) = res(i) + M(i,j)*col(j)
         enddo
      enddo
      end

      subroutine d2pvtodd(n,d0,dij,dd1,dd2,dd3,dd300,dd400,dd40000)
      implicit none
      double complex d0,dij(3,13) 
      double complex dd1(4,0:4),dd2(4,4,0:4),dd3(4,4,4,0:4)
      double complex dd300(4,0:4),dd400(4,4,0:4),dd40000(0:4)
      integer i,j,n,irn(3)

c  Initialization

      if (n.gt.0) then
         do i = 1,4
            DD2(i,n,n) = 0
            DD2(n,i,n) = 0
            do j = 1,4
               DD3(i,j,n,n) = 0
               DD3(i,n,j,n) = 0
               DD3(n,i,j,n) = 0
            enddo
            DD400(i,n,n)=0
            DD400(n,i,n)=0
         enddo
         DD300(n,n) = 0
      endif
      do i = 1,n-1
         irn(i)=i
      enddo
      do i = max(1,n),3
         irn(i)=i+1
      enddo

c D functions up to rank 3 (giuseppe)
c WARNING: Previously unused Dij(1,7-13) variables are used
c to define D00ij and D0000 functions! (See tensor_redux_Dij.f)
      DD3(irn(1),irn(1),irn(1),n) = dij(3,1)-dij(3,2)
     \     +3*dij(3,6)-3*dij(3,4)
      DD3(irn(1),irn(1),irn(2),n) = dij(3,4)+dij(3,2)-dij(3,8)
     \     -dij(3,5)+2*dij(3,10)-2*dij(3,6)
      DD3(irn(1),irn(1),irn(3),n) = dij(3,5)+dij(3,8)-2*dij(3,10)
      DD3(irn(1),irn(2),irn(2),n) = dij(3,6)+dij(3,7)-dij(3,2)
     \     -dij(3,9)+2*dij(3,8)-2*dij(3,10)
      DD3(irn(1),irn(2),irn(3),n) = dij(3,10)+dij(3,9)-dij(3,7)-dij(3,8)
      DD3(irn(2),irn(2),irn(2),n) = dij(3,2)-dij(3,3)
     \     -3*dij(3,8)+3*dij(3,9)
      DD3(irn(2),irn(2),irn(3),n) = dij(3,8)+dij(3,3)-2*dij(3,9)
      DD3(irn(1),irn(3),irn(3),n) = dij(3,7)-dij(3,9)
      DD3(irn(2),irn(3),irn(3),n) = dij(3,9)-dij(3,3)
      DD3(irn(3),irn(3),irn(3),n) = dij(3,3)

      DD3(irn(1),irn(2),irn(1),n) = DD3(irn(1),irn(1),irn(2),n)
      DD3(irn(2),irn(1),irn(1),n) = DD3(irn(1),irn(1),irn(2),n)
      DD3(irn(1),irn(3),irn(1),n) = DD3(irn(1),irn(1),irn(3),n)
      DD3(irn(3),irn(1),irn(1),n) = DD3(irn(1),irn(1),irn(3),n)
      DD3(irn(2),irn(1),irn(2),n) = DD3(irn(1),irn(2),irn(2),n)
      DD3(irn(2),irn(2),irn(1),n) = DD3(irn(1),irn(2),irn(2),n)
      DD3(irn(1),irn(3),irn(2),n) = DD3(irn(1),irn(2),irn(3),n)
      DD3(irn(2),irn(1),irn(3),n) = DD3(irn(1),irn(2),irn(3),n)
      DD3(irn(2),irn(3),irn(1),n) = DD3(irn(1),irn(2),irn(3),n)
      DD3(irn(3),irn(1),irn(2),n) = DD3(irn(1),irn(2),irn(3),n)
      DD3(irn(3),irn(2),irn(1),n) = DD3(irn(1),irn(2),irn(3),n)
      DD3(irn(3),irn(1),irn(3),n) = DD3(irn(1),irn(3),irn(3),n)
      DD3(irn(3),irn(3),irn(1),n) = DD3(irn(1),irn(3),irn(3),n)
      DD3(irn(2),irn(3),irn(2),n) = DD3(irn(2),irn(2),irn(3),n)
      DD3(irn(3),irn(2),irn(2),n) = DD3(irn(2),irn(2),irn(3),n)
      DD3(irn(3),irn(2),irn(3),n) = DD3(irn(2),irn(3),irn(3),n)
      DD3(irn(3),irn(3),irn(2),n) = DD3(irn(2),irn(3),irn(3),n)

      DD400(irn(3),irn(3),n) = dij(1,9)
      DD400(irn(2),irn(3),n) = dij(1,12)-dij(1,9)
      DD400(irn(1),irn(3),n) = dij(1,11)-dij(1,12)
      DD400(irn(2),irn(2),n) = dij(1,8)+dij(1,9)-2*dij(1,12)
      DD400(irn(1),irn(2),n) = dij(1,10)+dij(1,12)-dij(1,11)-dij(1,8)
      DD400(irn(1),irn(1),n) = dij(1,7)+dij(1,8)-2*dij(1,10)

      DD400(irn(3),irn(2),n) = DD400(irn(2),irn(3),n)
      DD400(irn(3),irn(1),n) = DD400(irn(1),irn(3),n)
      DD400(irn(2),irn(1),n) = DD400(irn(1),irn(2),n)

      DD40000(n) = dij(1,13)

      DD2(irn(1),irn(1),n) = dij(2,1)+dij(2,2)-2*dij(2,4)
      DD2(irn(1),irn(2),n) = dij(2,4)+dij(2,6)-dij(2,2)-dij(2,5)
      DD2(irn(1),irn(3),n) = dij(2,5)-dij(2,6)
      DD2(irn(2),irn(2),n) = dij(2,2)+dij(2,3)-2*dij(2,6)
      DD2(irn(2),irn(3),n) = dij(2,6)-dij(2,3)
      DD2(irn(3),irn(3),n) = dij(2,3)

      DD2(irn(2),irn(1),n) = DD2(irn(1),irn(2),n)
      DD2(irn(3),irn(1),n) = DD2(irn(1),irn(3),n)
      DD2(irn(3),irn(2),n) = DD2(irn(2),irn(3),n)

      DD300(irn(3),n) = dij(3,13)
      DD300(irn(2),n) = dij(3,12)-dij(3,13)
      DD300(irn(1),n) = dij(3,11)-dij(3,12)

      DD1(irn(3),n) = dij(1,3)
      DD1(irn(2),n) = dij(1,2)-dij(1,3)
      DD1(irn(1),n) = dij(1,1)-dij(1,2)

      if (n.eq.0) then
         DD1(1,0) = -d0 - dij(1,1)
         DD300(1,0) = - dij(3,11) - dij(2,7)
         DD40000(0) = dij(1,13)
         do i = 2,4
            DD2(1,i,0) = - DD2(2,i,0) - DD2(3,i,0) - DD2(4,i,0) 
     &                   - DD1(i,0) 
            DD2(i,1,0) = DD2(1,i,0)
            DD400(1,i,0) = - DD400(2,i,0) - DD400(3,i,0)
     &                     - DD400(4,i,0) - DD300(i,0)
            DD400(i,1,0) = DD400(1,i,0)
            do j = i,4
               DD3(1,i,j,0) = - DD3(2,i,j,0) - DD3(3,i,j,0)
     &                        - DD3(4,i,j,0) - DD2(i,j,0)
               DD3(1,j,i,0) = DD3(1,i,j,0)
               DD3(i,1,j,0) = DD3(1,i,j,0)
               DD3(j,1,i,0) = DD3(1,i,j,0)
               DD3(i,j,1,0) = DD3(1,i,j,0)
               DD3(j,i,1,0) = DD3(1,i,j,0)
            enddo
         enddo
         DD2(1,1,0) = - DD2(2,1,0) - DD2(3,1,0) - DD2(4,1,0) 
     &                - DD1(1,0) 
         DD400(1,1,0) = - DD400(2,1,0) - DD400(3,1,0)
     &                  - DD400(4,1,0) - DD300(1,0)         

         DD3(1,1,4,0) = - DD3(2,1,4,0) - DD3(3,1,4,0)
     &                  - DD3(4,1,4,0) - DD2(1,4,0)
         DD3(1,4,1,0) = DD3(1,1,4,0)
         DD3(4,1,1,0) = DD3(1,1,4,0)

         DD3(1,1,3,0) = - DD3(2,1,3,0) - DD3(3,1,3,0)
     &                  - DD3(4,1,3,0) - DD2(1,3,0)
         DD3(1,3,1,0) = DD3(1,1,3,0)
         DD3(3,1,1,0) = DD3(1,1,3,0)

         DD3(1,1,2,0) = - DD3(2,1,2,0) - DD3(3,1,2,0)
     &                  - DD3(4,1,2,0) - DD2(1,2,0)
         DD3(1,2,1,0) = DD3(1,1,2,0)
         DD3(2,1,1,0) = DD3(1,1,2,0)

         DD3(1,1,1,0) = - DD3(2,1,1,0) - DD3(3,1,1,0)
     &                  - DD3(4,1,1,0) - DD2(1,1,0)
      endif    

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                tens_red5 = 5-point tensors
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine tens_red5_old(psq, qsq, lsq, tsq,
     #                     pq, pl, pt, ql, qt, lt, 
     #                     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #                     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #                     E0, 
     #                     Eij)
      implicit none
      real * 8 psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt
      double complex D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
      double complex Dij_2345(3,13),Dij_1345(3,13),Dij_1245(3,13),
     #     Dij_1235(3,13),Dij_1234(3,13)
      double complex E0, Eij(4,46), Eijp(4,46), Eijpp(4,46), Eijo(4,46)
c
c  determine the Passarino-Veltman tensor decomposition for the four-point
c  tensor integrals
c
c                                                             d^4k
c   E0; E_mu; E_mu,nu; E_mu,nu,rho; E_mu,nu,rho_sig  =  Int -------
c                                                           (2pi)^4
c
c   1; k_mu; k_mu k_nu; k_mu k_nu k_rho; k_mu k_nu k_rho k_sig
c --------------------------------------------------------------------------
c   k^2  (k+p)^2  (k+p+q)^2  (k+p+q+l)^2  (k+p+q+l+t)^2
c
c  with
c
c   E_mu = p_mu E11  +  q_mu E12  +  l_mu E13  +  t_mu E14
c
c   E_munu = p_mu p_nu E21 + q_mu q_nu E22 + ...
c
c
C INPUT:  psq, pq,...                     kinematics invariants
c         D0_1345 = D00(p+q,l,t)          scalar 4-point functions
c         D0_1245 = D00(p,q+l,t)          in the PV notation
c         D0_1234 = D00(p,q,l)
c         D0_1235 = D00(p,q,l+t)      
c         D0_2345 = D00(q,l,t)

C         Dij_1235(n,m) = D_nm(1,2,3,5) ....    higher D_nm form factors
C                                               as in tens_red3 and tens_red4
c         E0 = E0(p,q,l,t)                      scalar five point function
c
c OUTPUT: Eij(n,m) = E_nm                    form factors in the tensor 
c                                            integrals a la PV
c         nm = 11, 12, 13, 14                ff's for E_mu
c         nm = 21, 22, 23, ..., 211    ff's for E_munu
c         nm = 31, 32, 33, ..., 323, 324     ff's for E_mu,nu,rho
c         nm = 41, 42, 43, ..., 445, 446
c
      double precision f1, f2, f3, f4, Xm1(4,4), X(4,4), deter
      double complex R(1:4),res(1:4)

      double precision s02,s03,s12,s13,s14,s23,s24,s34,  s01,s04,
     1       detx4,detx01t,detx02t,detx03t,detx04t,detx00t
      double precision x4(0:4,0:4), x4t(0:4,0:4), x4tt(0:4,0:4,0:4,0:4)
      double complex dd1(4,0:4), dd1n(4,0:4), ed2(4,4), dd2(4,4,0:4), 
     1           ed3(4,4,4), xed3(4,4,4), dd300(4,0:4), ed300(4),
     1           dd3(4,4,4,0:4), dd400(4,4,0:4), dd40000(0:4),
     1           xed4(4,4,4,4), ed4(4,4,4,4), ed400(4,4), ed40000
      double complex zz(4,3)
      integer i,j,k
      logical debug, lwr
      parameter (debug=.false.,lwr=.false.)
      character*20 text

      f1 = -psq
      f2 = -qsq-2*pq
      f3 = -lsq-2*ql-2*pl
      f4 = -tsq-2*lt-2*qt-2*pt

      X(1,1) = psq
      X(1,2) = pq
      X(1,3) = pl
      X(1,4) = pt
      X(2,1) = pq
      X(2,2) = qsq
      X(2,3) = ql
      X(2,4) = qt
      X(3,1) = pl
      X(3,2) = ql
      X(3,3) = lsq
      X(3,4) = lt
      X(4,1) = pt
      X(4,2) = qt
      X(4,3) = lt
      X(4,4) = tsq
     

      deter = psq*qsq*lsq*tsq-psq*qsq*lt**2-psq*ql**2*tsq+2*psq*ql*qt*lt
     #-psq*qt**2*lsq-pq**2*lsq*tsq+pq**2*lt**2+2*pq*ql*pl*tsq-2*pq*ql*pt
     #*lt-2*pq*qt*pl*lt+2*pq*qt*pt*lsq-qsq*pl**2*tsq+2*pl*qsq*pt*lt+pl**
     #2*qt**2-2*pl*qt*pt*ql-qsq*pt**2*lsq+pt**2*ql**2

      Xm1(1,1) = (qsq*lsq*tsq-qsq*lt**2-ql**2*tsq+2*ql*qt*lt-qt**2*lsq)/
     #deter
      Xm1(1,2) = -(pq*lsq*tsq-pq*lt**2-ql*pl*tsq+ql*pt*lt+qt*pl*lt-qt*pt
     #*lsq)/deter
      Xm1(1,3) = (pq*ql*tsq-pq*qt*lt-qsq*pl*tsq+qsq*pt*lt+pl*qt**2-qt*pt
     #*ql)/deter
      Xm1(1,4) = (-pq*ql*lt+pq*qt*lsq+qsq*pl*lt-qsq*pt*lsq-ql*pl*qt+pt*q
     #l**2)/deter
      Xm1(2,1) = Xm1(1,2)
      Xm1(2,2) = (psq*lsq*tsq-psq*lt**2-pl**2*tsq+2*pl*pt*lt-pt**2*lsq)/
     #deter
      Xm1(2,3) = (-psq*ql*tsq+psq*qt*lt+pl*pq*tsq-pl*pt*qt-pt*pq*lt+pt**
     #2*ql)/deter
      Xm1(2,4) = (psq*ql*lt-psq*qt*lsq-pl*pq*lt+pl**2*qt+pt*pq*lsq-pt*pl
     #*ql)/deter
      Xm1(3,1) = Xm1(1,3)
      Xm1(3,2) = Xm1(2,3)
      Xm1(3,3) = -(-psq*qsq*tsq+psq*qt**2+pq**2*tsq-2*pq*pt*qt+pt**2*qsq
     #)/deter
      Xm1(3,4) = (-psq*qsq*lt+psq*qt*ql+pq**2*lt-pq*pl*qt-pt*pq*ql+pt*pl
     #*qsq)/deter
      Xm1(4,1) = Xm1(1,4)
      Xm1(4,2) = Xm1(2,4)
      Xm1(4,3) = Xm1(3,4)
      Xm1(4,4) = -(-psq*qsq*lsq+psq*ql**2+pq**2*lsq-2*pq*pl*ql+pl**2*qsq
     #)/deter
      
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c p1 = p          with p.p = 0
c p2 = p+q
c p3 = p+q+l
c p4 = p+q+l+t    with p4.p4=0
      if(psq.ne.0d0) then
         print*,' psq must be zero. Here psq = ',psq
      endif
      s04 = psq+qsq+lsq+tsq+2*(pq+pl+pt+ql+qt+lt)
      if (lwr) print*,' p4.p4 = s04 =',s04
c s02:=p2.p2;    
      s02=2*pq+psq+qsq
c s03:=p3.p3;
      s03=lsq+2*pl+2*pq+psq+2*ql+qsq
c s12:=(p2-p1).(p2-p1);
      s12=qsq
c s13:=(p3-p1).(p3-p1);
      s13=lsq+2*ql+qsq
c s14:=(p4-p1).(p4-p1);
      s14=lsq+2*lt+2*ql+qsq+2*qt+tsq
c s23:=(p2-p3).(p2-p3);
      s23=lsq
c s24:=(p2-p4).(p2-p4);
      s24=lsq+2*lt+tsq
c s34:=(p3-p4).(p3-p4);
      s34=tsq
c detx4:=det(x5);
      detx4=2*s14*(s02**2*s13*s34-s02*s03*s12*s34-s02*s03*s13*s24+
     . s02*s03*s14*s23+s03**2*s12*s24)
c detx01t:= -det(xt1);
      detx01t=s02**2*s13*s34+s02**2*s14*s34-s02**2*s34**2-s02*s03*s12
     . *s34-s02*s03*s13*s24+2*s02*s03*s14*s23-s02*s03*s14*s24-s02*
     . s03*s14*s34+2*s02*s03*s24*s34+s02*s12*s34**2-s02*s13*s24*s34
     . -s02*s14*s23*s34+s03**2*s12*s24+s03**2*s14*s24-s03**2*s24**2-
     . s03*s12*s24*s34+s03*s13*s24**2-s03*s14*s23*s24
c detx02t:= det(xt2);
      detx02t=s14*(-s02*s03*s13+s02*s03*s14-s02*s03*s34+2*s02*s13*
     . s34+s03**2*s12-s03**2*s14+s03**2*s24-s03*s12*s34-s03*s13*s24+
     . s03*s14*s23)
c detx03t:= -det(xt3);
      detx03t=s14*(s02**2*s13-s02**2*s14+s02**2*s34-s02*s03*s12+s02*
     . s03*s14-s02*s03*s24-s02*s12*s34-s02*s13*s24+s02*s14*s23+2*
     . s03*s12*s24)
c detx04t:= det(xt4);
      detx04t=-s02**2*s13**2+s02**2*s13*s14+s02**2*s13*s34+2*s02*
     . s03*s12*s13-s02*s03*s12*s14-s02*s03*s12*s34-s02*s03*s13*s14-
     . s02*s03*s13*s24+2*s02*s03*s14*s23-s02*s12*s13*s34+s02*s13**2
     . *s24-s02*s13*s14*s23-s03**2*s12**2+s03**2*s12*s14+s03**2*s12*
     . s24+s03*s12**2*s34-s03*s12*s13*s24-s03*s12*s14*s23

      detx00t=s02**2*s13**2-2*s02**2*s13*s14-2*s02**2*s13*s34+s02
     . **2*s14**2-2*s02**2*s14*s34+s02**2*s34**2-2*s02*s03*s12*
     . s13+2*s02*s03*s12*s14+2*s02*s03*s12*s34+2*s02*s03*s13*
     . s14+2*s02*s03*s13*s24-2*s02*s03*s14**2-4*s02*s03*s14*s23
     . +2*s02*s03*s14*s24+2*s02*s03*s14*s34-2*s02*s03*s24*s34+
     . 2*s02*s12*s13*s34+2*s02*s12*s14*s34-2*s02*s12*s34**2-2
     . *s02*s13**2*s24+2*s02*s13*s14*s23+2*s02*s13*s14*s24-4*
     . s02*s13*s14*s34+2*s02*s13*s24*s34-2*s02*s14**2*s23+2*s02
     . *s14*s23*s34+s03**2*s12**2-2*s03**2*s12*s14-2*s03**2*s12*
     . s24+s03**2*s14**2-2*s03**2*s14*s24+s03**2*s24**2-2*s03*s12
     . **2*s34+2*s03*s12*s13*s24+2*s03*s12*s14*s23-4*s03*s12*
     . s14*s24+2*s03*s12*s14*s34+2*s03*s12*s24*s34+2*s03*s13*
     . s14*s24-2*s03*s13*s24**2-2*s03*s14**2*s23+2*s03*s14*s23*
     . s24+s12**2*s34**2-2*s12*s13*s24*s34-2*s12*s14*s23*s34+s13
     . **2*s24**2-2*s13*s14*s23*s24+s14**2*s23**2
c
      x4t(0,0)=detx00t
      x4t(0,1)=detx01t
      x4t(0,2)=detx02t
      x4t(0,3)=detx03t
      x4t(0,4)=detx04t
      x4t(1,1)=s02**2*s34**2-2*s02*s03*s24*s34+s03**2*s24**2
      x4t(1,2)=s03*s14*(s02*s34-s03*s24)
      x4t(1,3)=s02*s14*(-s02*s34+s03*s24)
      x4t(1,4)=-s02**2*s13*s34+s02*s03*s12*s34+s02*s03*s13*s24-
     &       2*s02*s03*s14*s23-s03**2*s12*s24
      x4t(2,2)=s03**2*s14**2
      x4t(2,3)=-s02*s03*s14**2
      x4t(2,4)=s03*s14*(s02*s13-s03*s12)
      x4t(3,3)=s02**2*s14**2
      x4t(3,4)=s02*s14*(-s02*s13+s03*s12)
      x4t(4,4)=s02**2*s13**2-2*s02*s03*s12*s13+s03**2*s12**2

      do i = 0,3
         do j = i+1,4
            x4t(j,i) = x4t(i,j)
         enddo
      enddo
      do i = 1,4
         do j = 1,4
            x4tt(0,i,0,j) = -(x4t(0,0)*x4t(i,j)-x4t(i,0)*x4t(0,j))/detx4
            do k = 1,4
               x4tt(0,i,j,k) = (x4t(0,k)*x4t(i,j)-x4t(i,k)*x4t(0,j))
     &                         /detx4
               x4tt(j,k,0,i) = x4tt(0,i,j,k)
            enddo
         enddo
      enddo

      do i = 1,4
         r(i) = (x4t(i,1)*(d0_1345-d0_2345) +
     &           x4t(i,2)*(d0_1245-d0_2345) +
     &           x4t(i,3)*(d0_1235-d0_2345) +
     &           x4t(i,4)*(d0_1234-d0_2345) - 
     &           x4t(i,0)*d0_2345)/detx4
      enddo
      eij(1,4) = r(4)
      do i = 3,1,-1
         eij(1,i) = eij(1,i+1)+r(i)
      enddo

      Eij(2,11) = (detx01t*(dij_1345(2,7)-dij_2345(2,7)) +
     &              detx02t*(dij_1245(2,7)-dij_2345(2,7)) +
     &              detx03t*(dij_1235(2,7)-dij_2345(2,7)) +
     &              detx04t*(dij_1234(2,7)-dij_2345(2,7)) )/detx4

      call d2pvtodd(4,d0_1234,dij_1234,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(3,d0_1235,dij_1235,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(2,d0_1245,dij_1245,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(1,d0_1345,dij_1345,dd1,dd2,dd3,dd300,dd400,dd40000)
      call d2pvtodd(0,d0_2345,dij_2345,dd1,dd2,dd3,dd300,dd400,dd40000)

c      DD1N(4,4) = 0
c      DD1N(3,4) = dij_1234(1,3)
c      DD1N(2,4) = dij_1234(1,2)-dij_1234(1,3)
c      DD1N(1,4) = dij_1234(1,1)-dij_1234(1,2)

c      DD1N(3,3) = 0
c      DD1N(4,3) = dij_1235(1,3)
c      DD1N(2,3) = dij_1235(1,2)-dij_1235(1,3)
c      DD1N(1,3) = dij_1235(1,1)-dij_1235(1,2)

c      DD1N(2,2) = 0
c      DD1N(4,2) = dij_1245(1,3)
c      DD1N(3,2) = dij_1245(1,2)-dij_1245(1,3)
c      DD1N(1,2) = dij_1245(1,1)-dij_1245(1,2)

c      DD1N(1,1) = 0
c      DD1N(4,1) = dij_1345(1,3)
c      DD1N(3,1) = dij_1345(1,2)-dij_1345(1,3)
c      DD1N(2,1) = dij_1345(1,1)-dij_1345(1,2)

c      DD1N(4,0) = dij_2345(1,3)
c      DD1N(3,0) = dij_2345(1,2)-dij_2345(1,3)
c      DD1N(2,0) = dij_2345(1,1)-dij_2345(1,2)
c      DD1N(1,0) = -d0_2345 - DD1N(2,0) - DD1N(3,0) - DD1N(4,0) 

      do i = 1,4
         do j = 1,4
            ed2(i,j) = x4t(i,1)*(dd1(j,1)-dd1(j,0)) +
     &                 x4t(i,2)*(dd1(j,2)-dd1(j,0)) +
     &                 x4t(i,3)*(dd1(j,3)-dd1(j,0)) +
     &                 x4t(i,4)*(dd1(j,4)-dd1(j,0)) -
     &                 x4t(i,0)*dd1(j,0) 
            ed2(i,j) = ed2(i,j) - 2*(
     &        x4tt(i,1,0,j)*(dij_1345(2,7)-dij_2345(2,7)) +
     &        x4tt(i,2,0,j)*(dij_1245(2,7)-dij_2345(2,7)) +
     &        x4tt(i,3,0,j)*(dij_1235(2,7)-dij_2345(2,7)) +
     &        x4tt(i,4,0,j)*(dij_1234(2,7)-dij_2345(2,7)) )
         enddo
      enddo
      do i = 1,4
         do j = i,4
            ed2(i,j) = (ed2(i,j)+ed2(j,i))/(2*detx4)
         enddo
      enddo

      eij(2,4) = ed2(4,4)
      eij(2,10) = ed2(4,4) + ed2(3,4)
      eij(2,9) = eij(2,10)+ ed2(2,4)
      eij(2,7) = eij(2,9) + ed2(1,4)
      eij(2,3) = ed2(4,4) + 2*ed2(3,4) + ed2(3,3)
      eij(2,8) = eij(2,3) + ed2(2,4) + ed2(2,3)
      eij(2,6) = eij(2,8) + ed2(1,4) + ed2(1,3)
      eij(2,2) = eij(2,8) + ed2(2,4) + ed2(2,3) + ed2(2,2)
      eij(2,5) = eij(2,2) + ed2(1,4) + ed2(1,3) + ed2(1,2)
      eij(2,1) = eij(2,5) + ed2(1,4) + ed2(1,3) + ed2(1,2) + ed2(1,1)

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      R(1) = 0.5d0*(Dij_1345(1,1)+D0_2345+f1*Eij(1,1)) - Eij(2,11)
      R(2) = 0.5d0*(Dij_1245(1,1)-Dij_1345(1,1)+f2*Eij(1,1))
      R(3) = 0.5d0*(Dij_1235(1,1)-Dij_1245(1,1)+f3*Eij(1,1))
      R(4) = 0.5d0*(Dij_1234(1,1)-Dij_1235(1,1)+f4*Eij(1,1))
      call prod_mat_col4(Xm1,R(1),res)
      Eijo(2,1) = res(1)
      Eijo(2,5) = res(2)
      Eijo(2,6) = res(3)
      Eijo(2,7) = res(4)


      R(1) = 0.5d0*(Dij_1345(1,1)-Dij_2345(1,1)+f1*Eij(1,2))
      R(2) = 0.5d0*(Dij_1245(1,2)-Dij_1345(1,1)+f2*Eij(1,2))-Eij(2,11)
      R(3) = 0.5d0*(Dij_1235(1,2)-Dij_1245(1,2)+f3*Eij(1,2))
      R(4) = 0.5d0*(Dij_1234(1,2)-Dij_1235(1,2)+f4*Eij(1,2))
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(2,5) = res(1)
      Eijo(2,2) = res(2)
      Eijo(2,8) = res(3)
      Eijo(2,9) = res(4)


      R(1) = 0.5d0*(Dij_1345(1,2)-Dij_2345(1,2)+f1*Eij(1,3))
      R(2) = 0.5d0*(Dij_1245(1,2)-Dij_1345(1,2)+f2*Eij(1,3))
      R(3) = 0.5d0*(Dij_1235(1,3)-Dij_1245(1,2)+f3*Eij(1,3))-Eij(2,11)
      R(4) = 0.5d0*(Dij_1234(1,3)-Dij_1235(1,3)+f4*Eij(1,3))
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(2,6) = res(1)
      Eijp(2,8) = res(2)
      Eijo(2,3) = res(3)
      Eijo(2,10) = res(4)

      R(1) = 0.5d0*(Dij_1345(1,3)-Dij_2345(1,3)+f1*Eij(1,4))
      R(2) = 0.5d0*(Dij_1245(1,3)-Dij_1345(1,3)+f2*Eij(1,4))
      R(3) = 0.5d0*(Dij_1235(1,3)-Dij_1245(1,3)+f3*Eij(1,4))
      R(4) = 0.5d0*(-Dij_1235(1,3)+f4*Eij(1,4))-Eij(2,11)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(2,7) = res(1)
      Eijp(2,9) = res(2)
      Eijp(2,10) = res(3)
      Eijo(2,4) = res(4)


      Eij(3,21) = -D0_2345/2-Eij(2,1)*f1/2-f4*Eij(2,7)/2
     #     -f2*Eij(2,5)/2-f3*Eij(2,6)/2
      

      Eij(3,22) = Dij_2345(1,1)/2-f3*Eij(2,8)/2-f4*Eij(2,9)/2
     #     -f1*Eij(2,5)/2-f2*Eij(2,2)/2

      Eij(3,23) = Dij_2345(1,2)/2-f2*Eij(2,8)/2-f4*Eij(2,10)/2
     #     -f3*Eij(2,3)/2-f1*Eij(2,6)/2

      Eij(3,24) = Dij_2345(1,3)/2-f3*Eij(2,10)/2-f1*Eij(2,7)/2
     #     -f2*Eij(2,9)/2-f4*Eij(2,4)/2

c      Eij(3,21) = 0d0
c      Eij(3,22) = 0d0
c      Eij(3,23) = 0d0
c      Eij(3,24) = 0d0


      R(1) = -0.5d0*D0_2345+0.5d0*Eij(2,1)*f1+0.5d0*Dij_1345(2,1) 
     #     - 2*Eij(3,21)
      R(2) = 0.5d0*f2*Eij(2,1)+0.5d0*Dij_1245(2,1)-0.5d0*Dij_1345(2,1)
      R(3) = -0.5d0*Dij_1245(2,1)+0.5d0*f3*Eij(2,1)+0.5d0*Dij_1235(2,1)
      R(4) = 0.5d0*f4*Eij(2,1)-0.5d0*Dij_1235(2,1)+0.5d0*Dij_1234(2,1)
      call prod_mat_col4(Xm1,R(1),res)
      eij(3,1) = res(1)
      eij(3,5) = res(2)
      eij(3,6) = res(3)
      eij(3,7) = res(4)


      R(1) =  -0.5d0*Dij_2345(2,3)+0.5d0*Eij(2,4)*f1+0.5d0*Dij_1345(2,3)
      R(2) = 0.5d0*f2*Eij(2,4)+0.5d0*Dij_1245(2,3)-0.5d0*Dij_1345(2,3)
      R(3) = 0.5d0*f3*Eij(2,4)-0.5d0*Dij_1245(2,3)+0.5d0*Dij_1235(2,3)
      R(4) = 0.5d0*f4*Eij(2,4)-0.5d0*Dij_1235(2,3)-2*Eij(3,24)
      call prod_mat_col4(Xm1,R(1),res)
      eij(3,14) = res(1)
      eij(3,15) = res(2)
      eij(3,16) = res(3)
      eij(3,4) = res(4)


      R(1) = -0.5d0*Dij_2345(2,2)+0.5d0*Eij(2,3)*f1+0.5d0*Dij_1345(2,2)
      R(2) = 0.5d0*f2*Eij(2,3)+0.5d0*Dij_1245(2,2)-0.5d0*Dij_1345(2,2)
      R(3) = 0.5d0*f3*Eij(2,3)-0.5d0*Dij_1245(2,2)+0.5d0*Dij_1235(2,3)
     #     -2*Eij(3,23)
      R(4) = 0.5d0*f4*Eij(2,3)-0.5d0*Dij_1235(2,3)+0.5d0*Dij_1234(2,3)
      call prod_mat_col4(Xm1,R(1),res)
      eij(3,11) = res(1)
      eij(3,12) = res(2)
      eij(3,3) = res(3)
      eij(3,13) = res(4)


      R(1) = -0.5d0*Dij_2345(2,1)+0.5d0*Eij(2,2)*f1+0.5d0*Dij_1345(2,1)
      R(2) = 0.5d0*Dij_1245(2,2)+0.5d0*f2*Eij(2,2)-0.5d0*Dij_1345(2,1)
     #     -2*Eij(3,22)
      R(3) = 0.5d0*f3*Eij(2,2)-0.5d0*Dij_1245(2,2)+0.5d0*Dij_1235(2,2)
      R(4) = 0.5d0*f4*Eij(2,2)-0.5d0*Dij_1235(2,2)+0.5d0*Dij_1234(2,2)
      call prod_mat_col4(Xm1,R(1),res)
      eij(3,8) = res(1)
      eij(3,2) = res(2)
      eij(3,9) = res(3)
      eij(3,10) = res(4)


      R(1) = -0.5d0*Dij_2345(2,6)+0.5d0*Dij_1345(2,6)+0.5d0*f1*Eij(2,10)
      R(2) = 0.5d0*f2*Eij(2,10)-0.5d0*Dij_1345(2,6)+0.5d0*Dij_1245(2,6)
      R(3) = -0.5d0*Dij_1245(2,6)+0.5d0*f3*Eij(2,10)+0.5d0*Dij_1235(2,3)
     #     -Eij(3,24)
      R(4) = 0.5d0*f4*Eij(2,10)-0.5d0*Dij_1235(2,3)-Eij(3,23)
      call prod_mat_col4(Xm1,R(1),res)
      eij(3,19) = res(1)
      eij(3,20) = res(2)
      Eijp(3,13) = res(3)
      Eijp(3,16) = res(4)


      R(1) = 0.5d0*Dij_2345(1,1)+0.5d0*f1*Eij(2,5)+0.5d0*Dij_1345(2,1)
     #     -Eij(3,22)
      R(2) = 0.5d0*f2*Eij(2,5)+0.5d0*Dij_1245(2,4)-0.5d0*Dij_1345(2,1)
     #     -Eij(3,21)
      R(3) = -0.5d0*Dij_1245(2,4)+0.5d0*f3*Eij(2,5)+0.5d0*Dij_1235(2,4)
      R(4) = 0.5d0*Dij_1234(2,4)-0.5d0*Dij_1235(2,4)+0.5d0*f4*Eij(2,5)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(3,5) = res(1)
      Eijp(3,8) = res(2)
      eij(3,17) = res(3)
      eij(3,18) = res(4)

      if (debug) then
        R(1) = 0.5d0*f1*Eij(2,8)+0.5d0*Dij_1345(2,4)-0.5d0*Dij_2345(2,4)
        R(2) = 0.5d0*Dij_1245(2,2)+0.5d0*f2*Eij(2,8)-0.5d0*Dij_1345(2,4)
     #        -Eij(3,23)
        R(3) = 0.5d0*f3*Eij(2,8)-0.5d0*Dij_1245(2,2)+0.5d0*Dij_1235(2,6)
     #        -Eij(3,22)
        R(4) = 0.5d0*Dij_1234(2,6)-0.5d0*Dij_1235(2,6)+0.5d0*f4*Eij(2,8)
        call prod_mat_col4(Xm1,R(1),res)
        Eijp(3,17) = res(1)
        Eijp(3,9) = res(2)
        Eijp(3,12) = res(3)
        Eijp(3,20) = res(4)
      endif

      Eij(4,36) = -0.5d0*(-D0_2345+f3*Eij(3,6)+f4*Eij(3,7)
     #     +f2*Eij(3,5)+Eij(3,1)*f1)/2

      Eij(4,37) = -0.5d0*(-Dij_2345(2,1)+f4*Eij(3,10)+f3*Eij(3,9)+
     #     f1*Eij(3,8)+f2*Eij(3,2))/2

      Eij(4,38) = 0.5d0*(Dij_2345(2,2)-f1*Eij(3,11)-f2*Eij(3,12)
     #     -f4*Eij(3,13)-f3*Eij(3,3))/2
               
      Eij(4,39) = -0.5d0*(-Dij_2345(2,3)+f2*Eij(3,15)+f1*Eij(3,14)+
     #     f3*Eij(3,16)+f4*Eij(3,4))/2
      Eij(4,40) = -0.5d0*(Dij_2345(1,1)+f4*Eij(3,18)+f3*Eij(3,17)
     #     +f2*Eij(3,8)+f1*Eij(3,5))/2

      Eij(4,41) =  -0.5d0*(Dij_2345(1,2)+f1*Eij(3,6)+f4*Eij(3,19)+
     #     f3*Eij(3,11)+f2*Eij(3,17))/2
      
      Eij(4,42) = -0.5d0*(Dij_2345(1,3)+f2*Eij(3,18)+f3*Eij(3,19)+
     #     f1*Eij(3,7)+f4*Eij(3,14))/2

      Eij(4,43) = -0.5d0*(-Dij_2345(2,4)+f4*Eij(3,20)+f3*Eij(3,12)+
     #     f1*Eij(3,17)+f2*Eij(3,9))/2
      Eij(4,44) = -0.5d0*(-Dij_2345(2,5)+f3*Eij(3,20)+f1*Eij(3,18)
     #     +f2*Eij(3,10)+f4*Eij(3,15))/2
      
      Eij(4,45) = -0.5d0*(-Dij_2345(2,6)+f2*Eij(3,20)+f1*Eij(3,19)+
     #     f4*Eij(3,16)+f3*Eij(3,13))/2
      
      Eij(4,46) = f3*lsq*Eij(3,3)/24+f4*tsq*Eij(3,4)/24
     #     +(f2*pt+f4*pq+f1*qt)*Eij(3,18)/12
     #     +(f3*pt+f1*lt+f4*pl)*Eij(3,19)/12
     #     +(f2*qt+f4*qsq/2)*Eij(3,10)/12+(f4*qt+f2*tsq/2)*Eij(3,15)/12
     #     +f1*psq*Eij(3,1)/24+f2*qsq*Eij(3,2)/24
     #     +(f1*pq+f2*psq/2)*Eij(3,5)/12
     #     +Dij_2345(2,7)/6+(f2*pl+f3*pq+f1*ql)*Eij(3,17)/12
     #     +(f4*psq/2+f1*pt)*Eij(3,7)/12+(f4*pt+f1*tsq/2)*Eij(3,14)/12
     #     +(f3*qsq/2+f2*ql)*Eij(3,9)/12+(f3*pl+f1*lsq/2)*Eij(3,11)/12
     #     +(f3*ql+f2*lsq/2)*Eij(3,12)/12+(f1*qsq/2+pq*f2)*Eij(3,8)/12
     #     -Dij_2345(2,5)*qt/12-Dij_2345(2,6)*lt/12
     #     +(f4*lt+f3*tsq/2)*Eij(3,16)/12+(f4*lsq/2+f3*lt)*Eij(3,13)/12
     #     +(f3*qt+f4*ql+f2*lt)*Eij(3,20)/12+Dij_2345(1,1)*pq/12
     #     +Dij_2345(1,2)*pl/12+Dij_2345(1,3)*pt/12
     #     -Dij_2345(2,4)*ql/12+(f1*pl+f3*psq/2)*Eij(3,6)/12
     #     -Dij_2345(2,2)*lsq/24-Dij_2345(2,1)*qsq/24
     #     -Dij_2345(2,3)*tsq/24-psq*D0_2345/24


      R(1) = 0.5d0*D0_2345+0.5d0*Eij(3,1)*f1+0.5d0*Dij_1345(3,1)
     #     -3*Eij(4,36)
      R(2) = 0.5d0*Dij_1245(3,1)+0.5d0*f2*Eij(3,1)-0.5d0*Dij_1345(3,1)
      R(3) = 0.5d0*Dij_1235(3,1)-0.5d0*Dij_1245(3,1)+0.5d0*f3*Eij(3,1)
      R(4) = -0.5d0*Dij_1235(3,1)+0.5d0*Dij_1234(3,1)+0.5d0*f4*Eij(3,1)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,1) = res(1)
      Eij(4,5) = res(2)
      Eij(4,6) = res(3)
      Eij(4,7) = res(4)

      R(1) = 0.5d0*Eij(3,2)*f1-0.5d0*Dij_2345(3,1)+0.5d0*Dij_1345(3,1)
      R(2) = 0.5d0*Dij_1245(3,2)+0.5d0*f2*Eij(3,2)-0.5d0*Dij_1345(3,1)
     #     -3*Eij(4,37)
      R(3) = -0.5d0*Dij_1245(3,2)+0.5d0*f3*Eij(3,2)+0.5d0*Dij_1235(3,2)
      R(4) = 0.5d0*Dij_1234(3,2)+0.5d0*f4*Eij(3,2)-0.5d0*Dij_1235(3,2)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,8) = res(1)
      Eij(4,2) = res(2)
      Eij(4,9) = res(3)
      Eij(4,10) = res(4)


      R(1) = 0.5d0*Eij(3,3)*f1-0.5d0*Dij_2345(3,2)+0.5d0*Dij_1345(3,2)
      R(2) = 0.5d0*Dij_1245(3,2)+0.5d0*f2*Eij(3,3)-0.5d0*Dij_1345(3,2)
      R(3) = -0.5d0*Dij_1245(3,2)+0.5d0*f3*Eij(3,3)+0.5d0*Dij_1235(3,3)
     #     -3*Eij(4,38)
      R(4) = 0.5d0*Dij_1234(3,3)+0.5d0*f4*Eij(3,3)-0.5d0*Dij_1235(3,3)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,11) = res(1)
      Eij(4,12) = res(2)
      Eij(4,3) = res(3)
      Eij(4,13) = res(4)


      R(1) = 0.5d0*Eij(3,4)*f1-0.5d0*Dij_2345(3,3)+0.5d0*Dij_1345(3,3)
      R(2) = 0.5d0*Dij_1245(3,3)-0.5d0*Dij_1345(3,3)+0.5d0*f2*Eij(3,4)
      R(3) = -0.5d0*Dij_1245(3,3)+0.5d0*f3*Eij(3,4)+0.5d0*Dij_1235(3,3)
      R(4) = 0.5d0*f4*Eij(3,4)-0.5d0*Dij_1235(3,3)-3*Eij(4,39)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,14) = res(1)
      Eij(4,15) = res(2)
      Eij(4,16) = res(3)
      Eij(4,4) = res(4)


      R(1) = 0.5d0*Dij_1345(3,6)+0.5d0*f1*Eij(3,12)-0.5d0*Dij_2345(3,6)
      R(2) = 0.5d0*Dij_1245(3,2)+0.5d0*f2*Eij(3,12)-0.5d0*Dij_1345(3,6)
     #     -Eij(4,38)
      R(3) = -0.5d0*Dij_1245(3,2)+0.5d0*f3*Eij(3,12)+0.5d0*Dij_1235(3,9)
     #     -2*Eij(4,43)
      R(4) = -0.5d0*Dij_1235(3,9)+0.5d0*f4*Eij(3,12)+0.5d0*Dij_1234(3,9)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,29) = res(1)
      Eij(4,20) = res(2)
      Eijp(4,12) = res(3)
      Eij(4,31) = res(4)


      R(1) = 0.5d0*Dij_1345(3,9)+0.5d0*f1*Eij(3,16)-0.5d0*Dij_2345(3,9)
      R(2) = 0.5d0*Dij_1245(3,9)+0.5d0*f2*Eij(3,16)-0.5d0*Dij_1345(3,9)
      R(3) = 0.5d0*f3*Eij(3,16)-0.5d0*Dij_1245(3,9)+0.5d0*Dij_1235(3,3)
     #     -Eij(4,39)
      R(4) = 0.5d0*f4*Eij(3,16)-0.5d0*Dij_1235(3,3)-2*Eij(4,45)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,33) = res(1)
      Eij(4,34) = res(2)
      Eij(4,22) = res(3)
      Eijp(4,16) = res(4)


      R(1) = 0.5d0*Dij_1345(3,5)+0.5d0*f1*Eij(3,7)-0.5d0*Dij_2345(1,3)
     #     -2*Eij(4,42)
      R(2) = 0.5d0*Dij_1245(3,5)-0.5d0*Dij_1345(3,5)+0.5d0*f2*Eij(3,7)
      R(3) = 0.5d0*f3*Eij(3,7)-0.5d0*Dij_1245(3,5)+0.5d0*Dij_1235(3,5)
      R(4) = -0.5d0*Dij_1235(3,5)+0.5d0*f4*Eij(3,7)-Eij(4,36)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(4,7) = res(1)
      Eij(4,24) = res(2)
      Eij(4,25) = res(3)
      Eij(4,19) = res(4)


      R(1) = 0.5d0*Dij_1345(3,5)+0.5d0*f1*Eij(3,10)-0.5d0*Dij_2345(3,5)
      R(2) = 0.5d0*Dij_1245(3,8)-0.5d0*Dij_1345(3,5)+0.5d0*f2*Eij(3,10)
     #     -2*Eij(4,44)
      R(3) = 0.5d0*f3*Eij(3,10)-0.5d0*Dij_1245(3,8)+0.5d0*Dij_1235(3,8)
      R(4) = -0.5d0*Dij_1235(3,8)+0.5d0*f4*Eij(3,10)-Eij(4,37)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,27) = res(1)
      Eijp(4,10) = res(2)
      Eij(4,28) = res(3)
      Eij(4,21) = res(4)


      R(1) = 0.5d0*Dij_2345(2,1)+0.5d0*f1*Eij(3,8)+0.5d0*Dij_1345(3,1)
     #     -Eij(4,37)
      R(2) = 0.5d0*Dij_1245(3,6)+0.5d0*f2*Eij(3,8)-0.5d0*Dij_1345(3,1)
     #     -2*Eij(4,40)
      R(3) = 0.5d0*f3*Eij(3,8)+0.5d0*Dij_1235(3,6)-0.5d0*Dij_1245(3,6)
      R(4) = -0.5d0*Dij_1235(3,6)+0.5d0*f4*Eij(3,8)+0.5d0*Dij_1234(3,6)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,17) = res(1)
      Eijp(4,8) = res(2)
      Eij(4,26) = res(3)
      Eijp(4,27) = res(4)


      R(1) = 0.5d0*Dij_2345(2,2)+0.5d0*Dij_1345(3,6)+0.5d0*f1*Eij(3,11)
     #     -Eij(4,38)
      R(2) = 0.5d0*Dij_1245(3,6)+0.5d0*f2*Eij(3,11)-0.5d0*Dij_1345(3,6)
      R(3) = 0.5d0*f3*Eij(3,11)+0.5d0*Dij_1235(3,7)-0.5d0*Dij_1245(3,6)
     #     -2*Eij(4,41)
      R(4) = -0.5d0*Dij_1235(3,7)+0.5d0*Dij_1234(3,7)+0.5d0*f4*Eij(3,11)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,18) = res(1)
      Eijp(4,29) = res(2)
      Eijp(4,11) = res(3)
      Eij(4,30) = res(4)


      R(1) = 0.5d0*f1*Eij(3,5)+0.5d0*Dij_1345(3,1)-0.5d0*Dij_2345(1,1)
     #     -2*Eij(4,40)
      R(2) = 0.5d0*Dij_1245(3,4)+0.5d0*f2*Eij(3,5)-0.5d0*Dij_1345(3,1)
     #     -Eij(4,36)
      R(3) = 0.5d0*f3*Eij(3,5)-0.5d0*Dij_1245(3,4)+0.5d0*Dij_1235(3,4)
      R(4) = -0.5d0*Dij_1235(3,4)+0.5d0*Dij_1234(3,4)+0.5d0*f4*Eij(3,5)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(4,5) = res(1)
      Eijp(4,17) = res(2)
      Eij(4,23) = res(3)
      Eijp(4,24) = res(4)


      R(1) = 0.5d0*Dij_1345(3,7)+0.5d0*f1*Eij(3,15)-0.5d0*Dij_2345(3,7)
      R(2) = 0.5d0*Dij_1245(3,9)+0.5d0*f2*Eij(3,15)-0.5d0*Dij_1345(3,7)
     #     -Eij(4,39)
      R(3) = 0.5d0*f3*Eij(3,15)-0.5d0*Dij_1245(3,9)+0.5d0*Dij_1235(3,9)
      R(4) = -0.5d0*Dij_1235(3,9)+0.5d0*f4*Eij(3,15)-2*Eij(4,44)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,32) = res(1)
      Eijp(4,21) = res(2)
      Eijp(4,34) = res(3)
      Eijp(4,15) = res(4)


      R(1) = 0.5d0*Dij_1345(3,10)+0.5d0*f1*Eij(3,20)
     #     -0.5d0*Dij_2345(3,10)
      R(2) = 0.5d0*Dij_1245(3,8)+0.5d0*f2*Eij(3,20)-0.5d0*Dij_1345(3,10)
     #     -Eij(4,45)
      R(3) = 0.5d0*f3*Eij(3,20)-0.5d0*Dij_1245(3,8)+0.5d0*Dij_1235(3,9)
     #     -Eij(4,44)
      R(4) = -0.5d0*Dij_1235(3,9)+0.5d0*f4*Eij(3,20)-Eij(4,43)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,35) = res(1)
      Eijp(4,28) = res(2)
      Eijp(4,31) = res(3)
      Eijpp(4,34) = res(4)





      if (debug) then
         R(1) = 0.5d0*Dij_2345(2,7)+0.5d0*Dij_1345(3,11)
     #        +0.5d0*f1*Eij(3,21)-Eij(4,46)
         R(2) = 0.5d0*Dij_1245(3,11)+0.5d0*f2*Eij(3,21)
     #        -0.5d0*Dij_1345(3,11)
         R(3) = 0.5d0*f3*Eij(3,21)-0.5d0*Dij_1245(3,11)
     #        +0.5d0*Dij_1235(3,11)
         R(4) = -0.5d0*Dij_1235(3,11)+0.5d0*Dij_1234(3,11)
     #        +0.5d0*f4*Eij(3,21)
         call prod_mat_col4(Xm1,R(1),res)
         Eijp(4,36) = res(1)
         Eijp(4,40) = res(2)
         Eijp(4,41) = res(3)
         Eijp(4,42) = res(4)
         
         
         
         R(1) = 0.5d0*Dij_1345(3,13)+0.5d0*f1*Eij(3,24)
     #        -0.5d0*Dij_2345(3,13)
         R(2) = 0.5d0*Dij_1245(3,13)+0.5d0*f2*Eij(3,24)
     #        -0.5d0*Dij_1345(3,13)
         R(3) = 0.5d0*f3*Eij(3,24)-0.5d0*Dij_1245(3,13)
     #        +0.5d0*Dij_1235(3,13)
         R(4) = -0.5d0*Dij_1235(3,13)+0.5d0*f4*Eij(3,24)-Eij(4,46)
         call prod_mat_col4(Xm1,R(1),res)
         Eijpp(4,42) = res(1)
         Eijp(4,44) = res(2)
         Eijp(4,45) = res(3)
         Eijp(4,39) = res(4)
      endif

      
      if (debug) then
         write(*,*) Eij(2,5),  Eijp(2,5)
         write(*,*) Eij(2,6),  Eijp(2,6)
         write(*,*) Eij(2,8),  Eijp(2,8)    
         write(*,*) Eij(2,7),  Eijp(2,7)
         write(*,*) Eij(2,9),  Eijp(2,9)
         write(*,*) Eij(2,10),  Eijp(2,10)
         
         write(*,*) 
         
         write(*,*) Eij(3,13),  Eijp(3,13)
         write(*,*) Eij(3,16),  Eijp(3,16)
         write(*,*) Eij(3,5),  Eijp(3,5) 
         write(*,*) Eij(3,8),  Eijp(3,8) 
         write(*,*) Eij(3,17),  Eijp(3,17)
         write(*,*) Eij(3,9),  Eijp(3,9) 
         write(*,*) Eij(3,12),  Eijp(3,12)
         write(*,*) Eij(3,20),  Eijp(3,20)
         write(*,*) Eij(4,12),  Eijp(4,12)
         write(*,*) Eij(4,16),  Eijp(4,16)
         write(*,*) Eij(4,7),  Eijp(4,7) 
         write(*,*) Eij(4,10),  Eijp(4,10)
         write(*,*) Eij(4,8),  Eijp(4,8) 
         write(*,*) Eij(4,27),  Eijp(4,27)
         write(*,*) Eij(4,29),  Eijp(4,29)
         write(*,*) Eij(4,11),  Eijp(4,11)
         write(*,*) Eij(4,5),  Eijp(4,5) 
         write(*,*) Eij(4,17),  Eijp(4,17)
         write(*,*) Eij(4,24),  Eijp(4,24)
         write(*,*) Eij(4,21),  Eijp(4,21)
         write(*,*) Eij(4,34),  Eijp(4,34),  Eijpp(4,34)
         write(*,*) Eij(4,15),  Eijp(4,15)
         write(*,*) Eij(4,28),  Eijp(4,28)
         write(*,*) Eij(4,31),  Eijp(4,31)
         write(*,*) Eij(4,36),  Eijp(4,36) 
         write(*,*) Eij(4,40),  Eijp(4,40) 
         write(*,*) Eij(4,41),  Eijp(4,41) 
         write(*,*) Eij(4,42),  Eijp(4,42),  Eijpp(4,42)
         write(*,*) Eij(4,44),  Eijp(4,44) 
         write(*,*) Eij(4,45),  Eijp(4,45) 
         write(*,*) Eij(4,39),  Eijp(4,39) 
      endif

      if(lwr) then
         call test_tensor_pent(psq, qsq, lsq, tsq,
     #                     pq, pl, pt, ql, qt, lt, 
     #                     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #                     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #                     E0, 
     #                     Eij)
      endif
c      read(*,'(a)') text
      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                tens_red5 = 5-point tensors
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine tens_red5_orig(psq, qsq, lsq, tsq,
     #                     pq, pl, pt, ql, qt, lt, 
     #                     D0_2345,D0_1345,D0_1245,D0_1235,D0_1234,
     #                     Dij_2345,Dij_1345,Dij_1245,Dij_1235,Dij_1234,
     #                     E0, 
     #                     Eij)
      implicit none
      real * 8 psq, qsq, lsq, tsq,
     #     pq, pl, pt, ql, qt, lt
      double complex D0_2345,D0_1345,D0_1245,D0_1235,D0_1234
      double complex Dij_2345(3,13),Dij_1345(3,13),Dij_1245(3,13),
     #     Dij_1235(3,13),Dij_1234(3,13)
      double complex E0, Eij(4,46), Eijp(4,46), Eijpp(4,46)
c
c  determine the Passarino-Veltman tensor decomposition for the four-point
c  tensor integrals
c
c                                                             d^4k
c   E0; E_mu; E_mu,nu; E_mu,nu,rho; E_mu,nu,rho_sig  =  Int -------
c                                                           (2pi)^4
c
c   1; k_mu; k_mu k_nu; k_mu k_nu k_rho; k_mu k_nu k_rho k_sig
c --------------------------------------------------------------------------
c   k^2  (k+p)^2  (k+p+q)^2  (k+p+q+l)^2  (k+p+q+l+t)^2
c
c  with
c
c   E_mu = p_mu E11  +  q_mu E12  +  l_mu E13  +  t_mu E14
c
c   E_munu = p_mu p_nu E21 + q_mu q_nu E22 + ...
c
c
C INPUT:  psq, pq,...                     kinematics invariants
c         D0_1345 = D00(p+q,l,t)          scalar 4-point functions
c         D0_1245 = D00(p,q+l,t)          in the PV notation
c         D0_1234 = D00(p,q,l)
c         D0_1235 = D00(p,q,l+t)      
c         D0_2345 = D00(q,l,t)

C         Dij_1235(n,m) = D_nm(1,2,3,5) ....    higher D_nm form factors
C                                               as in tens_red3 and tens_red4
c         E0 = E0(p,q,l,t)                      scalar five point function
c
c OUTPUT: Eij(n,m) = E_nm                    form factors in the tensor 
c                                            integrals a la PV
c         nm = 11, 12, 13, 14                ff's for E_mu
c         nm = 21, 22, 23, ..., 211    ff's for E_munu
c         nm = 31, 32, 33, ..., 323, 324     ff's for E_mu,nu,rho
c         nm = 41, 42, 43, ..., 445, 446
c
      double precision f1, f2, f3, f4, Xm1(4,4), X(4,4), deter
      double complex R(1:4),res(1:4)

      logical debug
      parameter (debug=.false.)

      f1 = -psq
      f2 = -qsq-2*pq
      f3 = -lsq-2*ql-2*pl
      f4 = -tsq-2*lt-2*qt-2*pt



      X(1,1) = psq
      X(1,2) = pq
      X(1,3) = pl
      X(1,4) = pt
      X(2,1) = pq
      X(2,2) = qsq
      X(2,3) = ql
      X(2,4) = qt
      X(3,1) = pl
      X(3,2) = ql
      X(3,3) = lsq
      X(3,4) = lt
      X(4,1) = pt
      X(4,2) = qt
      X(4,3) = lt
      X(4,4) = tsq
     

      deter = psq*qsq*lsq*tsq-psq*qsq*lt**2-psq*ql**2*tsq+2*psq*ql*qt*lt
     #-psq*qt**2*lsq-pq**2*lsq*tsq+pq**2*lt**2+2*pq*ql*pl*tsq-2*pq*ql*pt
     #*lt-2*pq*qt*pl*lt+2*pq*qt*pt*lsq-qsq*pl**2*tsq+2*pl*qsq*pt*lt+pl**
     #2*qt**2-2*pl*qt*pt*ql-qsq*pt**2*lsq+pt**2*ql**2

      Xm1(1,1) = (qsq*lsq*tsq-qsq*lt**2-ql**2*tsq+2*ql*qt*lt-qt**2*lsq)/
     #deter
      Xm1(1,2) = -(pq*lsq*tsq-pq*lt**2-ql*pl*tsq+ql*pt*lt+qt*pl*lt-qt*pt
     #*lsq)/deter
      Xm1(1,3) = (pq*ql*tsq-pq*qt*lt-qsq*pl*tsq+qsq*pt*lt+pl*qt**2-qt*pt
     #*ql)/deter
      Xm1(1,4) = (-pq*ql*lt+pq*qt*lsq+qsq*pl*lt-qsq*pt*lsq-ql*pl*qt+pt*q
     #l**2)/deter
      Xm1(2,1) = Xm1(1,2)
      Xm1(2,2) = (psq*lsq*tsq-psq*lt**2-pl**2*tsq+2*pl*pt*lt-pt**2*lsq)/
     #deter
      Xm1(2,3) = (-psq*ql*tsq+psq*qt*lt+pl*pq*tsq-pl*pt*qt-pt*pq*lt+pt**
     #2*ql)/deter
      Xm1(2,4) = (psq*ql*lt-psq*qt*lsq-pl*pq*lt+pl**2*qt+pt*pq*lsq-pt*pl
     #*ql)/deter
      Xm1(3,1) = Xm1(1,3)
      Xm1(3,2) = Xm1(2,3)
      Xm1(3,3) = -(-psq*qsq*tsq+psq*qt**2+pq**2*tsq-2*pq*pt*qt+pt**2*qsq
     #)/deter
      Xm1(3,4) = (-psq*qsq*lt+psq*qt*ql+pq**2*lt-pq*pl*qt-pt*pq*ql+pt*pl
     #*qsq)/deter
      Xm1(4,1) = Xm1(1,4)
      Xm1(4,2) = Xm1(2,4)
      Xm1(4,3) = Xm1(3,4)
      Xm1(4,4) = -(-psq*qsq*lsq+psq*ql**2+pq**2*lsq-2*pq*pl*ql+pl**2*qsq
     #)/deter
      
      R(1) = 0.5d0*(D0_1345-D0_2345+f1*E0)
      R(2) = 0.5d0*(D0_1245-D0_1345+f2*E0)
      R(3) = 0.5d0*(D0_1235-D0_1245+f3*E0)
      R(4) = 0.5d0*(D0_1234-D0_1235+f4*E0)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(1,1) = res(1)
      Eij(1,2) = res(2)
      Eij(1,3) = res(3)
      Eij(1,4) = res(4)
      

      Eij(2,11) = 0d0


      R(1) = 0.5d0*(Dij_1345(1,1)+D0_2345+f1*Eij(1,1)) - Eij(2,11)
      R(2) = 0.5d0*(Dij_1245(1,1)-Dij_1345(1,1)+f2*Eij(1,1))
      R(3) = 0.5d0*(Dij_1235(1,1)-Dij_1245(1,1)+f3*Eij(1,1))
      R(4) = 0.5d0*(Dij_1234(1,1)-Dij_1235(1,1)+f4*Eij(1,1))
      call prod_mat_col4(Xm1,R(1),res)
      Eij(2,1) = res(1)
      Eij(2,5) = res(2)
      Eij(2,6) = res(3)
      Eij(2,7) = res(4)
      


      R(1) = 0.5d0*(Dij_1345(1,1)-Dij_2345(1,1)+f1*Eij(1,2))
      R(2) = 0.5d0*(Dij_1245(1,2)-Dij_1345(1,1)+f2*Eij(1,2))-Eij(2,11)
      R(3) = 0.5d0*(Dij_1235(1,2)-Dij_1245(1,2)+f3*Eij(1,2))
      R(4) = 0.5d0*(Dij_1234(1,2)-Dij_1235(1,2)+f4*Eij(1,2))
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(2,5) = res(1)
      Eij(2,2) = res(2)
      Eij(2,8) = res(3)
      Eij(2,9) = res(4)


      R(1) = 0.5d0*(Dij_1345(1,2)-Dij_2345(1,2)+f1*Eij(1,3))
      R(2) = 0.5d0*(Dij_1245(1,2)-Dij_1345(1,2)+f2*Eij(1,3))
      R(3) = 0.5d0*(Dij_1235(1,3)-Dij_1245(1,2)+f3*Eij(1,3))-Eij(2,11)
      R(4) = 0.5d0*(Dij_1234(1,3)-Dij_1235(1,3)+f4*Eij(1,3))
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(2,6) = res(1)
      Eijp(2,8) = res(2)
      Eij(2,3) = res(3)
      Eij(2,10) = res(4)

      R(1) = 0.5d0*(Dij_1345(1,3)-Dij_2345(1,3)+f1*Eij(1,4))
      R(2) = 0.5d0*(Dij_1245(1,3)-Dij_1345(1,3)+f2*Eij(1,4))
      R(3) = 0.5d0*(Dij_1235(1,3)-Dij_1245(1,3)+f3*Eij(1,4))
      R(4) = 0.5d0*(-Dij_1235(1,3)+f4*Eij(1,4))-Eij(2,11)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(2,7) = res(1)
      Eijp(2,9) = res(2)
      Eijp(2,10) = res(3)
      Eij(2,4) = res(4)


c      Eij(3,21) = 0d0
c      Eij(3,22) = 0d0
c      Eij(3,23) = 0d0
c      Eij(3,24) = 0d0


      Eij(3,21) = -D0_2345/2-Eij(2,1)*f1/2-f4*Eij(2,7)/2
     #     -f2*Eij(2,5)/2-f3*Eij(2,6)/2
      

      Eij(3,22) = Dij_2345(1,1)/2-f3*Eij(2,8)/2-f4*Eij(2,9)/2
     #     -f1*Eij(2,5)/2-f2*Eij(2,2)/2

      Eij(3,23) = Dij_2345(1,2)/2-f2*Eij(2,8)/2-f4*Eij(2,10)/2
     #     -f3*Eij(2,3)/2-f1*Eij(2,6)/2

      Eij(3,24) = Dij_2345(1,3)/2-f3*Eij(2,10)/2-f1*Eij(2,7)/2
     #     -f2*Eij(2,9)/2-f4*Eij(2,4)/2

      

      R(1) = -0.5d0*D0_2345+0.5d0*Eij(2,1)*f1+0.5d0*Dij_1345(2,1) 
     #     - 2*Eij(3,21)
      R(2) = 0.5d0*f2*Eij(2,1)+0.5d0*Dij_1245(2,1)-0.5d0*Dij_1345(2,1)
      R(3) = -0.5d0*Dij_1245(2,1)+0.5d0*f3*Eij(2,1)+0.5d0*Dij_1235(2,1)
      R(4) = 0.5d0*f4*Eij(2,1)-0.5d0*Dij_1235(2,1)+0.5d0*Dij_1234(2,1)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(3,1) = res(1)
      Eij(3,5) = res(2)
      Eij(3,6) = res(3)
      Eij(3,7) = res(4)


      R(1) =  -0.5d0*Dij_2345(2,3)+0.5d0*Eij(2,4)*f1+0.5d0*Dij_1345(2,3)
      R(2) = 0.5d0*f2*Eij(2,4)+0.5d0*Dij_1245(2,3)-0.5d0*Dij_1345(2,3)
      R(3) = 0.5d0*f3*Eij(2,4)-0.5d0*Dij_1245(2,3)+0.5d0*Dij_1235(2,3)
      R(4) = 0.5d0*f4*Eij(2,4)-0.5d0*Dij_1235(2,3)-2*Eij(3,24)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(3,14) = res(1)
      Eij(3,15) = res(2)
      Eij(3,16) = res(3)
      Eij(3,4) = res(4)


      R(1) = -0.5d0*Dij_2345(2,2)+0.5d0*Eij(2,3)*f1+0.5d0*Dij_1345(2,2)
      R(2) = 0.5d0*f2*Eij(2,3)+0.5d0*Dij_1245(2,2)-0.5d0*Dij_1345(2,2)
      R(3) = 0.5d0*f3*Eij(2,3)-0.5d0*Dij_1245(2,2)+0.5d0*Dij_1235(2,3)
     #     -2*Eij(3,23)
      R(4) = 0.5d0*f4*Eij(2,3)-0.5d0*Dij_1235(2,3)+0.5d0*Dij_1234(2,3)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(3,11) = res(1)
      Eij(3,12) = res(2)
      Eij(3,3) = res(3)
      Eij(3,13) = res(4)


      R(1) = -0.5d0*Dij_2345(2,1)+0.5d0*Eij(2,2)*f1+0.5d0*Dij_1345(2,1)
      R(2) = 0.5d0*Dij_1245(2,2)+0.5d0*f2*Eij(2,2)-0.5d0*Dij_1345(2,1)
     #     -2*Eij(3,22)
      R(3) = 0.5d0*f3*Eij(2,2)-0.5d0*Dij_1245(2,2)+0.5d0*Dij_1235(2,2)
      R(4) = 0.5d0*f4*Eij(2,2)-0.5d0*Dij_1235(2,2)+0.5d0*Dij_1234(2,2)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(3,8) = res(1)
      Eij(3,2) = res(2)
      Eij(3,9) = res(3)
      Eij(3,10) = res(4)


      R(1) = -0.5d0*Dij_2345(2,6)+0.5d0*Dij_1345(2,6)+0.5d0*f1*Eij(2,10)
      R(2) = 0.5d0*f2*Eij(2,10)-0.5d0*Dij_1345(2,6)+0.5d0*Dij_1245(2,6)
      R(3) = -0.5d0*Dij_1245(2,6)+0.5d0*f3*Eij(2,10)+0.5d0*Dij_1235(2,3)
     #     -Eij(3,24)
      R(4) = 0.5d0*f4*Eij(2,10)-0.5d0*Dij_1235(2,3)-Eij(3,23)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(3,19) = res(1)
      Eij(3,20) = res(2)
      Eijp(3,13) = res(3)
      Eijp(3,16) = res(4)


      R(1) = 0.5d0*Dij_2345(1,1)+0.5d0*f1*Eij(2,5)+0.5d0*Dij_1345(2,1)
     #     -Eij(3,22)
      R(2) = 0.5d0*f2*Eij(2,5)+0.5d0*Dij_1245(2,4)-0.5d0*Dij_1345(2,1)
     #     -Eij(3,21)
      R(3) = -0.5d0*Dij_1245(2,4)+0.5d0*f3*Eij(2,5)+0.5d0*Dij_1235(2,4)
      R(4) = 0.5d0*Dij_1234(2,4)-0.5d0*Dij_1235(2,4)+0.5d0*f4*Eij(2,5)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(3,5) = res(1)
      Eijp(3,8) = res(2)
      Eij(3,17) = res(3)
      Eij(3,18) = res(4)



      if (debug) then
        R(1) = 0.5d0*f1*Eij(2,8)+0.5d0*Dij_1345(2,4)-0.5d0*Dij_2345(2,4)
        R(2) = 0.5d0*Dij_1245(2,2)+0.5d0*f2*Eij(2,8)-0.5d0*Dij_1345(2,4)
     #        -Eij(3,23)
        R(3) = 0.5d0*f3*Eij(2,8)-0.5d0*Dij_1245(2,2)+0.5d0*Dij_1235(2,6)
     #        -Eij(3,22)
        R(4) = 0.5d0*Dij_1234(2,6)-0.5d0*Dij_1235(2,6)+0.5d0*f4*Eij(2,8)
        call prod_mat_col4(Xm1,R(1),res)
        Eijp(3,17) = res(1)
        Eijp(3,9) = res(2)
        Eijp(3,12) = res(3)
        Eijp(3,20) = res(4)
      endif


      Eij(4,36) = -0.5d0*(-D0_2345+f3*Eij(3,6)+f4*Eij(3,7)
     #     +f2*Eij(3,5)+Eij(3,1)*f1)/2

      Eij(4,37) = -0.5d0*(-Dij_2345(2,1)+f4*Eij(3,10)+f3*Eij(3,9)+
     #     f1*Eij(3,8)+f2*Eij(3,2))/2

      Eij(4,38) = 0.5d0*(Dij_2345(2,2)-f1*Eij(3,11)-f2*Eij(3,12)
     #     -f4*Eij(3,13)-f3*Eij(3,3))/2
               
      Eij(4,39) = -0.5d0*(-Dij_2345(2,3)+f2*Eij(3,15)+f1*Eij(3,14)+
     #     f3*Eij(3,16)+f4*Eij(3,4))/2
      Eij(4,40) = -0.5d0*(Dij_2345(1,1)+f4*Eij(3,18)+f3*Eij(3,17)
     #     +f2*Eij(3,8)+f1*Eij(3,5))/2

      Eij(4,41) =  -0.5d0*(Dij_2345(1,2)+f1*Eij(3,6)+f4*Eij(3,19)+
     #     f3*Eij(3,11)+f2*Eij(3,17))/2
      
      Eij(4,42) = -0.5d0*(Dij_2345(1,3)+f2*Eij(3,18)+f3*Eij(3,19)+
     #     f1*Eij(3,7)+f4*Eij(3,14))/2

      Eij(4,43) = -0.5d0*(-Dij_2345(2,4)+f4*Eij(3,20)+f3*Eij(3,12)+
     #     f1*Eij(3,17)+f2*Eij(3,9))/2
      Eij(4,44) = -0.5d0*(-Dij_2345(2,5)+f3*Eij(3,20)+f1*Eij(3,18)
     #     +f2*Eij(3,10)+f4*Eij(3,15))/2
      
      Eij(4,45) = -0.5d0*(-Dij_2345(2,6)+f2*Eij(3,20)+f1*Eij(3,19)+
     #     f4*Eij(3,16)+f3*Eij(3,13))/2
      
      Eij(4,46) = f3*lsq*Eij(3,3)/24+f4*tsq*Eij(3,4)/24
     #     +(f2*pt+f4*pq+f1*qt)*Eij(3,18)/12
     #     +(f3*pt+f1*lt+f4*pl)*Eij(3,19)/12
     #     +(f2*qt+f4*qsq/2)*Eij(3,10)/12+(f4*qt+f2*tsq/2)*Eij(3,15)/12
     #     +f1*psq*Eij(3,1)/24+f2*qsq*Eij(3,2)/24
     #     +(f1*pq+f2*psq/2)*Eij(3,5)/12
     #     +Dij_2345(2,7)/6+(f2*pl+f3*pq+f1*ql)*Eij(3,17)/12
     #     +(f4*psq/2+f1*pt)*Eij(3,7)/12+(f4*pt+f1*tsq/2)*Eij(3,14)/12
     #     +(f3*qsq/2+f2*ql)*Eij(3,9)/12+(f3*pl+f1*lsq/2)*Eij(3,11)/12
     #     +(f3*ql+f2*lsq/2)*Eij(3,12)/12+(f1*qsq/2+pq*f2)*Eij(3,8)/12
     #     -Dij_2345(2,5)*qt/12-Dij_2345(2,6)*lt/12
     #     +(f4*lt+f3*tsq/2)*Eij(3,16)/12+(f4*lsq/2+f3*lt)*Eij(3,13)/12
     #     +(f3*qt+f4*ql+f2*lt)*Eij(3,20)/12+Dij_2345(1,1)*pq/12
     #     +Dij_2345(1,2)*pl/12+Dij_2345(1,3)*pt/12
     #     -Dij_2345(2,4)*ql/12+(f1*pl+f3*psq/2)*Eij(3,6)/12
     #     -Dij_2345(2,2)*lsq/24-Dij_2345(2,1)*qsq/24
     #     -Dij_2345(2,3)*tsq/24-psq*D0_2345/24


      R(1) = 0.5d0*D0_2345+0.5d0*Eij(3,1)*f1+0.5d0*Dij_1345(3,1)
     #     -3*Eij(4,36)
      R(2) = 0.5d0*Dij_1245(3,1)+0.5d0*f2*Eij(3,1)-0.5d0*Dij_1345(3,1)
      R(3) = 0.5d0*Dij_1235(3,1)-0.5d0*Dij_1245(3,1)+0.5d0*f3*Eij(3,1)
      R(4) = -0.5d0*Dij_1235(3,1)+0.5d0*Dij_1234(3,1)+0.5d0*f4*Eij(3,1)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,1) = res(1)
      Eij(4,5) = res(2)
      Eij(4,6) = res(3)
      Eij(4,7) = res(4)

      R(1) = 0.5d0*Eij(3,2)*f1-0.5d0*Dij_2345(3,1)+0.5d0*Dij_1345(3,1)
      R(2) = 0.5d0*Dij_1245(3,2)+0.5d0*f2*Eij(3,2)-0.5d0*Dij_1345(3,1)
     #     -3*Eij(4,37)
      R(3) = -0.5d0*Dij_1245(3,2)+0.5d0*f3*Eij(3,2)+0.5d0*Dij_1235(3,2)
      R(4) = 0.5d0*Dij_1234(3,2)+0.5d0*f4*Eij(3,2)-0.5d0*Dij_1235(3,2)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,8) = res(1)
      Eij(4,2) = res(2)
      Eij(4,9) = res(3)
      Eij(4,10) = res(4)


      R(1) = 0.5d0*Eij(3,3)*f1-0.5d0*Dij_2345(3,2)+0.5d0*Dij_1345(3,2)
      R(2) = 0.5d0*Dij_1245(3,2)+0.5d0*f2*Eij(3,3)-0.5d0*Dij_1345(3,2)
      R(3) = -0.5d0*Dij_1245(3,2)+0.5d0*f3*Eij(3,3)+0.5d0*Dij_1235(3,3)
     #     -3*Eij(4,38)
      R(4) = 0.5d0*Dij_1234(3,3)+0.5d0*f4*Eij(3,3)-0.5d0*Dij_1235(3,3)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,11) = res(1)
      Eij(4,12) = res(2)
      Eij(4,3) = res(3)
      Eij(4,13) = res(4)


      R(1) = 0.5d0*Eij(3,4)*f1-0.5d0*Dij_2345(3,3)+0.5d0*Dij_1345(3,3)
      R(2) = 0.5d0*Dij_1245(3,3)-0.5d0*Dij_1345(3,3)+0.5d0*f2*Eij(3,4)
      R(3) = -0.5d0*Dij_1245(3,3)+0.5d0*f3*Eij(3,4)+0.5d0*Dij_1235(3,3)
      R(4) = 0.5d0*f4*Eij(3,4)-0.5d0*Dij_1235(3,3)-3*Eij(4,39)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,14) = res(1)
      Eij(4,15) = res(2)
      Eij(4,16) = res(3)
      Eij(4,4) = res(4)


      R(1) = 0.5d0*Dij_1345(3,6)+0.5d0*f1*Eij(3,12)-0.5d0*Dij_2345(3,6)
      R(2) = 0.5d0*Dij_1245(3,2)+0.5d0*f2*Eij(3,12)-0.5d0*Dij_1345(3,6)
     #     -Eij(4,38)
      R(3) = -0.5d0*Dij_1245(3,2)+0.5d0*f3*Eij(3,12)+0.5d0*Dij_1235(3,9)
     #     -2*Eij(4,43)
      R(4) = -0.5d0*Dij_1235(3,9)+0.5d0*f4*Eij(3,12)+0.5d0*Dij_1234(3,9)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,29) = res(1)
      Eij(4,20) = res(2)
      Eijp(4,12) = res(3)
      Eij(4,31) = res(4)


      R(1) = 0.5d0*Dij_1345(3,9)+0.5d0*f1*Eij(3,16)-0.5d0*Dij_2345(3,9)
      R(2) = 0.5d0*Dij_1245(3,9)+0.5d0*f2*Eij(3,16)-0.5d0*Dij_1345(3,9)
      R(3) = 0.5d0*f3*Eij(3,16)-0.5d0*Dij_1245(3,9)+0.5d0*Dij_1235(3,3)
     #     -Eij(4,39)
      R(4) = 0.5d0*f4*Eij(3,16)-0.5d0*Dij_1235(3,3)-2*Eij(4,45)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,33) = res(1)
      Eij(4,34) = res(2)
      Eij(4,22) = res(3)
      Eijp(4,16) = res(4)


      R(1) = 0.5d0*Dij_1345(3,5)+0.5d0*f1*Eij(3,7)-0.5d0*Dij_2345(1,3)
     #     -2*Eij(4,42)
      R(2) = 0.5d0*Dij_1245(3,5)-0.5d0*Dij_1345(3,5)+0.5d0*f2*Eij(3,7)
      R(3) = 0.5d0*f3*Eij(3,7)-0.5d0*Dij_1245(3,5)+0.5d0*Dij_1235(3,5)
      R(4) = -0.5d0*Dij_1235(3,5)+0.5d0*f4*Eij(3,7)-Eij(4,36)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(4,7) = res(1)
      Eij(4,24) = res(2)
      Eij(4,25) = res(3)
      Eij(4,19) = res(4)


      R(1) = 0.5d0*Dij_1345(3,5)+0.5d0*f1*Eij(3,10)-0.5d0*Dij_2345(3,5)
      R(2) = 0.5d0*Dij_1245(3,8)-0.5d0*Dij_1345(3,5)+0.5d0*f2*Eij(3,10)
     #     -2*Eij(4,44)
      R(3) = 0.5d0*f3*Eij(3,10)-0.5d0*Dij_1245(3,8)+0.5d0*Dij_1235(3,8)
      R(4) = -0.5d0*Dij_1235(3,8)+0.5d0*f4*Eij(3,10)-Eij(4,37)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,27) = res(1)
      Eijp(4,10) = res(2)
      Eij(4,28) = res(3)
      Eij(4,21) = res(4)


      R(1) = 0.5d0*Dij_2345(2,1)+0.5d0*f1*Eij(3,8)+0.5d0*Dij_1345(3,1)
     #     -Eij(4,37)
      R(2) = 0.5d0*Dij_1245(3,6)+0.5d0*f2*Eij(3,8)-0.5d0*Dij_1345(3,1)
     #     -2*Eij(4,40)
      R(3) = 0.5d0*f3*Eij(3,8)+0.5d0*Dij_1235(3,6)-0.5d0*Dij_1245(3,6)
      R(4) = -0.5d0*Dij_1235(3,6)+0.5d0*f4*Eij(3,8)+0.5d0*Dij_1234(3,6)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,17) = res(1)
      Eijp(4,8) = res(2)
      Eij(4,26) = res(3)
      Eijp(4,27) = res(4)


      R(1) = 0.5d0*Dij_2345(2,2)+0.5d0*Dij_1345(3,6)+0.5d0*f1*Eij(3,11)
     #     -Eij(4,38)
      R(2) = 0.5d0*Dij_1245(3,6)+0.5d0*f2*Eij(3,11)-0.5d0*Dij_1345(3,6)
      R(3) = 0.5d0*f3*Eij(3,11)+0.5d0*Dij_1235(3,7)-0.5d0*Dij_1245(3,6)
     #     -2*Eij(4,41)
      R(4) = -0.5d0*Dij_1235(3,7)+0.5d0*Dij_1234(3,7)+0.5d0*f4*Eij(3,11)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,18) = res(1)
      Eijp(4,29) = res(2)
      Eijp(4,11) = res(3)
      Eij(4,30) = res(4)


      R(1) = 0.5d0*f1*Eij(3,5)+0.5d0*Dij_1345(3,1)-0.5d0*Dij_2345(1,1)
     #     -2*Eij(4,40)
      R(2) = 0.5d0*Dij_1245(3,4)+0.5d0*f2*Eij(3,5)-0.5d0*Dij_1345(3,1)
     #     -Eij(4,36)
      R(3) = 0.5d0*f3*Eij(3,5)-0.5d0*Dij_1245(3,4)+0.5d0*Dij_1235(3,4)
      R(4) = -0.5d0*Dij_1235(3,4)+0.5d0*Dij_1234(3,4)+0.5d0*f4*Eij(3,5)
      call prod_mat_col4(Xm1,R(1),res)
      Eijp(4,5) = res(1)
      Eijp(4,17) = res(2)
      Eij(4,23) = res(3)
      Eijp(4,24) = res(4)


      R(1) = 0.5d0*Dij_1345(3,7)+0.5d0*f1*Eij(3,15)-0.5d0*Dij_2345(3,7)
      R(2) = 0.5d0*Dij_1245(3,9)+0.5d0*f2*Eij(3,15)-0.5d0*Dij_1345(3,7)
     #     -Eij(4,39)
      R(3) = 0.5d0*f3*Eij(3,15)-0.5d0*Dij_1245(3,9)+0.5d0*Dij_1235(3,9)
      R(4) = -0.5d0*Dij_1235(3,9)+0.5d0*f4*Eij(3,15)-2*Eij(4,44)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,32) = res(1)
      Eijp(4,21) = res(2)
      Eijp(4,34) = res(3)
      Eijp(4,15) = res(4)


      R(1) = 0.5d0*Dij_1345(3,10)+0.5d0*f1*Eij(3,20)
     #     -0.5d0*Dij_2345(3,10)
      R(2) = 0.5d0*Dij_1245(3,8)+0.5d0*f2*Eij(3,20)-0.5d0*Dij_1345(3,10)
     #     -Eij(4,45)
      R(3) = 0.5d0*f3*Eij(3,20)-0.5d0*Dij_1245(3,8)+0.5d0*Dij_1235(3,9)
     #     -Eij(4,44)
      R(4) = -0.5d0*Dij_1235(3,9)+0.5d0*f4*Eij(3,20)-Eij(4,43)
      call prod_mat_col4(Xm1,R(1),res)
      Eij(4,35) = res(1)
      Eijp(4,28) = res(2)
      Eijp(4,31) = res(3)
      Eijpp(4,34) = res(4)





      if (debug) then
         R(1) = 0.5d0*Dij_2345(2,7)+0.5d0*Dij_1345(3,11)
     #        +0.5d0*f1*Eij(3,21)-Eij(4,46)
         R(2) = 0.5d0*Dij_1245(3,11)+0.5d0*f2*Eij(3,21)
     #        -0.5d0*Dij_1345(3,11)
         R(3) = 0.5d0*f3*Eij(3,21)-0.5d0*Dij_1245(3,11)
     #        +0.5d0*Dij_1235(3,11)
         R(4) = -0.5d0*Dij_1235(3,11)+0.5d0*Dij_1234(3,11)
     #        +0.5d0*f4*Eij(3,21)
         call prod_mat_col4(Xm1,R(1),res)
         Eijp(4,36) = res(1)
         Eijp(4,40) = res(2)
         Eijp(4,41) = res(3)
         Eijp(4,42) = res(4)
         
         
         
         R(1) = 0.5d0*Dij_1345(3,13)+0.5d0*f1*Eij(3,24)
     #        -0.5d0*Dij_2345(3,13)
         R(2) = 0.5d0*Dij_1245(3,13)+0.5d0*f2*Eij(3,24)
     #        -0.5d0*Dij_1345(3,13)
         R(3) = 0.5d0*f3*Eij(3,24)-0.5d0*Dij_1245(3,13)
     #        +0.5d0*Dij_1235(3,13)
         R(4) = -0.5d0*Dij_1235(3,13)+0.5d0*f4*Eij(3,24)-Eij(4,46)
         call prod_mat_col4(Xm1,R(1),res)
         Eijpp(4,42) = res(1)
         Eijp(4,44) = res(2)
         Eijp(4,45) = res(3)
         Eijp(4,39) = res(4)
      endif

      
      if (debug) then
         write(*,*) Eij(2,5),  Eijp(2,5)
         write(*,*) Eij(2,6),  Eijp(2,6)
         write(*,*) Eij(2,8),  Eijp(2,8)    
         write(*,*) Eij(2,7),  Eijp(2,7)
         write(*,*) Eij(2,9),  Eijp(2,9)
         write(*,*) Eij(2,10),  Eijp(2,10)
         
         write(*,*) 
         
         write(*,*) Eij(3,13),  Eijp(3,13)
         write(*,*) Eij(3,16),  Eijp(3,16)
         write(*,*) Eij(3,5),  Eijp(3,5) 
         write(*,*) Eij(3,8),  Eijp(3,8) 
         write(*,*) Eij(3,17),  Eijp(3,17)
         write(*,*) Eij(3,9),  Eijp(3,9) 
         write(*,*) Eij(3,12),  Eijp(3,12)
         write(*,*) Eij(3,20),  Eijp(3,20)
         write(*,*) Eij(4,12),  Eijp(4,12)
         write(*,*) Eij(4,16),  Eijp(4,16)
         write(*,*) Eij(4,7),  Eijp(4,7) 
         write(*,*) Eij(4,10),  Eijp(4,10)
         write(*,*) Eij(4,8),  Eijp(4,8) 
         write(*,*) Eij(4,27),  Eijp(4,27)
         write(*,*) Eij(4,29),  Eijp(4,29)
         write(*,*) Eij(4,11),  Eijp(4,11)
         write(*,*) Eij(4,5),  Eijp(4,5) 
         write(*,*) Eij(4,17),  Eijp(4,17)
         write(*,*) Eij(4,24),  Eijp(4,24)
         write(*,*) Eij(4,21),  Eijp(4,21)
         write(*,*) Eij(4,34),  Eijp(4,34),  Eijpp(4,34)
         write(*,*) Eij(4,15),  Eijp(4,15)
         write(*,*) Eij(4,28),  Eijp(4,28)
         write(*,*) Eij(4,31),  Eijp(4,31)
         write(*,*) Eij(4,36),  Eijp(4,36) 
         write(*,*) Eij(4,40),  Eijp(4,40) 
         write(*,*) Eij(4,41),  Eijp(4,41) 
         write(*,*) Eij(4,42),  Eijp(4,42),  Eijpp(4,42)
         write(*,*) Eij(4,44),  Eijp(4,44) 
         write(*,*) Eij(4,45),  Eijp(4,45) 
         write(*,*) Eij(4,39),  Eijp(4,39) 
      endif

      end
